import { Injectable } from '@angular/core';
import { Location } from '@angular/common';
import { HttpClient } from '@angular/common/http';
import { CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, CanLoad, Router, Route, UrlSegment } from '@angular/router';
import { Observable, of as observableOf, throwError } from 'rxjs';
import { switchMap, take, catchError, tap } from 'rxjs/operators';

import { SecurityService } from '../services/security.service';
import { ConfigService } from '../services/config.service';
import { FuncionalidadeModel } from '../models/funcionalidade.model';
import { ErrorModel } from '../models/error.model';
import { LocalStorage } from '../utils/local.storage';
import {
  DATASUS_ROUTE_HOME,
  DATASUS_ROUTE_ERROR_BACKEND,
  DATASUS_ROUTE_FORBIDDEN,
  DATASUS_ROUTE_CHOOSE_PROFILE,
} from '../constants/routes.contants';

@Injectable({
  providedIn: 'root'
})
export class PathGuard implements CanActivate, CanLoad {

  private defaultPathExceptHome = [
    DATASUS_ROUTE_ERROR_BACKEND,
    DATASUS_ROUTE_FORBIDDEN,
    DATASUS_ROUTE_CHOOSE_PROFILE,
  ];

  constructor(
    private httpClient: HttpClient,
    private router: Router,
    private location: Location,
    private configService: ConfigService,
    private securityService: SecurityService,
  ) { }

  canActivate(
    next: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<boolean> | Promise<boolean> | boolean {
    try {
      console.log('[PathGuard::canActivate]');
      const path = state.url;
      return this.checkFunctionalities(
        this.location.normalize(path)
      );
    } catch (error) {
      console.warn('[PathGuard::canActivate] catch error', error);
      return false;
    }
  }

  canLoad(
    route: Route,
    segments: UrlSegment[],
  ): Observable<boolean> | Promise<boolean> | boolean {
    try {
      console.log('[PathGuard::canLoad]');
      const path = segments.map((segment: UrlSegment) => segment.path).join('/');
      return this.checkFunctionalities(
        this.location.normalize(`/${path}`)
      );
    } catch (error) {
      console.warn('[PathGuard::canLoad] catch error', error);
      return false;
    }
  }

  private checkFunctionalities(path: string): Observable<boolean> | boolean {
    console.log('[PathGuard::checkFunctionalities]', path);
    if (path === DATASUS_ROUTE_HOME) {
      console.log('[PathGuard::checkFunctionalities] é a home');
      return true;
    }
    const filteredPaths = this.defaultPathExceptHome.filter(
      (defaultPath: string) => path.includes(this.location.normalize(defaultPath))
    ) || [];
    if (!!filteredPaths.length) {
      console.log('[PathGuard::checkFunctionalities] é uma das rotas padrões', filteredPaths);
      return true;
    }
    if (!this.securityService.hasPerfilSelecionado()) {
      console.log('[PathGuard::checkFunctionalities] Não tem perfil selecionado');
      return false;
    }
    const perfil = this.securityService.getStoredPerfilSelecionado();
    // const cacheKey = `×DATASUS·Funcionalidades:${perfil.sigla} @ ${this.configService.appKey}`;
    // if (LocalStorage.hasItem(cacheKey)) {
    //   return this.checkList(LocalStorage.getItem(cacheKey), path);
    // }
    return this.configService.getList([
      'backendUrl',
      'clientId',
    ]).pipe(
      switchMap((configs: string[]) => {
        const backendUrl = configs[0];
        const clientId = configs[1];
        console.log('[PathGuard::checkFunctionalities] Vamos recuperar as funcionalidades', backendUrl, clientId, perfil.sigla);
        const request$ = this.httpClient.get<FuncionalidadeModel[]>(
          `${backendUrl}/acesso/sistema/${clientId}/perfil/${perfil.sigla}/funcionalidade`
        ).pipe(
          take(1),
          catchError(
            (error: ErrorModel) => {
              console.warn('[PathGuard::checkFunctionalities] Erro', error);
              this.router.navigate([`/${DATASUS_ROUTE_FORBIDDEN}`]);
              return throwError(observableOf(false));
            }
          ),
          // tap(
          //   (funcionalidades: FuncionalidadeModel[]) => LocalStorage.setItem(cacheKey, funcionalidades)
          // ),
          switchMap(
            (funcionalidades: FuncionalidadeModel[]) => this.checkList(funcionalidades, path)
          ),
        );
        return request$;
      })
    );
  }

  private checkPath(funcionalidade: FuncionalidadeModel, path: string): boolean {
    if (!funcionalidade) {
      return false;
    }
    if (funcionalidade.bloqueado) {
      return false;
    }
    const pathFuncionalidade = this.location.normalize(funcionalidade.path);
    // Funcionalidade com parãmetro?
    // (Ex.: "rota/com/param/:id", "caminho/da/rota/:x/com/outro/param/:y" ou "caminho/:x/:y/feio")
    if (pathFuncionalidade.includes('/:')) {
      const regex = /\:[^/\W]*\/?/g;
      const terms = pathFuncionalidade.split(regex) || [];
      if (!terms.length) {
        return false;
      }
      return terms
        .filter((term: string) => !!term)
        .every((term: string) => path.includes(term));
    }
    // Rota cadastrada é exatamente a mesma cadastrada para acessar?
    return pathFuncionalidade === path;
  }

  private checkList(funcionalidades: FuncionalidadeModel[], path: string): Observable<boolean> {
    // console.log('[PathGuard::checkFunctionalities] Funcionalidades', funcionalidades);
    if (!funcionalidades || !funcionalidades.length) {
      console.warn('[PathGuard::checkFunctionalities] Sem funcionalidades');
      this.router.navigate([`/${DATASUS_ROUTE_FORBIDDEN}`]);
      return observableOf(false);
    }
    const filtered: FuncionalidadeModel[] = funcionalidades.filter(
      (funcionalidade: FuncionalidadeModel) => this.checkPath(funcionalidade, path)
    ) || [];
    if (filtered.length <= 0) {
      return throwError('Você não tem permissão para acessar.');
    }
    console.log('[PathGuard::checkFunctionalities] Achou a funcionalidade');
    return observableOf(true);
  }
}
