import { Injectable } from '@angular/core';
import {HttpClient, HttpParams} from '@angular/common/http';
import { Observable, of as observableOf } from 'rxjs';
import { switchMap, take, tap } from 'rxjs/operators';

import { ConfigService } from './config.service';
import { SessionStorage } from '../utils/session.storage';

@Injectable({
  providedIn: 'root'
})
export abstract class BaseService {

  constructor(
    private httpClient: HttpClient,
    private configService: ConfigService,
  ) { }

  protected baseDelete<T>(endpoint: string): Observable<T> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.delete<T>(
          `${backendUrl}/${this.sanitizeEndpoint(endpoint)}`,
        ).pipe(take(1))
      )
    );
  }

  protected baseGet<T>(endpoint: string): Observable<T> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.get<T>(
          `${backendUrl}/${this.sanitizeEndpoint(endpoint)}`,
        ).pipe(take(1))
      )
    );
  }

  protected baseGetWithParams<T>(endpoint: string, params: HttpParams | { [param: string]: string | string[]; }): Observable<T> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.get<T>(
          `${backendUrl}/${this.sanitizeEndpoint(endpoint)}`, {params: params}
        ).pipe(take(1))
      )
    );
  }

  protected baseGetCached<T>(endpoint: string): Observable<T> {
    const cacheKey = `÷${endpoint} @ ${this.configService.appKey}`;

    if (SessionStorage.hasItem(cacheKey)) {
      return observableOf(SessionStorage.getItem(cacheKey) as T);
    }

    return this.baseGet<T>(endpoint).pipe(
      tap((data: T) => SessionStorage.setItem(cacheKey, data))
    );
  }

  protected baseGetText(endpoint: string): Observable<string> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.get(
          `${backendUrl}/${this.sanitizeEndpoint(endpoint)}`, {
            responseType: 'text'
          }
        ).pipe(take(1))
      )
    );
  }

  protected baseHead<T>(endpoint: string): Observable<T> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.head<T>(
          `${backendUrl}/${this.sanitizeEndpoint(endpoint)}`,
        ).pipe(take(1))
      )
    );
  }

  protected baseJsonp<T>(endpoint: string, callback: string): Observable<T> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.jsonp<T>(
          `${backendUrl}/${this.sanitizeEndpoint(endpoint)}`,
          callback,
        ).pipe(take(1))
      )
    );
  }

  protected baseOptions<T>(endpoint: string): Observable<T> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.options<T>(
          `${backendUrl}/${this.sanitizeEndpoint(endpoint)}`,
        ).pipe(take(1))
      )
    );
  }

  protected basePatch<T>(endpoint: string, data: any): Observable<T> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.patch<T>(
          `${backendUrl}/${this.sanitizeEndpoint(endpoint)}`,
          data,
        ).pipe(take(1))
      )
    );
  }

  protected basePost<T>(endpoint: string, data: any): Observable<T> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.post<T>(
          `${backendUrl}/${this.sanitizeEndpoint(endpoint)}`,
          data
        ).pipe(take(1))
      )
    );
  }

  protected basePostFile(files: File[], fileNames: string[]): Observable<string> {
    const formData = new FormData();
    files.forEach((file: File, index: number) => formData.set(fileNames[index], file, file.name));
    return this.configService.getList(['backendUrl', 'clientId']).pipe(
      switchMap(
        (configs: string[]) => this.httpClient.post(
          `${configs[0]}/arquivo/sistema/${configs[1]}/upload`,
          formData,
          { responseType: 'text' },
        ).pipe(take(1))
      )
    );
  }

  protected basePublicPostFile(files: File[], fileNames: string[]): Observable<string> {
    console.log(
      '%c[!] BaseService.basePublicPostFile(): Vc está usando um método temporário para atender um singelo felino',
      'color: #ff6; background-color: #424242; font-size: 1.2em'
    );
    const formData = new FormData();
    files.forEach((file: File, index: number) => formData.set(fileNames[index], file, file.name));
    return this.configService.getList(['backendUrl', 'clientId']).pipe(
      switchMap(
        (configs: string[]) => this.httpClient.post(
          `${configs[0]}/public/arquivo/sistema/${configs[1]}/upload`,
          formData,
          { responseType: 'text' },
        ).pipe(take(1))
      )
    );
  }

  protected basePut<T>(endpoint: string, data: any): Observable<T> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.put<T>(
          `${backendUrl}/${this.sanitizeEndpoint(endpoint)}`,
          data,
        ).pipe(take(1))
      )
    );
  }

  private sanitizeEndpoint(endpoint: string): string {
    return endpoint.trim().replace(/^\//, '');
  }

}
