
/**
 * Atalho para `encodeURIComponent()`
 * @param uri Algo que precisa de ser codificado
 */
export function urlEncode(uri: string): string {
  return encodeURIComponent(uri);
}

/**
 * Atalho para `decodeURIComponent()`
 * @param uri Algo que precisa de ser decodificado
 */
export function urlDecode(uri: string): string {
  return decodeURIComponent(uri);
}

/**
 * Atalho para codificar em base64 usando UTF-8
 * @param str Texto a ser codificado
 */
export function base64Encode(str: string): string {
  return btoa(
    urlEncode(str)
      .replace(
        /%([0-9A-F]{2})/g,
        (_: string, char: string) => String.fromCharCode(parseInt('0x' + char, 16)),
      )
  );
}

/**
 * Atalho para decodificar um texto em UTF-8 na codificação base64
 * @param base64 Texto a ser decodificado
 */
export function base64Decode(base64: string): string {
  return urlDecode(
    atob(base64)
      .split('')
      .map(char => '%' + ('00' + char.charCodeAt(0).toString(16)).slice(-2))
      .join('')
  );
}

/**
 * Atalho para decodificar um texto em UTF-8 na codificação base64
 * Retornando um TypedArray: Uint8Array
 * @param base64 Texto a ser decodificado
 */
export function base64DecodeToTypedArray(base64: string): Uint8Array {
  const binaryString = base64Decode(base64);
  const binaryLen = binaryString.length;
  const bytes = new Uint8Array(binaryLen);
  for (let i = 0; i < binaryLen; i++) {
    bytes[i] = binaryString.charCodeAt(i);
  }
  return bytes;
}

/**
 * Atalho para decodificar um texto em UTF-8 na codificação base64
 * Retornando um Blob
 * @param base64 Texto a ser decodificado
 * @param contentType Mime type do Blob
 */
export function base64DecodeToBlob(base64: string, contentType: string): Blob {
  contentType = contentType || 'application/octet-stream';
  return new Blob([ base64DecodeToTypedArray(base64) ], { type: contentType });
}

/**
 * Atalho para para converter uma string em objeto Blob
 * Retornando um Blob
 * @param str Texto a ser decodificado
 * @param contentType Mime type do Blob
 */
export function stringToBlob(str: string, contentType: string): Blob {
  contentType = contentType || 'application/octet-stream';
  return new Blob([stringToTypedArray(str)], { type: contentType });
}

/**
 * Atalho para decodificar um texto em TypedArray
 * Retornando um TypedArray: Uint8Array
 * @param str Texto a ser decodificado
 */
export function stringToTypedArray(str: string): Uint8Array {
  const binaryLen = str.length;
  const bytes = new Uint8Array(binaryLen);
  for (let i = 0; i < binaryLen; i++) {
    bytes[i] = str.charCodeAt(i);
  }
  return bytes;
}
