import { Injectable } from '@angular/core';
import { Observable, from, forkJoin } from 'rxjs';
import { switchMap, take, catchError } from 'rxjs/operators';

import { ConfigService, urlEncode, AuthService } from '@datasus/core';
import { IsTokenPresentResult } from '../models/is-token-present-result.model';
import {
    GetTokenInfoResult,
    AliveResult,
    ConfigResult, SignAndSendExtraInfo, SignAndSendResult, SignAndSendParameters } from '../../public-api';


@Injectable({
  providedIn: 'root'
})
export class EvoSdkService {

  constructor(
    private configService: ConfigService,
    private authService: AuthService,
  ) { }

  isTokenPresent(): Observable<IsTokenPresentResult> {
    return this.request<IsTokenPresentResult>('tokenServices', 'isTokenPresent');
  }

  getTokenInfo(): Observable<GetTokenInfoResult> {
    return this.request<GetTokenInfoResult>('tokenServices', 'getTokenInfo');
  }

  alive(): Observable<AliveResult> {
    return this.request<AliveResult>('alive');
  }

  config(): Observable<ConfigResult> {
    return this.request<ConfigResult>('config');
  }

  signAndSend(idArquivo: string, extraInfo?: SignAndSendExtraInfo): Observable<SignAndSendResult> {

    const transform = (obj: any): string => {
      return JSON.stringify(obj)
        .replace(/[{}]/g, '')
        .replace(/"/g, '_')
        .replace(/_:_/g, '__')
        .replace(/_,_/g, '___');
    };

    const extraInfoParam = !!extraInfo
      ? `?extra-info=${transform(extraInfo)}`
      : '';

    return forkJoin(
      this.configService.get('backendUrl'),
      this.authService.cookie(),
    ).pipe(
      switchMap(
        (forkJoinResults: string[]) => {
          const backendUrl = forkJoinResults[0];
          const backendCookie = forkJoinResults[1];
          const sistema = idArquivo.replace(idArquivo.substr(-46), '').toUpperCase();
          const downloadUrl = urlEncode(`${backendUrl}/arquivo/sistema/${sistema}/id/${idArquivo}/download`);
          const uploadUrl = urlEncode(`${backendUrl}/assinatura-digital/${idArquivo}${extraInfoParam}`);
          const parameters: SignAndSendParameters = {
            'checked.0': true,
            'colAlias.0': '$arquivo',
            'colName.0': 'arquivo',
            'url.0': downloadUrl,
            'arquivo.0': 'arquivo.assinado',
            colCount: 1,
            'config.type': 'local',
            digestAlgorithm: 'SHA256',
            generateLog: true,
            sendMode: 'Multipart',
            showCertDialog: false,
            signingAlgorithm: 'SHA256WithRSA',
            serverURL: uploadUrl,
            cookie: backendCookie,
          };
          return this.request<SignAndSendResult>('sign', 'signAndSend', parameters);
        }
      )
    );
  }

  private request<T>(handler: string, service?: string, parameters?: any): Observable<T> {
    const formData: string[] = [];
    Object.keys(parameters || {}).forEach(
      (key: string) => formData.push(`${key}=${parameters[key]}`)
    );
    const createRequest: (host: string) => Observable<T> = (host: string) => {
      const url = `${host}/${handler}/` + (service ? `?service=${service}` : '');
      return from(
        fetch(url, {
          method: 'post',
          headers: {
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
          },
          body: formData.join('&'),
        }).then(response => response.json())
      );
    };

    return createRequest('https://localhost:8686').pipe(
      catchError(() => createRequest('http://localhost:8585')),
      take(1),
    );
  }
}
