import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  DoCheck,
  ElementRef,
  EventEmitter,
  forwardRef,
  Input,
  OnChanges,
  OnDestroy,
  OnInit,
  Optional,
  Output,
  Self,
  ViewChild,
} from '@angular/core';
import { FormControlName, ControlValueAccessor, NgControl } from '@angular/forms';
import { MatSelectChange, MatSelect } from '@angular/material/select';
import { CdkVirtualScrollViewport } from '@angular/cdk/scrolling';
import { ErrorStateMatcher } from '@angular/material';
import { MatFormFieldControl } from '@angular/material/form-field';
import { Subject, Subscription } from 'rxjs';
import { debounceTime, map } from 'rxjs/operators';

import { FilterPipe } from '@datasus/commons';
import { ErrorModel, compareObjects } from '@datasus/core';

@Component({
  selector: 'datasus-select-autocomplete',
  templateUrl: './select-autocomplete.component.html',
  styleUrls: ['./select-autocomplete.component.scss'],
  providers: [{
    provide: MatFormFieldControl,
    useExisting: forwardRef(() => SelectAutocompleteComponent),
    multi: true
  },
],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SelectAutocompleteComponent implements OnInit, OnDestroy, OnChanges, DoCheck, ControlValueAccessor {

  selectedOptionDisplay: string;
  selectedOptionTooltip: string;
  filteredOptions: any[] = [];
  filteredOptionsChange: Subject<any> = new Subject<any>();
  displayString: string;
  selectedValue: any = null;

  @Input() placeholder: string;
  @Input() floatLabel = true;
  @Input() options: any[] = [];
  @Input() disabled = false;
  @Input() required: boolean;
  @Input() display = 'display';
  @Input() optionField: (string | number);
  @Input() inputSearchPlaceholder = 'Pesquisar';
  @Input() filterSearch: string;
  @Input() appearance: ('legacy' | 'standard' | 'fill' | 'outline') = 'standard';
  @Input() tooltip: string;
  @Input() customErroMessage: string;
  @Input() loading = false;
  @Input() loadingDiameter = '18';
  @Input()
  get errorStateMatcher(): ErrorStateMatcher {
    return this.selectElem.errorStateMatcher;
  }
  set errorStateMatcher(val) {
    this.selectElem.errorStateMatcher = val;
  }

  @Output() selectionChange: EventEmitter<any> = new EventEmitter();
  @Output() openedChangeSelect: EventEmitter<boolean> = new EventEmitter();

  @ViewChild('selectElem') selectElem: MatSelect;
  @ViewChild(CdkVirtualScrollViewport) cdkVirtualScrollViewport: CdkVirtualScrollViewport;
  @ViewChild('searchInput') searchInput: ElementRef<HTMLInputElement>;

  // @Input() compareWith = (o1: any, o2: any) => compareObjects(o1, o2);

  private filteredOptionsChangeSubscription: Subscription;

  constructor(
    @Optional() @Self() ngControl: NgControl,
    @Optional() private controlName: FormControlName,
    private changeDetectorRef: ChangeDetectorRef,
  ) {
    if (ngControl) {
      ngControl.valueAccessor = this;
    }
  }

  ngOnInit(): void {
    this.options = this.options || [];
    this.filteredOptions = [...this.options];
    const filterPipe = new FilterPipe();
    this.filteredOptionsChangeSubscription = this.filteredOptionsChange.pipe(
      debounceTime(400),
      map((value: string) => filterPipe.transform(this.options, value, this.filterSearch)),
    ).subscribe(
      (filtro: any[]) => {
        this.filteredOptions = [...filtro];
        this.rerender();
      },
      (error: ErrorModel) => console.error('Erro SelectAutocompleteComponent', error.message)
    );
  }

  ngOnDestroy(): void {
    this.filteredOptionsChangeSubscription.unsubscribe();
  }

  ngOnChanges(): void {
    this.options = this.options || [];
    this.filteredOptions = [...this.options];
    this.setValue();
  }

  ngDoCheck(): void {
    this.selectElem.updateErrorState();
  }

  onChange: any = () => { };
  onTouched: any = () => { };

  writeValue(obj: any): void {
    this.selectedValue = obj;
    this.setValue();
  }

  registerOnChange(fn: any): void {
    this.selectElem.registerOnChange(fn);
  }

  registerOnTouched(fn: any): void {
    this.selectElem.registerOnTouched(fn);
  }

  setDisabledState(isDisabled: boolean): void {
    this.selectElem.setDisabledState(isDisabled);
  }

  filterItem(value: any) {
    this.filteredOptionsChange.next(value);
  }

  stopEventAndAppendSpace(event: Event): void {
    event.stopPropagation();
    event.preventDefault();
    this.searchInput.nativeElement.value += ' ';
  }

  clearFilterItem() {
    this.searchInput.nativeElement.value = '';
    this.filterItem(this.searchInput.nativeElement.value);
  }

  onSelectionChange(event: MatSelectChange) {
    this.selectedValue = event.value;
    this.selectedOptionDisplay = event.source.triggerValue;
    this.setValue();
    this.onChange(this.selectedValue);
    this.onTouched();
    this.selectionChange.emit(this.selectedValue);
    this.rerender();
  }

  openedChange(isOpen: boolean) {
    if (isOpen) {
      setTimeout(() => this.scrollToMiddle(), 0);
      setTimeout(() => this.scrollToBottom(), 1);
      setTimeout(() => this.scrollToMiddle(), 2);
      setTimeout(() => this.scrollToTop(), 3);
      setTimeout(() => this.scrollToMiddle(), 4);
      setTimeout(() => this.scrollToBottom(), 5);
      setTimeout(() => this.scrollToMiddle(), 6);
      setTimeout(() => this.scrollToTop(), 7);
    }
    this.openedChangeSelect.emit(isOpen);
    this.rerender();
  }

  compare(o1: any, o2: any): boolean {
    if (!!this.optionField && o1 != null) {
      return o1 === o2[this.optionField];
    }
    return compareObjects(o1, o2);
  }

  setValue() {
    this.selectedOptionDisplay = null;
    if (this.selectedValue !== null && this.optionField) {
      const selectedOptionValue = this.options.find(item => item[this.optionField] === this.selectedValue);
      this.selectedOptionDisplay = this.findFieldInObj(this.display, selectedOptionValue);
      this.selectedOptionTooltip = this.findFieldInObj(this.tooltip, selectedOptionValue);
    }
    this.rerender();
  }

  findFieldInObj(field: string, obj: any): any {
    if (obj instanceof Object && obj.hasOwnProperty(field)) {
      return obj[field];
    }
    return null;
  }

  private scrollToTop(): void {
    const x = 0, y = 0;
    this.cdkVirtualScrollViewport.elementRef.nativeElement.scrollTo(x, y);
    this.rerender();
  }

  private scrollToMiddle(): void {
    const x = 0, y = Math.ceil(this.cdkVirtualScrollViewport.elementRef.nativeElement.scrollHeight / 2);
    this.cdkVirtualScrollViewport.elementRef.nativeElement.scrollTo(x, y);
    this.rerender();
  }

  private scrollToBottom(): void {
    const x = 0, y = this.cdkVirtualScrollViewport.elementRef.nativeElement.scrollHeight;
    this.cdkVirtualScrollViewport.elementRef.nativeElement.scrollTo(x, y);
    this.rerender();
  }

  private rerender(): void {
    this.changeDetectorRef.detectChanges();
    this.cdkVirtualScrollViewport.checkViewportSize();
  }
}
