import { Component, OnInit, Input, OnDestroy } from '@angular/core';
import { KeyValue } from '@angular/common';
import { MatSidenav } from '@angular/material/sidenav';
import { Subscription } from 'rxjs';
import { map, finalize } from 'rxjs/operators';

import { NotificacaoModel, NotificacaoService, compareObjects, ErrorModel, redirectTo } from '@datasus/core';

import { RelativeDatePipe } from '../../pipes/relative-date.pipe';

interface PayloadToRemove {
  notification: NotificacaoModel;
  actionId?: number;
}

@Component({
  selector: 'datasus-notification-sidenav',
  templateUrl: './notification-sidenav.component.html',
  styleUrls: ['./notification-sidenav.component.scss'],
})
export class NotificationSidenavComponent implements OnInit, OnDestroy {

  @Input() sidenav: MatSidenav;
  notifications: Map<string, NotificacaoModel[]>;
  loading = false;

  private sidenavOpenedStartSubscription: Subscription;

  constructor(
    private notificacaoService: NotificacaoService,
  ) { }

  ngOnInit(): void {
    this.sidenavOpenedStartSubscription = this.sidenav.openedStart.subscribe(() => {
      this.loading = true;
      this.notifications = null;
      this.notificacaoService.getNotifications().pipe(
        finalize(() => this.loading = false),
        map((notifications: NotificacaoModel[]) => this.notificationToMap(notifications)),
      ).subscribe((notifications: Map<string, NotificacaoModel[]>) => this.notifications = notifications);
    });
  }

  ngOnDestroy(): void {
    this.loading = false;
    this.notifications = null;
    this.sidenavOpenedStartSubscription.unsubscribe();
  }

  removeAll(): void {
    this.loading = true;
    const ids: number[] = [];
    this.notifications.forEach((value: NotificacaoModel[], key: string) => {
      value.forEach((item: NotificacaoModel) => {
        item.removendo = true;
        ids.push(item.id);
      });
    });
    this.notifications = null;
    this.notificacaoService.remove(ids).pipe(
      finalize(() => this.loading = false),
    ).subscribe(
      () => console.log('Notificações removidas com sucesso!'),
      (error: ErrorModel) => console.error('Não foi possível remover todas as notificações', error),
    );
  }

  removeItem(payloadToRemove: PayloadToRemove): void {
    const notificationToRemove: NotificacaoModel = payloadToRemove.notification;
    notificationToRemove.removendo = true;
    if (!!payloadToRemove.actionId) {
      this.notificacaoService.performAction(notificationToRemove.id, payloadToRemove.actionId).subscribe(
        () => redirectTo(notificationToRemove.acoes.find(acao => acao.id === payloadToRemove.actionId).url),
        (error: ErrorModel) => console.error('Não foi possível executar a ação da notificação', error)
      );
    } else {
      this.notificacaoService.remove([notificationToRemove.id]).subscribe(
        () => {
          this.notifications.forEach((value: NotificacaoModel[], key: string) => {
            const valueFiltered = value.filter(
              (notification: NotificacaoModel) => !compareObjects(notification, notificationToRemove)
            );
            if (valueFiltered.length > 0) {
              this.notifications.set(key, valueFiltered);
            } else {
              this.notifications.delete(key);
            }
          });
        },
        (error: ErrorModel) => console.error('Não foi possível remover a notificação', error)
      );
    }
  }

  notificationsKeepOrder(akv: KeyValue<string, NotificacaoModel[]>, bkv: KeyValue<string, NotificacaoModel[]>): number {
    return 1;
  }

  private notificationToMap(notifications: NotificacaoModel[]): Map<string, NotificacaoModel[]> {
    if (!(notifications instanceof Array)) {
      return null;
    }
    if (!notifications.length) {
      return null;
    }

    const result = new Map<string, NotificacaoModel[]>();
    const today = new Date();
    const relativeDatePipe = new RelativeDatePipe();

    notifications.forEach((item: NotificacaoModel) => {
      item.removendo = false;
      const header = relativeDatePipe.transform(item.dataEnvio, (item.dataAtual || today), true);
      if (result.has(header)) {
        const items = result.get(header);
        items.push(item);
        result.set(header, items);
      } else {
        result.set(header, [item]);
      }
    });
    return result;
  }

}
