import { Injectable } from '@angular/core';
import { MatBottomSheet, MatBottomSheetRef } from '@angular/material/bottom-sheet';
import { MatSnackBar } from '@angular/material/snack-bar';

import { DialogAlertComponent } from '../components/dialog-alert/dialog-alert.component';
import { DialogConfirmationComponent } from '../components/dialog-confirmation/dialog-confirmation.component';
import { DialogPromptComponent } from '../components/dialog-prompt/dialog-prompt.component';
import { DialogAlertConfig } from '../configs/dialog-alert.config';
import { DialogConfirmationConfig } from '../configs/dialog-confirmation.config';
import { DialogPromptConfig } from '../configs/dialog-prompt.config';
import { DialogNotificationConfig } from '../configs/dialog-notification.config';

@Injectable({
  providedIn: 'root'
})
export class DialogService {

  constructor(
    private bottomSheet: MatBottomSheet,
    private snackBar: MatSnackBar
  ) { }

  showAlert(configOrMessage: DialogAlertConfig | string): void {
    const config: DialogAlertConfig = (typeof configOrMessage === 'string')
        ? { message: configOrMessage } : configOrMessage;
    const configDefault = {
      message: '',
      messages: [],
      title: 'Alerta!',
      type: 'warn',
    };
    const dialog: MatBottomSheetRef = this.bottomSheet.open(DialogAlertComponent, {
      ariaLabel: 'Diálogo com uma mensagem',
      data: { ...configDefault, ...config }
    });

    dialog.afterDismissed().subscribe((result?: boolean) => {
      if (config.callback) {
        config.callback(result || false);
      }
    });
  }

  showConfirmation(config: DialogConfirmationConfig): void {
    const options: string[] = [];

    config.anwsers.forEach((value: () => void, key: string) => {
      options.push(key);
    });

    const configDefault = {
      title: 'Confirmação…',
      type: 'primary',
      options,
    };

    const configToRemove = { anwsers: null };

    const dialog = this.bottomSheet.open(DialogConfirmationComponent, {
      disableClose: true,
      ariaLabel: `Diálogo com a pergunta: ${config.question}`,
      data: { ...configDefault, ...config, ...configToRemove }
    });

    dialog.afterDismissed().subscribe((button: string) => {
      if (config.anwsers.get(button)) {
        config.anwsers.get(button)();
      }
    });
  }


  showPrompt(config: DialogPromptConfig): void {
    const configDefault = {
      label: 'Informe aqui',
      button: 'Aplicar',
      title: 'Pergunta',
      required: false,
      type: 'primary',
    };
    const dialog: MatBottomSheetRef = this.bottomSheet.open(DialogPromptComponent, {
      disableClose: !!config.required,
      ariaLabel: 'Diálogo com uma mensagem e um campo para resposta',
      data: { ...configDefault, ...config }
    });

    dialog.afterDismissed().subscribe((result?: string) => {
      if (config.callback) {
        config.callback(result || '');
      }
    });
  }

  showNotification(configOrMessage: DialogNotificationConfig | string): void {
    setTimeout(() => {
      const config: DialogNotificationConfig = (typeof configOrMessage === 'string')
        ? { message: configOrMessage } : configOrMessage;

      const secondsToClose = config.secondsToClose || 5;
      const message = config.message;
      const action = (config.action ? Object.keys(config.action)[0].toUpperCase() : '');

      const snackBarRef = this.snackBar.open(message, action, {
        duration: secondsToClose * 1000,
        horizontalPosition: 'right',
        verticalPosition: 'top',
      });

      if (config.dismiss) {
        snackBarRef.afterDismissed().subscribe(config.dismiss);
      }

      if (config.action) {
        const actionCallback = Object.values(config.action);
        if (actionCallback.length) {
          snackBarRef.onAction().subscribe(actionCallback[0]);
        }
      }
    }, 50);
  }
}
