package br.gov.datasus.scodes.acl.controller;

import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.scodes.acl.api.UsuarioApi;
import br.gov.datasus.scodes.acl.model.dto.IdentificacaoUsuarioDTO;
import br.gov.datasus.scodes.acl.model.dto.UserInfoDTO;
import br.gov.datasus.scodes.acl.model.dto.UsuarioDTO;
import br.gov.datasus.scodes.acl.properties.AppProperties;
import br.gov.datasus.scodes.acl.properties.MessageProperity;
import br.gov.datasus.scodes.acl.service.UsuarioService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.oauth2.client.OAuth2RestTemplate;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Api(tags = "Usuario")
@RestController
@AllArgsConstructor
public class UsuarioController implements UsuarioApi {

    private final UsuarioService service;

    private final OAuth2RestTemplate oauthRestTemplate;

    @Autowired
    protected AppProperties.IntegracaoAutorizador integracaoAutorizador;

    @ApiOperation(value = "Obter identificacao usuario por id, cpf ou email")
    @Override
    public ResponseEntity<IdentificacaoUsuarioDTO> obterIdentificacaoUsuario(
        @RequestParam(name = "id", required = false) Long id,
        @RequestParam(name = "email", required = false) String email,
        @RequestParam(name = "cpf", required = false) String cpf) {
        if (id == null && email == null && cpf == null) {
            throw new BusinessException(MessageProperity.USUARIO_PARAMETROS_NAO_INFORMADOS);
        }
        return ResponseEntity.ok(service.obterIdentificacao(id, email, cpf));
    }

    @ApiOperation(value = "Obter os dados completos do usuario por id, cpf ou email")
    @Override
    public ResponseEntity<UsuarioDTO> obterUsuarioCompleto(@RequestParam(name = "id", required = false) Long id,
                                                           @RequestParam(name = "email", required = false) String email,
                                                           @RequestParam(name = "cpf", required = false) String cpf) {
        if (id == null && email == null && cpf == null) {
            throw new BusinessException(MessageProperity.USUARIO_PARAMETROS_NAO_INFORMADOS);
        }
        return ResponseEntity.ok(service.obterUsuario(id, email, cpf));
    }

    @ApiOperation(value = "Obter lista de usuarios por sistema, perfil e esfera")
    @Override
    public ResponseEntity<List<IdentificacaoUsuarioDTO>> obterUsuariosPorSistemaPerfilEsfera(
        @RequestParam(name = "siglaSistema", required = true) String siglaSistema,
        @RequestParam(name = "siglaPerfil", required = true) String siglaPerfil,
        @RequestParam(name = "codigoEsferaAtuacao", required = true) String codigoEsferaAtuacao) {
        List<IdentificacaoUsuarioDTO> lista = service.obterUsuariosPorSistemaPerfilEsfera(siglaSistema, siglaPerfil,
            codigoEsferaAtuacao);
        return ResponseEntity.ok(lista);

    }

    @ApiOperation(value = "Obter lista de CPFs por sistema ou sistema e perfil ou sistema, perfil e esfera")
    @Override
    public ResponseEntity<List<String>> obterCpfsPorSistemaPerfilEsfera(
        @RequestParam(name = "siglaSistema", required = true) String siglaSistema,
        @RequestParam(name = "siglaPerfil", required = false) String siglaPerfil,
        @RequestParam(name = "codigoEsferaAtuacao", required = false) String codigoEsferaAtuacao) {
        if (StringUtils.isNotBlank(codigoEsferaAtuacao) && StringUtils.isBlank(siglaPerfil)) {
            throw new BusinessException(MessageProperity.SIGLA_PERFIL_NAO_INFORMADA);
        }
        return ResponseEntity.ok(service.obterCpfsPorSistemaPerfilEsfera(siglaSistema, siglaPerfil, codigoEsferaAtuacao));
    }

    @ApiOperation(value = "Autenticação do usuário logado.")
    @Override
    public ResponseEntity<Boolean> autenticarUsuario(@RequestBody String hash) {
        UserInfoDTO userInfo = autorizadorRequestUserInfo();
        return ResponseEntity.ok(service.autenticarUsuario(userInfo.getCpf(), hash));
    }

    private UserInfoDTO autorizadorRequestUserInfo() {
        return oauthRestTemplate.getForObject(integracaoAutorizador.urlUserInfo, UserInfoDTO.class);
    }

}
