package br.gov.datasus.authorization.service;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Base64;

import javax.net.ssl.HttpsURLConnection;

import org.jose4j.json.internal.json_simple.JSONArray;
import org.jose4j.json.internal.json_simple.JSONObject;
import org.jose4j.json.internal.json_simple.parser.JSONParser;
import org.jose4j.jwk.PublicJsonWebKey;
import org.jose4j.jwt.JwtClaims;
import org.jose4j.jwt.consumer.JwtConsumer;
import org.jose4j.jwt.consumer.JwtConsumerBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;

import br.gov.datasus.authorization.configuration.properties.ApplicationProperties;
import br.gov.datasus.authorization.configuration.properties.MessageConstants;
import br.gov.datasus.authorization.configuration.properties.MessageSource;
import br.gov.datasus.authorization.enums.AuthorizationScopeEnum;
import br.gov.datasus.authorization.exception.AuthorizationException;
import br.gov.datasus.authorization.model.scpa.Usuario;
import br.gov.datasus.authorization.security.oauth2.model.User;
import br.gov.datasus.authorization.security.oauth2.model.UserAuthority;

@Service
public class IntegracaoLoginCidadaoService {

    private static final String URL_JWK_TOKEN = "/jwk";

    private static final String URL_AUTHORIZATION = "/token?grant_type=authorization_code&code=%s&redirect_uri=%s";

    private static final String URL_SELEO_CONFIABILIDADE = "/api/info/usuario/selo";

    private static final String BASIC_PROPERTY_PARAMS = "%s:%s";

    private static final String BASIC_PROPERTY = "Basic %s";

    private static final String BEARER = "Bearer ";

    private static final String ACCEPT = "Accept";

    private static final String AUTHORIZATION = "Authorization";

    private static final String APPLICATION_JSON = "application/json";

    private static final String REQUEST_PROPERTY_AUTHORIZATION = "authorization";

    private static final String KEY_TOKEN_CIDADAO_AMR = "amr";

    private static final String KEY_TOKEN_CIDADAO_EMAIL = "email";

    private static final String KEY_TOKEN_CIDADAO_NAME = "name";

    private static final String KEY_TOKEN_CIDADAO_NIVEL = "nivel";

    private static final String KEY_TOKEN_CIDADAO_KEYS = "keys";

    private static final String KEY_ID_TOKEN = "id_token";

    private static final String KEY_ACCESS_TOKEN = "access_token";

    private static final String SEPARADOR_ROLE = "_";

    private static final String SEPARADOR_URL = "/";

    private static final String ROLE_CIDADAO = "ROLE_CIDADAO";

    private static final String ROLE_CIDADAO_CONFIABILIDADE_NIVEL = "ROLE_CIDADAO_CONFIABILIDADE_NIVEL";

    @Autowired
    private ApplicationProperties.External external;

    @Autowired
    private UsuarioScpaService usuarioScpaService;

    public UserDetails loadUser(String code) throws AuthenticationException {

        String tokens = requestTokenCidadao(code);

        ArrayList<UserAuthority> list = new ArrayList<UserAuthority>();
        list.add(new UserAuthority(ROLE_CIDADAO));

        User user = new User();

        user.getUserInfo().setOrigem(AuthorizationScopeEnum.GOV_BR.getCodigo());
        try {
            JSONParser parser = new JSONParser();
            JSONObject tokensJson = (JSONObject) parser.parse(tokens);
            String idToken = (String) tokensJson.get(KEY_ID_TOKEN);
            JwtClaims idTokenJwtClaims = processToClaims(idToken);
            user.setUsername(idTokenJwtClaims.getSubject());
            user.getUserInfo().setCpf(idTokenJwtClaims.getSubject());
            user.getUserInfo().setNome(idTokenJwtClaims.getStringClaimValue(KEY_TOKEN_CIDADAO_NAME));
            user.getUserInfo().setEmail(idTokenJwtClaims.getClaimValue(KEY_TOKEN_CIDADAO_EMAIL).toString());
            // Fator de autenticação do usuário. Pode ser “passwd” com senha, ou “x509” com
            // certificado digital ou certificado em nuvem.
            list.add(new UserAuthority(ROLE_CIDADAO + SEPARADOR_ROLE
                    + idTokenJwtClaims.getClaimValue(KEY_TOKEN_CIDADAO_AMR).toString().toUpperCase()));
            String confiabilidadeJson = obterFatorConfiabilidade((String) tokensJson.get(KEY_ACCESS_TOKEN));
            for (Object o : (JSONArray) parser.parse(confiabilidadeJson)) {
                if (o instanceof JSONObject) {
                    list.add(new UserAuthority(ROLE_CIDADAO_CONFIABILIDADE_NIVEL + SEPARADOR_ROLE
                            + ((JSONObject) o).get(KEY_TOKEN_CIDADAO_NIVEL).toString().toUpperCase()));
                }
            }
            Usuario usuarioScpa = usuarioScpaService.findUserByCpf(user.getUsername());
            if (usuarioScpa != null) {
                usuarioScpaService.addRolesScpa(user, usuarioScpa);
            }
        } catch (Exception e) {
            throw new AuthorizationException("Token Cidadão Inválido!");
        }
        user.getAuthorities().addAll(list);
        return user;
    }

    private JwtClaims processToClaims(String token) throws Exception {
        URL url = new URL(external.urlProviderLoginCidadao + URL_JWK_TOKEN);
        HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();
        conn.setRequestMethod(HttpMethod.GET.toString());
        conn.setRequestProperty(ACCEPT, APPLICATION_JSON);
        if (conn.getResponseCode() != HttpStatus.OK.value()) {
            throw new AuthorizationException(
                    MessageSource.get().message(MessageConstants.HTTP_ERROR, String.valueOf(conn.getResponseCode())));
        }
        BufferedReader br = new BufferedReader(new InputStreamReader((conn.getInputStream())));
        String ln = null, jwk = "";
        while ((ln = br.readLine()) != null) {
            jwk += ln;
        }
        conn.disconnect();
        JSONParser parser = new JSONParser();
        JSONObject tokensJson = (JSONObject) parser.parse(jwk);
        JSONArray keys = (JSONArray) tokensJson.get(KEY_TOKEN_CIDADAO_KEYS);
        JSONObject keyJSONObject = (JSONObject) keys.get(0);
        String key = keyJSONObject.toJSONString();
        PublicJsonWebKey pjwk = PublicJsonWebKey.Factory.newPublicJwk(key);
        JwtConsumer jwtConsumer = new JwtConsumerBuilder().setRequireExpirationTime()
                .setExpectedAudience(external.providerCidadaoClientId).setMaxFutureValidityInMinutes(60)
                .setAllowedClockSkewInSeconds(30).setRequireSubject()
                .setExpectedIssuer(external.urlProviderLoginCidadao + SEPARADOR_URL)
                .setVerificationKey(pjwk.getPublicKey()).build();

        return jwtConsumer.processToClaims(token);
    }

    private String obterFatorConfiabilidade(String accessToken) throws Exception {
        String retorno = "";
        URL url = new URL(external.urlApiCidadao + URL_SELEO_CONFIABILIDADE);
        HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();
        conn.setRequestMethod(HttpMethod.GET.toString());
        conn.setRequestProperty(ACCEPT, APPLICATION_JSON);
        conn.setRequestProperty(AUTHORIZATION, BEARER + accessToken);
        if (conn.getResponseCode() != HttpStatus.OK.value()) {
            throw new AuthorizationException(
                    MessageSource.get().message(MessageConstants.HTTP_ERROR, String.valueOf(conn.getResponseCode())));
        }
        String output;
        BufferedReader br = new BufferedReader(new InputStreamReader((conn.getInputStream())));
        while ((output = br.readLine()) != null) {
            retorno += output;
        }
        conn.disconnect();
        return retorno;
    }

    private String requestTokenCidadao(String code) {
        String retorno = "";
        try {
            String redirectUrl = URLEncoder.encode(external.urlCallbackLoginCidadao, StandardCharsets.UTF_8.name());
            URL url = new URL(String.format(external.urlProviderLoginCidadao + URL_AUTHORIZATION, code, redirectUrl));
            HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();
            conn.setRequestMethod(HttpMethod.POST.toString());
            conn.setRequestProperty(ACCEPT, APPLICATION_JSON);
            conn.setRequestProperty(REQUEST_PROPERTY_AUTHORIZATION,
                    String.format(BASIC_PROPERTY,
                            Base64.getEncoder().encodeToString(
                                    String.format(BASIC_PROPERTY_PARAMS, external.providerCidadaoClientId,
                                            external.providerCidadaoClientSecret).getBytes())));
            if (conn.getResponseCode() != HttpStatus.OK.value()) {
                throw new AuthorizationException(MessageSource.get().message(MessageConstants.HTTP_ERROR,
                        String.valueOf(conn.getResponseCode())));
            }
            BufferedReader br = new BufferedReader(new InputStreamReader((conn.getInputStream())));
            String tokens = null;
            while ((tokens = br.readLine()) != null) {
                retorno += tokens;
            }
            conn.disconnect();
        } catch (Exception e) {
            throw new AuthorizationException(MessageSource.get().message(MessageConstants.TOKEN_INVALID));
        }
        return retorno;
    }

}
