package br.gov.datasus.authorization.service;

import java.nio.charset.StandardCharsets;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;

import com.google.common.hash.Hashing;

import br.gov.datasus.authorization.configuration.properties.MessageConstants;
import br.gov.datasus.authorization.configuration.properties.MessageSource;
import br.gov.datasus.authorization.enums.AuthorizationScopeEnum;
import br.gov.datasus.authorization.exception.AuthorizationException;
import br.gov.datasus.authorization.model.scpa.Role;
import br.gov.datasus.authorization.model.scpa.Usuario;
import br.gov.datasus.authorization.repository.scpa.UsuarioScpaRepository;
import br.gov.datasus.authorization.security.oauth2.model.User;
import br.gov.datasus.authorization.security.oauth2.model.UserAuthority;

@Service
public class UsuarioScpaService {

    private static final String SEPARADOR_EMAIL = "@";

    private static final String SEPARADOR_ROLE = "_";

    private static final String DOMINIO_VALIDO = "_DOMINIO-VALIDO";

    private static final String SISRH_VALIDO = "_SISRH-VALIDO";

    private static final String ROLE = "ROLE_";

    @Autowired
    private UsuarioScpaRepository usuarioScpaRepository;

    public Boolean authenticate(String username, String password) {
        final String scpaPassword = Hashing.sha256().hashString(password, StandardCharsets.UTF_8).toString();
        return usuarioScpaRepository.autenticarUsuario(username, scpaPassword);
    }

    public UserDetails loadUser(String username, String password) throws AuthenticationException {
        final String scpaPassword = Hashing.sha256().hashString(password, StandardCharsets.UTF_8).toString();
        Optional<List<Usuario>> usuarios = usuarioScpaRepository.findUsuarioByLoginSenha(username, scpaPassword);
        usuarios.orElseThrow(
                () -> new AuthorizationException(MessageSource.get().message(MessageConstants.USER_PASSWORD_INVALID)));
        Usuario usuarioScpa = usuarios.get().get(0);
        User user = criarUsuario(usuarioScpa);
        addRolesScpa(user, usuarioScpa);
        return user;
    }

    public void addRolesScpa(User user, Usuario usuarioScpa) {
        Set<String> sistemas = new HashSet<>();
        List<Role> roles = usuarioScpaRepository.findByRolesPerfil(usuarioScpa.getId());
        roles.forEach(authority -> {
            user.getAuthorities().add(new UserAuthority(
                    ROLE + authority.getSiglaSistema() + SEPARADOR_ROLE + authority.getSiglaPerfil()));
            if (authority.getValidaRH()
                    && usuarioScpaRepository.findByRolesValidaRH(usuarioScpa.getCpf(), usuarioScpa.getEmail()) > 0) {
                user.getAuthorities().add(new UserAuthority(ROLE + authority.getSiglaSistema() + SISRH_VALIDO));
            }
            if (authority.getValidaDominios() && !sistemas.contains(authority.getSiglaSistema())
                    && usuarioScpaRepository.findByRolesValidaDominio(usuarioScpa.getEmail().split(SEPARADOR_EMAIL)[1],
                            authority.getIdSistema()) > 0) {
                user.getAuthorities().add(new UserAuthority(ROLE + authority.getSiglaSistema() + DOMINIO_VALIDO));
            }
            sistemas.add(authority.getSiglaSistema());
        });
        sistemas.forEach(authority -> user.getAuthorities().add(new UserAuthority(ROLE + authority)));
    }

    private User criarUsuario(Usuario usuario) {
        User user = new User();
        user.setUsername(usuario.getCpf());
        user.getUserInfo().setKey(usuario.getId());
        user.getUserInfo().setNome(usuario.getNome());
        user.getUserInfo().setCpf(usuario.getCpf());
        user.getUserInfo().setEmail(usuario.getEmail());
        user.getUserInfo().setCnes(usuario.getNumeroCns());
        user.getUserInfo().setNome(usuario.getNome());
        user.getUserInfo().setOrigem(AuthorizationScopeEnum.SCPA.getCodigo());
        return user;
    }

    public Usuario findUserByCpf(String cpf) {
        List<Usuario> usuarios = usuarioScpaRepository.findByCpf(cpf);
        if (!usuarios.isEmpty()) {
            return usuarios.get(0);
        }
        return null;
    }

}
