package br.gov.datasus.scodes.controller;

import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.*;
import br.gov.datasus.scodes.model.dto.filter.AlteracaoDataRetornoFilterDTO;
import br.gov.datasus.scodes.model.dto.filter.PesquisaDRAutorDemandaReciboFilterDTO;
import br.gov.datasus.scodes.model.dto.filter.RelatorioDataRetornoAutorDemandaFilterDTO;
import br.gov.datasus.scodes.model.dto.filter.ValidarAlteracaoNovaDataRetornoFilterDTO;
import br.gov.datasus.scodes.model.dto.projection.ResultadoAlterarDataRetornoDTO;
import br.gov.datasus.scodes.service.AlteracaoDataRetornoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "AlteracaoDataRetorno")
@RequestMapping(value = "/alteracao-data-retorno")
@RequiredArgsConstructor
@RestController
public class AlteracaoDataRetornoController {

    private final AlteracaoDataRetornoService service;

    @ApiOperation(value = "Obter alteracao-data-retorno por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<AlteracaoDataRetornoDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todos alteracao-data-retorno")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<AlteracaoDataRetornoDTO>> obterTodos() {
        return ResponseEntity.ok(service.obterTodos());
    }

    @ApiOperation(value = "Obter todos resultado-alterar-data-retorno")
    @GetMapping(path = "/resultado-alterar-data-retorno/{codigoAlteracaoDataRetorno}",produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<ResultadoAlterarDataRetornoDTO>> obterResultadoAlterarDataRetorno(@PathVariable Long codigoAlteracaoDataRetorno) {
        return ResponseEntity.ok(service.obterResultadoAlterarDataRetorno(codigoAlteracaoDataRetorno));
    }

    @ApiOperation(value = "Obter todos alteracao-data-retorno paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<AlteracaoDataRetornoDTO>> obterTodosPaginado(
        @Valid @RequestBody PageableRequestModel<AlteracaoDataRetornoFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }


    @ApiOperation(value = "Obter pesquisa-autor-demanda paginado")
    @PostMapping(value = "/pesquisa-autor-demanda/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<ResultadoPesquisaDRAutorDemandaReciboDTO>> pesquisaDTAutorDemandaRecibo(
        @Valid @RequestBody PageableRequestModel<PesquisaDRAutorDemandaReciboFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.pesquisaDTAutorDemandaRecibo(pageableModel));
    }

    @ApiOperation(value = "Incluir alteracao-data-retorno")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<SalvarAlteracaoDataRetornoDTO> incluir(@Valid @RequestBody SalvarAlteracaoDataRetornoDTO dto) {
        return new ResponseEntity<SalvarAlteracaoDataRetornoDTO>(service.salvarAlteracaoDataRetorno(dto), HttpStatus.CREATED);
    }

    @ApiOperation(value = "Atualizar alteracao-data-retorno")
    @PutMapping(value = "/{id}", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<AlteracaoDataRetornoDTO> alterar(@Valid @RequestBody SalvarAlteracaoDataRetornoDTO dto, @PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.alterarDataRetorno(id, dto));
    }

    @ApiOperation(value = "Excluir alteracao-data-retorno por id")
    @DeleteMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> excluir(@PathVariable(name = "id") Long id) {
        service.excluirAlteracaoDataRetorno(id);
        return new ResponseEntity<Void>(HttpStatus.OK);
    }

    @ApiOperation(value = "Obter pesquisa-autor-demanda-adr")
    @GetMapping(value = "/pesquisa-autor-demanda-adr/{codigoAlteracaoDataRetorno}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<ResultadoPesquisaDRAutorDemandaReciboOrdemProdutoDTO>> pesquisaDRAutorDemandaADRPorCodigo(@PathVariable Long codigoAlteracaoDataRetorno) {
        return ResponseEntity.ok(service.pesquisaDRAutorDemandaADRPorCodigo(codigoAlteracaoDataRetorno));
    }

    @ApiOperation(value = "Gerar relatório de alteração de data retorno")
    @PostMapping(value = "/relatorio-csv", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ArquivoBase64DTO> gerarRelatorioCsv(@Valid @RequestBody RelatorioDataRetornoAutorDemandaFilterDTO filter) {
        return ResponseEntity.ok(service.gerarRelatorioCsv(filter));
    }

    @ApiOperation(value = "Gerar relatório consolidado do inventário")
    @GetMapping(value = "/{codigoAlteracaoDataRetorno}/relatorio-pdf", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ArquivoBase64DTO> gerarRelatorioPdf(@PathVariable Long codigoAlteracaoDataRetorno) {
        return ResponseEntity.ok(service.gerarRelatorioPdf(codigoAlteracaoDataRetorno));
    }

    @ApiOperation(value = "Validação validar-nova-data-retorno")
    @PostMapping("/validar-nova-data-retorno")
    public ResponseEntity<Void> validarNovaDataRetorno(
        @Valid @RequestBody ValidarAlteracaoNovaDataRetornoFilterDTO dto) {
        service.validarNovaDataRetorno(dto);
        return new ResponseEntity<Void>(HttpStatus.OK);
    }

    @ApiOperation(value = "Validação validar-nova-data-retorno-editar")
    @PostMapping("/validar-nova-data-retorno-editar/{codigoAlteracaoDataRetorno}")
    public ResponseEntity<Void> validarNovaDataRetornoEditar(@PathVariable Long codigoAlteracaoDataRetorno,
        @Valid @RequestBody ValidarAlteracaoNovaDataRetornoFilterDTO dto) {
        service.validarNovaDataRetornoEditar(codigoAlteracaoDataRetorno, dto);
        return new ResponseEntity<Void>(HttpStatus.OK);
    }
}
