package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.AcessoDTO;
import br.gov.datasus.scodes.model.dto.ArquivoBase64DTO;
import br.gov.datasus.scodes.model.dto.filter.AcessoFilterDTO;
import br.gov.datasus.scodes.model.dto.filter.RelatorioAcessoFilterDTO;
import br.gov.datasus.scodes.model.entity.Acesso;
import br.gov.datasus.scodes.model.entity.UnidadeDispensadora;
import br.gov.datasus.scodes.model.entity.UnidadeOrganizacional;
import br.gov.datasus.scodes.model.mapper.AcessoMapper;
import br.gov.datasus.scodes.repository.AcessoRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.thymeleaf.TemplateEngine;
import org.thymeleaf.context.Context;

import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class AcessoService {

	private final AcessoRepository repository;
	private final AcessoMapper mapper;
	private final TemplateEngine templateEngine;

	public AcessoDTO obterPorId(Long id) {
		Acesso entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
		return mapper.toDto(entity);
	}

	public PageableResponseModel<AcessoDTO> obterTodosPaginado(PageableRequestModel<AcessoFilterDTO> pageableModel) {
		DynamicParameters parameters = DynamicParameters.get();
		pageableModel.getFilter().ifPresent(filter -> {
			Optional.ofNullable(filter.getNomeLogin())
				.ifPresent(nomeLogin -> parameters.appendLike("nomeLogin", nomeLogin));
		});
		Pageable pagination = PaginationHelper.toPageable(pageableModel, "a");
		PageableResponseModel<Acesso> pageableResponseModel = repository.findAll(Acesso.OBTER_TODOS_PAGINADO, parameters, pagination, Acesso.class);
		return mapper.toDtoPageableResponse(pageableResponseModel);
	}
	
	public ArquivoBase64DTO gerarRelatorio(RelatorioAcessoFilterDTO filterDTO) {
		DynamicParameters parameters = DynamicParameters.get();
		Optional.ofNullable(filterDTO).ifPresent(filter -> {
			Optional.ofNullable(filter.getNomeLogin())
				.ifPresent(nomeLogin -> parameters.appendLike("nomeLogin", nomeLogin));
			
			Optional.ofNullable(filter.getNomeUsuario())
				.ifPresent(nomeUsuario -> parameters.appendLike("nomeUsuario", nomeUsuario));
			
			Optional.ofNullable(filter.getDataEntrada())
				.map(dataEntrada -> dataEntrada.toInstant().atZone(ZoneId.systemDefault()).toLocalDate())
				.ifPresent(dataEntrada -> parameters.appendDate("dataEntrada", dataEntrada));
			
			Optional.ofNullable(filter.getDataSaida())
				.map(dataSaida -> dataSaida.toInstant().atZone(ZoneId.systemDefault()).toLocalDate().plusDays(1))
				.ifPresent(dataSaida -> parameters.appendDate("dataSaida", dataSaida));
		});
		
		List<Acesso> acessos = repository.findAll(Acesso.OBTER_TODOS_RELATORIO, parameters, Acesso.class);

		List<Map<String, Object>> itens = new ArrayList<>();
        acessos.forEach(acesso -> {
		    Map<String, Object> item = new HashMap<>();
            item.put("nomeLogin", acesso.getUsuarioLocal().getNomeLogin());
            item.put("nomeUsuario", acesso.getUsuarioLocal().getNomeUsuario());
            item.put("unidadeOrganizacional", Optional.ofNullable(acesso.getUsuarioLocal().getUnidadeOrganizacional())
                .map(UnidadeOrganizacional::getDescricao).orElse(""));
            item.put("unidadesDispensadoras", Optional.ofNullable(acesso.getUsuarioLocal().getUnidadesDispensadoras())
                .orElse(new HashSet<>()).stream().map(UnidadeDispensadora::getDescricao).collect(Collectors.toList()));
            itens.add(item);
        });

		Context ctx = new Context(Locale.getDefault());
		ctx.setVariable("itens", itens);
		String csv = this.templateEngine.process("relatorio_acessos.txt", ctx);
		String base64 = Base64.getEncoder().encodeToString(csv.getBytes());
		return new ArquivoBase64DTO(base64, "relatorio_acessos.csv", "text/csv");
		
	}
}
