package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.EspecificacaoDTO;
import br.gov.datasus.scodes.model.dto.filter.EspecificacaoFilterDTO;
import br.gov.datasus.scodes.model.entity.Especificacao;
import br.gov.datasus.scodes.model.mapper.EspecificacaoMapper;
import br.gov.datasus.scodes.repository.EspecificacaoRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class EspecificacaoService {

    private final EspecificacaoRepository repository;
    private final EspecificacaoMapper mapper;

    public EspecificacaoDTO obterPorId(Long id) {
        Especificacao entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<EspecificacaoDTO> obterTodos() {
        Iterable<Especificacao> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<EspecificacaoDTO> obterTodosPaginado(PageableRequestModel<EspecificacaoFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "espec");
        DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getIdProduto())
                .ifPresent(idProduto -> parameters.append("idProduto", idProduto));
        });
        PageableResponseModel<Especificacao> pageableResponseModel = repository.findAll(Especificacao.OBTER_TODOS_PAGINADO, parameters, pagination, Especificacao.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio(Long  idProduto) {
        if (idProduto != null) {
            return repository.obterDominio(idProduto);
        }
        return repository.obterDominio();
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public EspecificacaoDTO incluir(EspecificacaoDTO dto) {
        Especificacao entity = mapper.toEntity(dto);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public EspecificacaoDTO alterar(Long id, EspecificacaoDTO dto) {
        Especificacao entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        dto.setId(id);
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void excluir(Long id) {
        Especificacao entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        if (repository.countVinculosItens(entity) > 0) {
            throw new BusinessException(MessageProperty.ESPECIFICACAO_EXCLUSAO_VINCULO);
        }
        repository.delete(entity);
    }


}
