package br.gov.datasus.scodes.service;

import java.util.List;
import java.util.Optional;

import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.GrupoDTO;
import br.gov.datasus.scodes.model.dto.filter.GrupoFilterDTO;
import br.gov.datasus.scodes.model.entity.Grupo;
import br.gov.datasus.scodes.model.mapper.GrupoMapper;
import br.gov.datasus.scodes.repository.GrupoRepository;

import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class GrupoService {

	private final GrupoRepository repository;
	private final GrupoMapper mapper;

	public GrupoDTO obterPorId(Long id) {
		Grupo entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
		return mapper.toDto(entity);
	}

	public List<GrupoDTO> obterTodos() {
		Iterable<Grupo> itens = repository.findAll();
		return mapper.toDto(itens);
	}

	public PageableResponseModel<GrupoDTO> obterTodosPaginado(PageableRequestModel<GrupoFilterDTO> pageableModel) {
		Pageable pagination = PaginationHelper.toPageable(pageableModel, "g");
        DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getNome())
                .ifPresent(nome -> parameters.appendLike("nome", nome));
        });
		PageableResponseModel<Grupo> pageableResponseModel = repository.findAll(Grupo.OBTER_TODOS_PAGINADO, parameters,
				pagination, Grupo.class);
		return mapper.toDtoPageableResponse(pageableResponseModel);
	}

	public List<DominioItemModel> obterDominio() {
		return repository.obterDominio();
	}

	@Transactional(propagation = Propagation.REQUIRED)
	public GrupoDTO incluir(GrupoDTO dto) {
		Grupo entity = mapper.toEntity(dto);
		return mapper.toDto(repository.save(entity));
	}

	@Transactional(propagation = Propagation.REQUIRED)
	public GrupoDTO alterar(Long id, GrupoDTO dto) {
		Grupo entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
		dto.setId(id);
		mapper.fromDto(dto, entity);
		repository.save(entity);
		return mapper.toDto(entity);
	}

	@Transactional(propagation = Propagation.REQUIRED)
	public void excluir(Long id) {
		Grupo entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
		repository.delete(entity);
	}
}
