package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.model.NullCondition;
import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.InventarioItemConsolidadoDTO;
import br.gov.datasus.scodes.model.dto.InventarioItemDTO;
import br.gov.datasus.scodes.model.dto.filter.InventarioItemFilterDTO;
import br.gov.datasus.scodes.model.entity.InventarioItem;
import br.gov.datasus.scodes.model.enums.TipoMovimentacaoEnum;
import br.gov.datasus.scodes.model.mapper.InventarioItemMapper;
import br.gov.datasus.scodes.model.mapper.generic.InventarioItemConsolidadoGenericMapper;
import br.gov.datasus.scodes.model.mapper.generic.InventarioItemGenericMapper;
import br.gov.datasus.scodes.repository.InventarioItemRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Set;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class InventarioItemService {

	private final InventarioItemRepository repository;
	private final InventarioItemMapper mapper;
	private final InventarioItemGenericMapper inventarioItemGenericMapper;
    private final InventarioItemConsolidadoGenericMapper inventarioItemConsolidadoGenericMapper;
	private final MovimentacaoService movimentacaoService;
    private final EntradaEstoqueItemService entradaEstoqueItemService;

	public InventarioItemDTO obterPorId(Long id) {
        InventarioItem entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
		return mapper.toDto(entity);
	}

    public PageableResponseModel<InventarioItemDTO> obterTodosPaginado(PageableRequestModel<InventarioItemFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "ii");
        DynamicParameters parameters = this.buildParameters(pageableModel.getFilter().orElse(null));
        return repository.findAllNative("inventario_item", parameters, pagination, inventarioItemGenericMapper);
    }

    public List<InventarioItemDTO> obterTodos(InventarioItemFilterDTO filterDTO) {
        DynamicParameters parameters = this.buildParameters(filterDTO);
        return repository.findAllNative("inventario_item", parameters, inventarioItemGenericMapper);
    }

    public List<InventarioItemDTO> obterItens(Long idInventario, Set<Long> idItens) {
        DynamicParameters parameters = DynamicParameters.get().append("idInventario", idInventario);
        if (idItens != null && !idItens.isEmpty()) {
            parameters.append("idItens", idItens);
        }
        return repository.findAllNative("inventario_item", parameters, inventarioItemGenericMapper);
    }

    public List<InventarioItemConsolidadoDTO> obterItensConsolidado(Long idInventario) {
        DynamicParameters parameters = DynamicParameters.get().append("idInventario", idInventario);
        return repository.findAllNative("inventario_consolidado", parameters, inventarioItemConsolidadoGenericMapper);
    }

    private DynamicParameters buildParameters(InventarioItemFilterDTO filterDto) {
        DynamicParameters parameters = DynamicParameters.get();
        Optional.ofNullable(filterDto).ifPresent(filter -> {
            Optional.ofNullable(filter.getIdInventario())
                .ifPresent(idInventario -> parameters.append("idInventario", idInventario));
            Optional.ofNullable(filter.getCodigoItem())
                .ifPresent(codigoItem -> parameters.append("codigoItem", codigoItem));
            Optional.ofNullable(filter.getIdProduto())
                .ifPresent(idProduto -> parameters.append("idProduto", idProduto));
            Optional.ofNullable(filter.getIdIsNull()).ifPresent(idIsNull -> {
                if (idIsNull) {
                    parameters.append("idIsNull", NullCondition.IS_NULL);
                } else {
                    parameters.append("idIsNull", NullCondition.IS_NOT_NULL);
                }
            });
        });
	    return parameters;
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public List<InventarioItemDTO> salvar(List<InventarioItemDTO> dtos) {
	    List<InventarioItem> entities = new ArrayList<>();
        dtos.forEach(dto -> {
            InventarioItem entity;
            if (dto.getId() != null) {
                entity = repository.findById(dto.getId()).orElseThrow(ResourceNotFoundException::new);
                mapper.fromDto(dto, entity);
            } else {
                entity = mapper.toEntity(dto);
            }
            repository.save(entity);
            entities.add(entity);
        });
        return mapper.toDto(entities);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void consolidar(InventarioItem inventarioItem) {
        inventarioItem.setQuantidadeEstoqueDataContagem(
            movimentacaoService.obterQuantidadeItemPorDataMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem()
            )
        );
        inventarioItem.setQuantidadeCompraEntrada(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.COMPRA_ENTRADA
            )
        );
        inventarioItem.setQuantidadeOutraEntrada(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.OUTRA_ENTRADA
            )
        );
        inventarioItem.setQuantidadePerdaSaida(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.PERDA_TRANSFERENCIA
            )
        );
        inventarioItem.setQuantidadeTransferenciaSaida(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.TRANSFERENCIA_SAIDA
            )
        );
        inventarioItem.setQuantidadeDispensacaoSaida(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.DISPENSACAO
            )
        );
        inventarioItem.setQuantidadeEstornoEntrada(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.ESTORNO_DISPENSACAO
            )
        );
        inventarioItem.setQuantidadeReciboInvalidadoEntrada(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.RECIBO_INVALIDADO_DISPENSACAO
            )
        );
        inventarioItem.setQuantidadeOutraSaida(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.OUTRA_SAIDA
            )
        );
        inventarioItem.setQuantidadePerdaVencimentoSaida(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.PERDA_VENCIMENTO_SAIDA
            )
        );
        inventarioItem.setQuantidadeQuebraSaida(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.QUEBRA_SAIDA
            )
        );
        inventarioItem.setQuantidadeAjusteSaida(
            movimentacaoService.obterQuantidadeItemPorTipoMovimentacao(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora(),
                inventarioItem.getDataContagem(),
                TipoMovimentacaoEnum.AJUSTES_SAIDA
            )
        );
        inventarioItem.setQuantidadeItemPosCalculos(this.calculaQuantidadeItemPosCalculos(inventarioItem));
        inventarioItem.setQuantidadeItemDiferenca(this.calculaQuantidadeItemDiferenca(inventarioItem));
        inventarioItem.setValorItem(
            entradaEstoqueItemService.obterValorItemUltimaEntrada(
                inventarioItem.getItem(),
                inventarioItem.getUnidadeDispensadora()
            )
        );
        repository.save(inventarioItem);
    }

    private Long calculaQuantidadeItemPosCalculos(InventarioItem inventarioItem) {
	    return inventarioItem.getQuantidadeEstoqueDataContagem()
            + inventarioItem.getQuantidadeCompraEntrada()
            + inventarioItem.getQuantidadeOutraEntrada()
            + inventarioItem.getQuantidadeReciboInvalidadoEntrada()
            + inventarioItem.getQuantidadeEstornoEntrada()
            - inventarioItem.getQuantidadePerdaSaida()
            - inventarioItem.getQuantidadeQuebraSaida()
            - inventarioItem.getQuantidadePerdaVencimentoSaida()
            - inventarioItem.getQuantidadeAjusteSaida()
            - inventarioItem.getQuantidadeOutraSaida()
            - inventarioItem.getQuantidadeDispensacaoSaida();
    }

    private Long calculaQuantidadeItemDiferenca(InventarioItem inventarioItem) {
	    return inventarioItem.getQuantidadeFisica() - inventarioItem.getQuantidadeItemPosCalculos();
    }

}
