package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.ArquivoBase64DTO;
import br.gov.datasus.scodes.model.dto.InventarioDTO;
import br.gov.datasus.scodes.model.dto.InventarioItemConsolidadoDTO;
import br.gov.datasus.scodes.model.dto.InventarioItemDTO;
import br.gov.datasus.scodes.model.dto.filter.InventarioFilterDTO;
import br.gov.datasus.scodes.model.entity.Inventario;
import br.gov.datasus.scodes.model.mapper.InventarioMapper;
import br.gov.datasus.scodes.repository.InventarioRepository;
import br.gov.datasus.scodes.repository.UnidadeDispensadoraRepository;
import lombok.RequiredArgsConstructor;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.ByteArrayOutputStream;
import java.time.ZoneId;
import java.util.*;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class InventarioService {

	private final InventarioRepository repository;
	private final UnidadeDispensadoraRepository unidadeDispensadoraRepository;
	private final InventarioMapper mapper;
	private final InventarioItemService inventarioItemService;
    private final ReportService reportService;

	public InventarioDTO obterPorId(Long id) {
		Inventario entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
		return mapper.toDto(entity);
	}

    public PageableResponseModel<InventarioDTO> obterTodosPaginado(PageableRequestModel<InventarioFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "i");
        DynamicParameters parameters = DynamicParameters.get();

        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getDescricao())
                .ifPresent(descricao -> parameters.appendLike("descricao", descricao));
            Optional.ofNullable(filter.getIdUnidadeDispensadora())
                .ifPresent(idUnidadeDispensadora -> parameters.append("idUnidadeDispensadora", idUnidadeDispensadora));
            Optional.ofNullable(filter.getDataInicio())
                .map(dataInicio -> dataInicio.toInstant().atZone(ZoneId.systemDefault()).toLocalDate())
                .ifPresent(dataInicio -> parameters.appendDate("dataInicio", dataInicio));
            Optional.ofNullable(filter.getDataFim())
                .map(dataFim -> dataFim.toInstant().atZone(ZoneId.systemDefault()).toLocalDate().plusDays(1))
                .ifPresent(dataFim -> parameters.appendDate("dataFim", dataFim));
            Optional.ofNullable(filter.getSituacaoAtivo())
                .ifPresent(situacaoAtivo -> parameters.append("situacaoAtivo", situacaoAtivo));
        });

        PageableResponseModel<Inventario> pageableResponseModel = repository.findAll(Inventario.OBTER_TODOS_PAGINADO,
            parameters, pagination, Inventario.class);

        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

	public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public InventarioDTO incluir(InventarioDTO dto) {
        Inventario entity = mapper.toEntity(dto);
        entity.setUnidadeDispensadora(
            unidadeDispensadoraRepository.findById(entity.getUnidadeDispensadora().getId())
                .orElseThrow(() -> new BusinessException(MessageProperty.UNIDADE_DISPENSADORA_NAO_ENCONTRADA))
        );
        if (repository.countByUnidadeDispensadoraAndSituacaoAtivoIsTrue(entity.getUnidadeDispensadora()) > 0) {
            throw new BusinessException(MessageProperty.INVENTARIO_ATIVO_UNIDADE_DISPENSADORA);
        }
        if (dto.getDescricao() == null || dto.getDescricao().isEmpty()) {
            entity.setDescricao(gerarDescricao(entity));
        }
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void consolidar(Long id) {
        Inventario entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        if (!entity.getSituacaoAtivo()) {
            throw new BusinessException(MessageProperty.INVENTARIO_JA_CONSOLIDADO);
        }
        entity.setSituacaoAtivo(false);
        entity.getItens().forEach(inventarioItemService::consolidar);
        repository.save(entity);
    }

    private String gerarDescricao(Inventario entity) {
	    Long count = repository.countByUnidadeDispensadora(entity.getUnidadeDispensadora());
	    return ++count + "° - " + entity.getUnidadeDispensadora().getDescricao();
    }

    public ArquivoBase64DTO gerarRelatorioItem(Long id, Set<Long> idItens) {
        Inventario entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        List<InventarioItemDTO> itens = inventarioItemService.obterItens(id, idItens);
        return this.gerarRelatorio("inventario_item", entity, itens);
    }

    public ArquivoBase64DTO gerarRelatorioConsolidado(Long id) {
        Inventario entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        List<InventarioItemConsolidadoDTO> itens = inventarioItemService.obterItensConsolidado(id);
        return this.gerarRelatorio("inventario_consolidado", entity, itens);
    }

    private ArquivoBase64DTO gerarRelatorio(String reportName, Inventario entity, List<?> itens) {
        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("UNIDADE_DISPENSADORA", entity.getUnidadeDispensadora().getDescricao());
        parameters.put("DESCRICAO_INVENTARIO", entity.getDescricao());
        parameters.put("DATA_CADASTRO", entity.getDataCadastro());
        // @todo verificar se usuário logado tem perfil admin
        parameters.put("IS_ADMIN", true);

        ByteArrayOutputStream outputStream = reportService.exportToXLSX(
            "classpath:/jasperreports/" + reportName + ".jasper",
            parameters,
            new JRBeanCollectionDataSource(itens)
        );

        return new ArquivoBase64DTO(
            Base64.getEncoder().encodeToString(outputStream.toByteArray()),
            "relatorio_" + reportName + ".xlsx",
            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
        );
	}
}
