package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.ItemDTO;
import br.gov.datasus.scodes.model.dto.filter.ItemFilterDTO;
import br.gov.datasus.scodes.model.entity.*;
import br.gov.datasus.scodes.model.mapper.ItemMapper;
import br.gov.datasus.scodes.repository.EspecificacaoRepository;
import br.gov.datasus.scodes.repository.ItemRepository;
import br.gov.datasus.scodes.repository.MarcaRepository;
import lombok.RequiredArgsConstructor;
import org.hibernate.exception.ConstraintViolationException;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class ItemService {

    private final ItemRepository repository;
    private final ItemMapper mapper;
    private final MarcaRepository marcaRepository;
    private final EspecificacaoRepository especificacaoRepository;

    @PersistenceContext
    private final EntityManager entityManager;

    public ItemDTO obterPorId(Long id) {
        Item entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public PageableResponseModel<ItemDTO> obterTodosPaginado(PageableRequestModel<ItemFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "item");
        DynamicParameters parameters = DynamicParameters.get();

        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getIdProduto())
                .ifPresent(idProduto -> parameters.append("idProduto", idProduto));
            Optional.ofNullable(filter.getIdEspecificacao())
                .ifPresent(idEspecificacao -> parameters.append("idEspecificacao", idEspecificacao));
            Optional.ofNullable(filter.getIdCategoria())
                .ifPresent(idCategoria -> parameters.append("idCategoria", idCategoria));
            Optional.ofNullable(filter.getDescricao())
                .ifPresent(descricao -> parameters.appendLike("descricao", descricao));
            Optional.ofNullable(filter.getCodigo())
                .ifPresent(codigo -> parameters.appendLike("codigo", codigo));
        });

        PageableResponseModel<Item> pageableResponseModel = repository.findAll(Item.OBTER_TODOS_PAGINADO, parameters, pagination, Item.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    public List<ItemDTO> obterSimilares(Long id) {
        Item entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity.getSimilares());
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public ItemDTO incluir(ItemDTO dto) {
        Item entity = mapper.toEntity(dto);
        return mapper.toDto(save(entity));
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public ItemDTO alterar(Long id, ItemDTO dto) {
        Item entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        dto.setId(id);
        mapper.fromDto(dto, entity);
        return mapper.toDto(save(entity));
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void alterarSimilares(Long id, Set<Long> idSimilares) {
        Item entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        if (idSimilares != null) {
            Set<Item> itensSimiliares = idSimilares.stream()
                .map((idSimilar) -> {
                    Item itemSimiliar = new Item();
                    itemSimiliar.setId(idSimilar);
                    return itemSimiliar;
                }).collect(Collectors.toSet());
            entity.setSimilares(itensSimiliares);
        };
        repository.save(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void excluir(Long id) {
        Item entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        try {
            repository.delete(entity);
            entityManager.flush();
        } catch (PersistenceException e) {
            if (e.getCause() instanceof ConstraintViolationException) {
                throw new BusinessException(MessageProperty.ITEM_EXCLUSAO_VINCULO);
            }
            throw e;
        }
    }

    private Item save(Item item) {
        Especificacao especificacao = especificacaoRepository.findById(item.getEspecificacao().getId())
            .orElseThrow(() -> new BusinessException(MessageProperty.ESPECIFICACAO_NAO_ENCONTRADA));
        item.setEspecificacao(especificacao);
        if (item.getMarca() != null && item.getMarca().getId() == null) {
            item.setMarca(novaMarca(item));
        }
        item.setCodigo(novoCodigo(item));
        return repository.save(item);
    }

    private String novoCodigo(Item item) {
        try {
            StringBuilder sb = new StringBuilder();
            Especificacao especificacao = item.getEspecificacao();
            Produto produto = especificacao.getProduto();
            // Categoria
            sb.append(produto.getCategoria().getId());
            // Código do Grupo Anatômico
            sb.append(especificacao.getGrupoEspecificacao().getCodigoGrupoAnatomico());
            // Produto
            sb.append(String.format("%05d", produto.getId()));
            // Apresentação
            sb.append(String.format("%02d", especificacao.getApresentacao().getId()));
            // Especificação
            sb.append(String.format("%02d", especificacao.getId()));
            // Tipo Medicamento
            sb.append(String.format("%02d", item.getTipoItem().getId()));
            // Marca
            if (item.getMarca() != null) {
                sb.append(String.format("%02d", item.getMarca().getId()));
            }
            // Novo Código
            return sb.toString();
        } catch (Throwable e) {
            throw new BusinessException(MessageProperty.ERRO_AO_GERAR_NOVO_CODIGO_ITEM, e);
        }
    }

    private Marca novaMarca(Item item) {
        Marca marca = new Marca();
        marca.setNome(item.getMarca().getNome());
        return marcaRepository.save(marca);
    }

    public List<DominioItemModel> obterAutocompletePorCodigo(String codigo) {
        return repository.obterAutocompletePorCodigo(codigo);
    }
}
