package br.gov.datasus.scodes.service;

import java.util.Collections;
import java.util.Optional;

import org.apache.commons.lang3.BooleanUtils;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.ModeloDTO;
import br.gov.datasus.scodes.model.dto.filter.ModeloFilterDTO;
import br.gov.datasus.scodes.model.entity.Modelo;
import br.gov.datasus.scodes.model.mapper.ModeloMapper;
import br.gov.datasus.scodes.repository.ModeloRepository;
import br.gov.datasus.scodes.repository.TipoDemandaRepository;
import br.gov.datasus.scodes.repository.TipoModeloRepository;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class ModeloService {

    private final TipoDemandaRepository tipoDemandaRepository;
    private final TipoModeloRepository tipoModeloRepository;
    private final ModeloRepository repository;
    private final ModeloMapper mapper;

    public ModeloDTO obterPorId(final Long id) {
        final Optional<Modelo> one = repository.findById(id);
        final Modelo entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public PageableResponseModel<ModeloDTO> obterTodosPaginado(final PageableRequestModel<ModeloFilterDTO> pageableModel) {
        final Pageable pagination = PaginationHelper.toPageable(pageableModel, "modelo");
        final DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getTipoModelo()).map(tipoModelo -> tipoModelo.getId())
                .ifPresent(tipoModeloId -> parameters.append("tipoModeloId", tipoModeloId));

            Optional.ofNullable(filter.getTipoDemanda()).map(tipoDemanda -> tipoDemanda.getId())
                .ifPresent(tipoDemandaId -> parameters.append("tipoDemandaId", tipoDemandaId));

            Optional.ofNullable(filter.getDescritor())
                .ifPresent(descritor -> parameters.appendLike("descritor", descritor));

            Optional.ofNullable(filter.getSituacaoInativo())
                .ifPresent(situacaoInativo -> parameters.append("situacaoInativo", BooleanUtils.isNotFalse(situacaoInativo)));
        });
        final PageableResponseModel<Modelo> pageableResponseModel = repository.findAll(
            Modelo.OBTER_TODOS_PAGINADO, parameters, pagination, Modelo.class
        );
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public ModeloDTO incluir(final ModeloDTO dto) {
        final Modelo entity = mapper.toEntity(dto);
        return mapper.toDto(validateAndSave(entity));
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public ModeloDTO alterar(final Long id, final ModeloDTO dto) {
        final Optional<Modelo> one = repository.findById(id);
        final Modelo entity = one.orElseThrow(() -> new ResourceNotFoundException());
        dto.setId(id);
        mapper.fromDto(dto, entity);
        return mapper.toDto(validateAndSave(entity));
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void excluir(final Long id) {
        final Optional<Modelo> one = repository.findById(id);
        final Modelo entity = one.orElseThrow(() -> new ResourceNotFoundException());
        repository.delete(entity);
    }

    private Modelo validateAndSave(final Modelo entity) {
        entity.setTipoDemanda(
            tipoDemandaRepository.findById(entity.getTipoDemanda().getId())
                .orElseThrow(() -> new BusinessException(MessageProperty.TIPO_DEMANDA_NAO_ENCONTRADA))
        );
        entity.setTipoModelo(
            tipoModeloRepository.findById(entity.getTipoModelo().getId())
                .orElseThrow(() -> new BusinessException(MessageProperty.TIPO_MODELO_NAO_ENCONTRADO))
        );
        Boolean tipoModeloTipoDemandaInvalid = Optional
            .ofNullable(entity.getTipoModelo().getTipoDemandas())
            .orElse(Collections.emptySet())
            .stream()
            .filter(tipoDemanda -> tipoDemanda.equals(entity.getTipoDemanda()))
            .count() == 0;
        if (tipoModeloTipoDemandaInvalid) {
            throw new BusinessException(MessageProperty.TIPO_DEMANDA_INCOPATIVEL_COM_TIPO_MODELO);
        }
        return repository.save(entity);
    }
}
