package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.repository.ProdutoRepository;
import br.gov.datasus.scodes.model.dto.filter.ProdutoFilterDTO;
import br.gov.datasus.scodes.model.mapper.ProdutoMapper;
import br.gov.datasus.scodes.model.dto.ProdutoDTO;
import br.gov.datasus.scodes.model.entity.Produto;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.annotation.Propagation;
import lombok.RequiredArgsConstructor;
import java.util.List;
import java.util.Optional;

import org.springframework.data.domain.Pageable;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class ProdutoService {

    private final ProdutoRepository repository;
    private final ProdutoMapper mapper;

    public ProdutoDTO obterPorId(Long id) {
        Produto entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<ProdutoDTO> obterTodos() {
        Iterable<Produto> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<ProdutoDTO> obterTodosPaginado(
        PageableRequestModel<ProdutoFilterDTO> pageableModel
    ) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "prod");
        final DynamicParameters parameters = DynamicParameters.get();

        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getId()).ifPresent(
                id -> parameters.append("id", id)
            );
            Optional.ofNullable(filter.getNome()).ifPresent(
                nome -> parameters.appendLike("nome", nome)
            );
            Optional.ofNullable(filter.getIdCategoria()).ifPresent(
                idCategoria -> parameters.append("idCategoria", idCategoria)
            );
            Optional.ofNullable(filter.getIdEspecificacao()).ifPresent(
                idEspecificacao -> parameters.append("idEspecificacao", idEspecificacao)
            );
            Optional.ofNullable(filter.getIdGrupoEspecificacao()).ifPresent(
                idGrupoEspecificacao -> parameters.append("idGrupoEspecificacao", idGrupoEspecificacao)
            );
            Optional.ofNullable(filter.getCodigoItem()).ifPresent(
                codigoItem -> parameters.append("codigoItem", codigoItem)
            );
        });
        PageableResponseModel<Produto> pageableResponseModel = repository.findAll(Produto.OBTER_TODOS_PAGINADO, parameters, pagination, Produto.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public ProdutoDTO incluir(ProdutoDTO dto) {
        Produto entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public ProdutoDTO alterar(Long id, ProdutoDTO dto) {
        Produto entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        dto.setId(id);
        mapper.fromDto(dto, entity);
        return mapper.toDto(repository.save(entity));
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void excluir(Long id) {
        Produto entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        if (repository.countVinculosEspecificacoes(entity) > 0) {
            throw new BusinessException(MessageProperty.PRODUTO_EXCLUSAO_VINCULO);
        }
        repository.delete(entity);
    }

    public List<DominioItemModel> autocomplete(String termoProduto, Long idCategoria) {
        if (idCategoria != null) {
            return repository.autocomplete(termoProduto, idCategoria);
        }
        return repository.autocomplete(termoProduto);
    }
}
