package br.gov.datasus.scodes.service;

import java.util.List;
import java.util.Optional;

import br.gov.datasus.scodes.model.dto.UfDTO;
import br.gov.datasus.scodes.model.entity.Uf;
import br.gov.datasus.scodes.model.mapper.UfMapper;
import br.gov.datasus.scodes.util.ParameterUtil;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.ProfissionalDTO;
import br.gov.datasus.scodes.model.dto.filter.ProfissionalFilterDTO;
import br.gov.datasus.scodes.model.entity.Profissional;
import br.gov.datasus.scodes.model.mapper.ProfissionalMapper;
import br.gov.datasus.scodes.repository.ProfissionalRepository;
import br.gov.datasus.scodes.repository.TipoProfissaoRepository;
import br.gov.datasus.scodes.repository.UfRepository;
import lombok.RequiredArgsConstructor;

import static java.util.Optional.ofNullable;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class ProfissionalService {

    private final ProfissionalRepository repository;
    private final UfRepository ufRepository;
    private final TipoProfissaoRepository tipoProfissaoRepository;

    private final ProfissionalMapper mapper;
    private final UfMapper ufMapper;
    private final ParameterUtil parameterUtil;

    public ProfissionalDTO obterPorId(Long id) {
        Optional<Profissional> one = repository.findById(id);
        Profissional entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<ProfissionalDTO> obterTodos() {
        Iterable<Profissional> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<ProfissionalDTO> pesquisarPaginado(PageableRequestModel<ProfissionalFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "Profissional");
        final DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            ofNullable(filter.getProfissional().getNome()).ifPresent(nome -> parameters.appendLike("nome", nome));
            ofNullable(filter.getConselho()).ifPresent(conselho -> parameters.append("conselho", conselho));
            ofNullable(filter.getTipoProfissao()).ifPresent(tipoProfissao -> parameters.append("tipoProfissaoId", tipoProfissao.getId()));
            ofNullable(filter.getUf()).ifPresent(uf -> parameters.append("ufId", uf.getId()));
            ofNullable(filter.getEmail()).ifPresent(email -> parameters.append("email", email));
        });

        final PageableResponseModel<Profissional> pageableResponseModel = repository.findAll(Profissional.OBTER_TODOS_PAGINADO, parameters, pagination, Profissional.class);

        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public ProfissionalDTO incluir(final ProfissionalDTO dto) {
        final Profissional entity = mapper.toEntity(dto);
        return mapper.toDto(validateAndSave(entity));
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public ProfissionalDTO alterar(final Long id, final ProfissionalDTO dto) {
        final Optional<Profissional> one = repository.findById(id);
        final Profissional entity = one.orElseThrow(() -> new ResourceNotFoundException());
        mapper.fromDto(dto, entity);
        return mapper.toDto(validateAndSave(entity));
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void excluir(final Long id) {
        final Optional<Profissional> one = repository.findById(id);
        final Profissional entity = one.orElseThrow(() -> new ResourceNotFoundException());
        repository.delete(entity);
    }

    private Profissional validateAndSave(final Profissional profissional) {
        ofNullable(profissional.getStatus()).orElseThrow(() -> new BusinessException(MessageProperty.STATUS_OBRIGATORIO));
        ofNullable(profissional.getNome()).orElseThrow(() -> new BusinessException(MessageProperty.NOME_OBRIGATORIO));
        ofNullable(profissional.getTipoProfissao()).orElseThrow(() -> new BusinessException(MessageProperty.TIPO_PROFISSAO_OBRIGATORIO));
        ofNullable(profissional.getUf()).orElseThrow(() -> new BusinessException(MessageProperty.UF_OBRIGATORIO));

        profissional.setTipoProfissao(
                tipoProfissaoRepository.findById(profissional.getTipoProfissao().getId())
                .orElseThrow(() -> new BusinessException(MessageProperty.TIPO_PROFISSAO_OBRIGATORIO))
                );
        profissional.setUf(
                ufRepository.findById(profissional.getUf().getId())
                .orElseThrow(() -> new BusinessException(MessageProperty.UF_OBRIGATORIO))
                );

        return repository.save(profissional);
    }

    public List<ProfissionalDTO> pesquisarPatrono(final Integer tipoProfissaoId, final String uf, final String oab, final String nome) {
        final DynamicParameters parameters = DynamicParameters.get();
        ofNullable(tipoProfissaoId).ifPresent(tipoProfissao -> parameters.append("tipoProfissaoId", tipoProfissao));
        ofNullable(uf).ifPresent(ufId -> parameters.append("ufId", ufId));
        ofNullable(nome).ifPresent(nomeLike -> parameters.appendLike("nome", nomeLike));
        ofNullable(oab).ifPresent(conselho -> parameters.appendLike("conselho", conselho));

        final List<Profissional> profissionais = repository.findAll(Profissional.PESQUISAR_PATRONO, parameters,  Profissional.class);

        return mapper.toDto(profissionais);
    }

    public List<ProfissionalDTO> pesquisarPatrono(ProfissionalFilterDTO dtoFilter) {

        final DynamicParameters parameters = DynamicParameters.get();

        ofNullable(dtoFilter).ifPresent(filter -> {
            parameterUtil.append(parameters, filter.getTipoProfissao().getId(), "tipoProfissaoId" );
            parameterUtil.append(parameters, filter.getUf().getId(), "ufId");
            parameterUtil.appendLike(parameters, ofNullable(filter.getProfissional()).map(ProfissionalDTO::getNome).orElse(null), "nome");
            parameterUtil.appendLike(parameters, filter.getConselho(), "conselho");
        });

        final List<Profissional> profissionais = repository.findAll(Profissional.PESQUISAR_PATRONO, parameters,  Profissional.class);

        return mapper.toDto(profissionais);
    }

    public List<UfDTO> obterUfsProfissional() {
        return ufMapper.toDto(repository.obterUfsProfissional());
    }

}
