import { Component, Inject, LOCALE_ID, OnInit } from '@angular/core';
import { PageEvent } from '@angular/material/paginator';

import { download, ErrorModel, PageableResponseModel, requestItemsParams, stringToBlob } from '@datasus/core';
import { DialogService, LayoutService } from '@datasus/layout';

import { Observable } from 'rxjs';
import { finalize } from 'rxjs/operators';

import { AcessoService } from '../../services/acesso.service';
import { AcessoModel } from '../../models/acesso.model';
import { formatDate } from '@angular/common';
import { MensagemEnum } from 'src/app/shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-log-acesso',
  templateUrl: './log-acesso.component.html',
  styleUrls: ['./log-acesso.component.scss']
})
export class LogAcessoComponent implements OnInit {

  loading = false;

  total = 0;
  pageSize = 30;

  items: AcessoModel[] = [];

  displayedColumns: string[] = [
    'nomeLogin',
    'codigoSessao',
    'ipCliente',
    'ipServidor',
    'dataEntrada',
    'dataSaida',
    'descricaoFalha'
  ];

  onMobile$: Observable<boolean>;

  constructor(
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private acessoService: AcessoService,
    @Inject(LOCALE_ID) private locale: string
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Log de Acessos');
    this.onMobile$ = this.layoutService.onMobile();
    this.requestItems();
  }

  requestItems(event?: PageEvent): void {
    this.loading = true;

    this.acessoService.obterTodosPaginado({
      filter: null,
      ...requestItemsParams({
        to: {pageSize: this.pageSize},
        withPageEvent: event,
        changingPageSizeDefaultTo: this.pageSize,
        withSort: {active: 'id', direction: 'desc'},
      })
    }).pipe(
      finalize(() => this.loading = false)
    ).subscribe(
      (page: PageableResponseModel<AcessoModel>) => {
        this.total = page.total;
        this.items = page.items;

        if (!this.items || !this.items.length) {
          return this.dialogService.showNotification(MensagemEnum.NO_RESULTS_FOUND);
        }
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  gerarRelatorio(): void {
    const csvArray: string[] = [];

    const titles = ['Usuário', 'Sessão', 'IP Cliente', 'IP Servidor', 'Entrada', 'Saída', 'Falha'];
    csvArray.push(titles.map(v => `"${v}"`).join(';'));

    if (this.items && this.items.length) {
      this.items.forEach((item: AcessoModel) => {
        const row = [
          item.nomeLogin,
          item.codigoSessao,
          item.ipCliente || '',
          item.ipServidor || '',
          formatDate(item.dataEntrada, 'dd/MM/yyyy HH:mm:ss', this.locale),
          item.dataSaida ? formatDate(item.dataSaida, 'dd/MM/yyyy HH:mm:ss', this.locale) : '',
          item.descricaoFalha || '',
        ];
        csvArray.push(row.map(v => `"${v.replace(/"/g, '""')}"`).join(';'));
      });
    }

    const csvStr = csvArray.join('\n');

    download(stringToBlob(csvStr, 'text/csv'), 'relatorio_ultimos_acessos.csv');
  }

}
