import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';

import { Observable } from 'rxjs';
import { finalize } from 'rxjs/operators';

import { DialogService, LayoutService } from '@datasus/layout';
import { base64DecodeToBlob, download, ErrorModel } from '@datasus/core';

import { AcessoService } from '../../services/acesso.service';
import { AcessoFilterModel } from '../../models/acesso-filter.model';
import { ArquivoBase64Model } from '../../../../shared/models/arquivo-base64.model';
import { MensagemEnum } from '../../../../shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-relatorio-acesso',
  templateUrl: './relatorio-acesso.component.html',
})
export class RelatorioAcessoComponent implements OnInit {

  filter: AcessoFilterModel;
  formGroup: FormGroup;

  processing = false;

  onMobile$: Observable<boolean>;

  constructor(
    private formBuilder: FormBuilder,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private acessoService: AcessoService
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Relatório de Acessos');

    this.onMobile$ = this.layoutService.onMobile();

    this.formGroup = this.formBuilder.group({
      nomeLogin: [null],
      nomeUsuario: [null],
      dataEntrada: [null, Validators.compose([Validators.required])],
      dataSaida: [null, Validators.compose([Validators.required])],
    });
  }

  gerarRelatorio(event: Event): void {
    event.preventDefault();

    if (this.formGroup.invalid) {
      return this.dialogService.showNotification(MensagemEnum.FILTERS_REQUIRED);
    }

    this.filter = this.formGroup.value;

    this.processing = true;
    this.acessoService.relatorio(this.filter).pipe(
      finalize(() => this.processing = false)
    ).subscribe(
      (arquivo: ArquivoBase64Model) => download(base64DecodeToBlob(arquivo.base64, arquivo.mimeType), arquivo.filename),
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED : '';
  }

}
