import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { SelectionModel } from '@angular/cdk/collections';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { Observable } from 'rxjs';
import { finalize} from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';

import { GrupoModel } from '../../models/grupo.model';
import { GrupoService } from '../../services/grupo.service';
import { PerfilModel } from '../../../perfil/models/perfil.model';
import {MensagemEnum} from '../../../../shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-grupo-form',
  templateUrl: './grupo.form.component.html',
  styleUrls: ['./grupo.form.component.scss'],
})
export class GrupoFormComponent implements OnInit {

  public grupo: GrupoModel;
  public formGroup: FormGroup;

  public saving: boolean;

  public perfis: PerfilModel[];
  public perfisColumns: string[];

  public selection: SelectionModel<PerfilModel>;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private grupoService: GrupoService,
  ) { }

  public ngOnInit(): void {
    this.layoutService.setTitle('Cadastro de Grupo');

    this.grupo = {} as GrupoModel;

    this.formGroup = this.formBuilder.group({
      descricao: [
        null,
        Validators.compose([
          Validators.required,
        ])
      ],
      nome: [
        null,
        Validators.compose([
          Validators.required,
        ])
      ],
      situacaoAtivo: [
        true,
        Validators.compose([
          Validators.required,
        ])
      ],
    });

    this.perfisColumns = ['nome', 'action'];
    this.perfis = this.route.snapshot.data.perfis;

    this.selection = new SelectionModel<PerfilModel>(true, []);

    if (this.route.snapshot.data.item) {
      this.editItem(this.route.snapshot.data.item);
    }
  }

  public editItem(item: GrupoModel): void {
    this.grupo = item;
    this.layoutService.setTitle('Alterar Grupos de Acesso');
    this.formGroup.get('nome').setValue(this.grupo.nome);
    this.formGroup.get('descricao').setValue(this.grupo.descricao);
    this.formGroup.get('situacaoAtivo').setValue(this.grupo.situacaoAtivo);

    if (this.grupo.perfis) {
      this.selection.select(...this.perfis.filter(perfil => this.grupo.perfis.find(v => v.id === perfil.id)));
    }
  }

  public save(event: Event): void {
    event.preventDefault();

    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }

    this.saving = true;

    this.request().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate(['/controle-acesso/grupo', 'listar']);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  public isAllChecked() {
    const numSelected = this.selection.selected.length;
    const numRows = this.perfis.length;
    return numSelected === numRows;
  }

  public toogleAll() {
    this.isAllChecked() ?
      this.selection.clear() :
      this.perfis.forEach(row => this.selection.select(row));
  }

  public checkboxLabel(perfil?: PerfilModel): string {
    if (!perfil) {
      return `${this.isAllChecked() ? 'marcar' : 'desmarcar'} todos`;
    }
    return `${this.selection.isSelected(perfil) ? 'desmarcar' : 'marcar'} perfil ${perfil.nome}`;
  }

  public resetForm(event: MouseEvent) {
    event.preventDefault();
    this.formGroup.get('nome').setValue(null);
    this.formGroup.get('descricao').setValue(null);
    this.formGroup.get('situacaoAtivo').setValue(false);
    this.selection.clear();
  }

  private request(): Observable<GrupoModel> {
    if (this.grupo.id) {
      return this.grupoService.alterar(<GrupoModel>{
        ...this.formGroup.value,
        perfis: this.selection.selected,
      }, this.grupo.id);
    }
    return this.grupoService.incluir(this.formGroup.value);
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
            '';
  }

}
