import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Observable } from 'rxjs';
import { finalize } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';

import { PerfilModel } from '../../models/perfil.model';
import { PerfilService } from '../../services/perfil.service';
import { AreaService } from '../../../../shared/services/area.service';
import { PerfilFuncionalidadeModel } from '../../models/perfil-funcionalidade.model';
import { TreeFuncionalidadeComponent } from './tree-funcionalidade/tree-funcionalidade.component';
import { AreaModel } from 'src/app/shared/models/area.model';
import {MensagemEnum} from '../../../../shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-perfil-form',
  templateUrl: './perfil.form.component.html',
  styleUrls: ['./perfil.form.component.scss'],
})
export class PerfilFormComponent implements OnInit {

  public formGroup: FormGroup;
  public saving: boolean;
  public perfil: PerfilModel;

  @ViewChild('treeFuncionalidade')
  public treeFuncionalidade: TreeFuncionalidadeComponent;

  public areas: AreaModel[];

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private perfilService: PerfilService,
    private areaService: AreaService
  ) { }

  public ngOnInit(): void {
    this.layoutService.setTitle('Cadastro de Perfil');

    this.perfil = {} as PerfilModel;

    this.formGroup = this.formBuilder.group({
      descricao: [
        null,
        Validators.compose([
          Validators.required,
        ])
      ],
      nome: [
        null,
        Validators.compose([
          Validators.required,
        ])
      ],
      situacaoAtivo: [
        true,
        Validators.compose([
          Validators.required,
        ])
      ],
      area: [
        null,
        Validators.compose([
          Validators.required,
        ])
      ],
      demandaAdministrativa: [
        false,
        Validators.compose([
          Validators.required,
        ])
      ],
      demandaJudicial: [
        false,
        Validators.compose([
          Validators.required,
        ])
      ],
    });

    if (this.route.snapshot.data.item) {
      this.editItem(this.route.snapshot.data.item);
    }

    this.areaService.obterTodos().subscribe(data => this.areas = data);
  }

  public editItem(item: PerfilModel): void {
    this.perfil = item;

    this.layoutService.setTitle('Alterar Perfil de Acesso');

    this.formGroup.get('descricao').setValue(this.perfil.descricao);
    this.formGroup.get('nome').setValue(this.perfil.nome);
    this.formGroup.get('situacaoAtivo').setValue(this.perfil.situacaoAtivo);
    this.formGroup.get('area').setValue(this.perfil.area);
    this.formGroup.get('demandaAdministrativa').setValue(this.perfil.demandaAdministrativa);
    this.formGroup.get('demandaJudicial').setValue(this.perfil.demandaJudicial);
  }

  public save(event: Event): void {
    event.preventDefault();

    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }

    this.saving = true;

    this.request().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate(['/controle-acesso/perfil', 'listar']);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  public resetForm(event: MouseEvent) {
    event.preventDefault();

    this.formGroup.get('descricao').setValue(null);
    this.formGroup.get('nome').setValue(null);
    this.formGroup.get('situacaoAtivo').setValue(false);
    this.formGroup.get('area').setValue(null);
    this.formGroup.get('demandaAdministrativa').setValue(false);
    this.formGroup.get('demandaJudicial').setValue(false);
    this.treeFuncionalidade.clearChecked();
  }

  public compareArea(a1: AreaModel, a2: AreaModel) {
    return (a1 && a2 && a1.id === a2.id);
  }

  private request(): Observable<PerfilModel> {
    if (this.perfil.id) {
      return this.perfilService.alterar(<PerfilModel>{
        ...this.formGroup.value,
        perfilFuncionalidades: this.perfil.perfilFuncionalidades
      }, this.perfil.id);
    }
    return this.perfilService.incluir(this.formGroup.value);
  }

  public onChangePerfilFuncionalidade(changedPerfilFuncionalidade: PerfilFuncionalidadeModel) {
    if (!Array.isArray(this.perfil.perfilFuncionalidades)) {
      this.perfil.perfilFuncionalidades = [];
    }

    const index = this.perfil.perfilFuncionalidades.findIndex(perfilFuncionalidade =>
      perfilFuncionalidade.funcionalidade.id === changedPerfilFuncionalidade.funcionalidade.id);
    if (index !== -1) {
      this.perfil.perfilFuncionalidades[index] = changedPerfilFuncionalidade;
    } else {
      this.perfil.perfilFuncionalidades.push(changedPerfilFuncionalidade);
    }
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      '';
  }
}
