import { Component, OnInit, ViewChild, OnDestroy, ChangeDetectorRef } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { Router } from '@angular/router';
import { PageEvent, MatPaginator } from '@angular/material/paginator';
import { MatTabGroup } from '@angular/material';
import { ScrollDispatcher, CdkScrollable } from '@angular/cdk/overlay';
import { Observable, Subscription } from 'rxjs';
import { tap, finalize } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { requestItemsParams, PageableResponseModel, ErrorModel } from '@datasus/core';

import { MensagemEnum } from 'src/app/shared/enums/mensagem.enum';

import { UsuarioLocalFilterModel } from '../../models/usuario-local-filter.model';
import { UsuarioLocalService } from '../../services/usuario-local.service';
import { UsuarioLocalModel } from '../../models/usuario-local.model';

@Component({
  selector: 'scodes-usuario-list',
  templateUrl: './usuario.list.component.html',
  styleUrls: ['./usuario.list.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class UsuarioListComponent implements OnInit, OnDestroy {

  @ViewChild('paginator')
  public paginator: MatPaginator;

  public filtro: UsuarioLocalFilterModel;

  public items: UsuarioLocalModel[] = [];
  public infinityItems: UsuarioLocalModel[] = [];

  public displayedColumns: string[];

  public onMobile$: Observable<boolean>;

  public formGroup: FormGroup;

  public loading = false;
  public infinityItemsMessage = '';
  public total = 0;
  public pageSize = 10;

  public actionButtonCtrl: Map<UsuarioLocalModel, boolean> = new Map();

  private scrollDispatcherSubscription: Subscription;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private scrollDispatcher: ScrollDispatcher,
    private changeDetectorRef: ChangeDetectorRef,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private usuarioLocalService: UsuarioLocalService,
  ) { }

  public ngOnInit(): void {
    this.layoutService.setTitle('Listagem de Usuários');

    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );

    this.displayedColumns = [
      'nomeLogin',
      'actions',
    ];

    this.formGroup = this.formBuilder.group({
      nomeLogin: [null],
    });
  }

  public ngOnDestroy(): void {
    this.scrollRegister(false);
    this.infinityItems = [];
  }

  public search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();

    if (!this.formGroup.valid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }

    if (tabs) {
      tabs.selectedIndex = 1;
    }

    this.infinityItems = [];
    this.requestItems();
  }

  public requestItems(event?: PageEvent): void {
    this.filtro = this.formGroup.value;
    this.loading = true;
    this.infinityItemsMessage = '';

    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }

    this.usuarioLocalService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({
        to: {pageSize: this.pageSize},
        changingPageSizeDefaultTo: this.pageSize,
        withPageEvent: event,
        withSort: {active: 'nomeLogin', direction: 'asc'},
      }),
    }).pipe(
      finalize(() => this.loading = false)
    ).subscribe(
      (page: PageableResponseModel<UsuarioLocalModel>) => {
        this.total = page.total;
        this.items = page.items;

        if (!this.items) {
          return this.dialogService.showNotification(MensagemEnum.NO_RESULTS_FOUND);
        }

        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  public edit(item: UsuarioLocalModel) {
    this.router.navigate(['/controle-acesso/usuario', 'editar', item.id]);
  }

  public delete(item: UsuarioLocalModel) {
    const buttons: Map<string, () => void> = new Map();
    buttons.set('Sim', () => {
      this.layoutService.loading();
      this.usuarioLocalService.excluir(item.id).pipe(
        finalize(() => this.layoutService.loaded()),
      ).subscribe(
        () => {
          this.dialogService.showNotification(MensagemEnum.REGISTRO_EXCLUIDO);
          this.requestItems();
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
    });
    buttons.set('Não', () => {});

    this.dialogService.showConfirmation({
      question: MensagemEnum.CONFIRMA_REMOCAO,
      title: 'Cuidado!',
      anwsers: buttons,
      type: 'warn',
    });
  }

  private scrollRegister(register: boolean): void {
    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }
}
