import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators, ValidationErrors, ValidatorFn, FormGroupDirective } from '@angular/forms';

import { Observable } from 'rxjs';
import { finalize } from 'rxjs/operators';

import { DialogService, LayoutService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';
import { DadosPessoaisService } from '../../services/dados-pessoais.service';
import { MensagemEnum } from '../../../shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-alteracao-senha',
  templateUrl: './alteracao-senha.component.html',
})
export class AlteracaoSenhaComponent implements OnInit {

  formGroup: FormGroup;
  saving = false;

  hideSenhaAtual = true;
  hideNovaSenha = true;
  hideConfirmaSenha = true;

  onMobile$: Observable<boolean>;

  constructor(
    private formBuilder: FormBuilder,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private dadosPessoaisService: DadosPessoaisService
  ) {
  }

  ngOnInit(): void {
    this.layoutService.setTitle('Alteração de Senha');

    this.onMobile$ = this.layoutService.onMobile();

    this.formGroup = this.formBuilder.group({
      senhaAtual: [
        null,
        Validators.compose([
          Validators.required,
        ])
      ],
      novaSenha: [
        null,
        Validators.compose([
          Validators.required,
          Validators.minLength(5),
        ])
      ],
      confirmaSenha: [
        null,
        Validators.compose([
          Validators.required,
        ])
      ]
    }, {
      validators: Validators.compose([
        this.confirmPasswordValidator('novaSenha', 'confirmaSenha'),
      ])
    });
  }

  save(event: Event, form: FormGroupDirective): void {
    event.preventDefault();

    if (this.formGroup.invalid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }

    const senhaAtual = this.formGroup.get('senhaAtual').value;
    const novaSenha = this.formGroup.get('novaSenha').value;

    this.saving = true;
    this.dadosPessoaisService.alterarSenha(senhaAtual, novaSenha).pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification(MensagemEnum.SENHA_ALTERADA);
        form.resetForm();
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      fc.hasError('minlength') ? MensagemEnum._(MensagemEnum.MIN_LENGTH, fc.getError('minlength')) :
      fc.hasError('confirmPassword') ? MensagemEnum.CONFIRM_PASSWORD :
      '';
  }

  private confirmPasswordValidator(controlName: string, matchingControlName: string): ValidatorFn | null {
    return (formGroup: FormGroup): ValidationErrors | null => {
      const control = formGroup.get(controlName);
      const matchingControl = formGroup.get(matchingControlName);

      if (matchingControl.errors && !matchingControl.errors.confirmPassword) {
        return null;
      }

      if (control.value !== matchingControl.value) {
        matchingControl.setErrors({confirmPassword: true});
      } else {
        matchingControl.setErrors(null);
      }

      return null;
    };
  }

}
