import {
  Component,
  OnInit,
  Input,
  ChangeDetectionStrategy,
  Output,
  ViewChild,
  ElementRef,
  EventEmitter
} from '@angular/core';
import { FormGroup } from '@angular/forms';
import { finalize, tap, switchMap, debounceTime } from 'rxjs/operators';
import { CidadeModel } from 'src/app/shared/models/cidade.model';
import { Observable } from 'rxjs';
import { MatAutocompleteSelectedEvent } from '@angular/material';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';

import { UfService } from 'src/app/shared/services/uf.service';
import { CidadeService } from 'src/app/shared/services/cidade.service';
import { DemandasService } from '../../services/demandas.service';
import { AutorModel } from 'src/app/shared/models/autor.model';
import { UfModel } from 'src/app/shared/models/uf.model';

@Component({
  selector: 'scodes-autor',
  templateUrl: './autor.component.html',
  styleUrls: ['./autor.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AutorComponent implements OnInit {

  @ViewChild('submitButton') submitButton: ElementRef;

  @Input() parentForm: FormGroup;
  @Input() dataCadastro: Date;
  @Input() cepLoading: boolean;
  @Input() loadingCpf: boolean;

  idade: number;

  // Viewport Observable
  onMobile$: Observable<boolean>;

  listaUfs: UfModel[];
  listaCidades: CidadeModel[];

  listaSexos = [
    { label: 'Masculino', value: 'M' },
    { label: 'Feminino', value: 'F' }
  ];

  @Output() autorId: EventEmitter<number> = new EventEmitter();


  constructor(
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private ufService: UfService,
    private cidadeService: CidadeService,
    private demandaService: DemandasService,
  ) { }

  ngOnInit() {

    this.onMobile$ = this.layoutService.onMobile();

    this.ufService.obterTodos()
      .subscribe(
        (response) => this.listaUfs = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );

    this.parentForm.get('estado').valueChanges
      .pipe(
        tap(() => this.parentForm.get('cidade').setValue(null)),
        switchMap((uf: UfModel) => this.cidadeService.obterCidadesPorUf(uf.id)),
      ).subscribe(
        (resp: any) => this.listaCidades = resp,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );

    this.parentForm.get('dataNascimento').valueChanges
      .subscribe((data) => {
        this.idade = null;
        if (data !== null) {
          this.obterAnos(data);
        }
      });

    this.parentForm.get('cidade').valueChanges
      .pipe(debounceTime(350))
      .subscribe(
        (cidade) => {
          if (typeof cidade === 'string') {
            return this.cidadeService.obterCidadesPorNome(cidade, this.parentForm.get('estado').value.id)
              .subscribe(
                (response) => this.listaCidades = response,
                (error: ErrorModel) => this.dialogService.showNotification(error.message));
          }
        });
  }

  obterCidades(event: MatAutocompleteSelectedEvent) {
    if (event.option.value && event.option.value.descricao) {
      this.cidadeService
        .obterPorId(event.option.value.id)
        .subscribe(
          (response) => this.parentForm.get('cidade').setValue(response),
          (error: ErrorModel) => this.dialogService.showNotification(error.message));
    }
  }

  compareUf(option: UfModel, selectValue: UfModel): boolean {
    return !!selectValue && selectValue.id === option.id;
  }

  hasError = (controlName: string, errorName: string) => {
    return this.parentForm.controls[controlName].hasError(errorName);
  }

  displayFn(cidadeDominio?: CidadeModel): string | undefined {
    return cidadeDominio ? cidadeDominio.descricao : undefined;
  }

  obterAutorPorCpf(): void {
    this.loadingCpf = true;
    this.demandaService
      .obterAutorPorCpf(this.parentForm.get('cpf').value)
      .pipe(finalize(() => this.loadingCpf = false))
      .subscribe(
        (response: AutorModel[]) => {
          if (response.length > 0) {
            const autor = response[0];
            this.autorId.emit(autor.id ? autor.id : null);
            this.parentForm.get('nome').setValue(autor.nome);
            this.parentForm.get('dataNascimento').setValue(autor.dataNascimento);
            this.parentForm.get('cpf').setValue(autor.cpf);
            this.parentForm.get('cns').setValue(autor.cns);
            this.parentForm.get('rg').setValue(autor.rg);
            this.parentForm.get('cep').setValue(autor.cep);
            this.parentForm.get('nomeMae').setValue(autor.nomeMae);
            this.parentForm.get('sexo').setValue(autor.sexo);
            this.parentForm.get('logradouro').setValue(autor.logradouro);
            this.parentForm.get('bairro').setValue(autor.bairro);
            this.parentForm.get('estado').setValue(autor.cidade.uf);
            this.parentForm.get('cidade').setValue(autor.cidade);
            this.parentForm.get('numero').setValue(autor.numero);
            this.parentForm.get('complemento').setValue(autor.complemento);
            this.parentForm.get('email').setValue(autor.email);
            this.parentForm.get('telefone').setValue(autor.telefone);
            this.parentForm.get('obsTelefone').setValue(autor.obsTelefone);
            this.parentForm.get('outroTelefone').setValue(autor.outroTelefone);
            this.parentForm.get('obsOutroTelefone').setValue(autor.obsOutroTelefone);
          }
        },
        (error: ErrorModel) => this.dialogService.showNotification(error.message)
      );
  }

  buscarCepAutor(event: Event): void {
    event.stopPropagation();
    event.preventDefault();
    const cep = this.parentForm.get('cep');
    if (cep.valid) {
      this.cepLoading = true;
      this.demandaService.getCep(cep.value).pipe(
        finalize(() => this.cepLoading = false)
      ).subscribe(
        (resp: any) => {
          this.parentForm.get('logradouro').setValue(resp.logradouro);
          this.parentForm.get('bairro').setValue(resp.bairro.nome);
          this.parentForm.get('estado').setValue(
            this.listaUfs.find((uf: UfModel) => uf.descricao === resp.siglaUf)
          );
          this.parentForm.get('cidade').setValue(resp.idMunicipioIbge);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
    }
  }

  obterAnos(dataNascimento: Date): number {
    const nascimento = new Date(dataNascimento);
    const d = new Date,
      ano_atual = d.getFullYear(),
      mes_atual = d.getMonth() + 1,
      dia_atual = d.getDate(),

      ano_aniversario = +nascimento.getFullYear(),
      mes_aniversario = +nascimento.getMonth(),
      dia_aniversario = +nascimento.getDate();

    this.idade = ano_atual - ano_aniversario;

    if (mes_atual < mes_aniversario || mes_atual === mes_aniversario && dia_atual < dia_aniversario) {
      this.idade--;
    }
    return this.idade < 0 ? 0 : this.idade;
  }

  onSubmit(): void {
    this.submitButton.nativeElement.click();
  }



}
