import { Component, OnInit, Inject } from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Observable, fromEvent } from 'rxjs';
import { FileValidator, FileInput } from 'ngx-material-file-input';
import { debounceTime, map, finalize } from 'rxjs/operators';

import { DialogService } from '@datasus/layout';

import { ArquivoDemandaModel } from '../../models/arquivo-demanda.model';
import { ArquivoArvoreService } from '../../services/arquivo-arvore.service';
import { ErrorModel } from '@datasus/core';

export interface DataArquivo {
  diretorio: ArquivoDemandaModel;
  autorDemandaId: number;
}

@Component({
  selector: 'scodes-dialog-adicionar-arquivo',
  templateUrl: './dialog-adicionar-arquivo.component.html',
  styleUrls: ['./dialog-adicionar-arquivo.component.scss']
})
export class DialogAdicionarArquivoComponent implements OnInit {

  formGroup: FormGroup;
  onMobile$: Observable<boolean>;
  arquivoBase64: string;
  loading: boolean;

  constructor(
    public dialogRef: MatDialogRef<DialogAdicionarArquivoComponent>,
    @Inject(MAT_DIALOG_DATA) public data: DataArquivo,
    private formBuilder: FormBuilder,
    private dialogService: DialogService,
    private arquivoArvoreService: ArquivoArvoreService,
  ) { }

  ngOnInit() {
    console.log('dialogRef data -->', this.data);

    this.formGroup = this.formBuilder.group({
      dataProtocolo: [null, Validators.required],
      fileInputArquivo: [null,
        Validators.compose([Validators.required, FileValidator.maxContentSize(3145728)])
      ]

    });

    this.formGroup.get('fileInputArquivo').valueChanges
      .pipe(debounceTime(500))
      .subscribe((value: FileInput) => {
        this.arquivoBase64 = null;
        if (this.formGroup.get('fileInputArquivo').valid &&
            value !== null &&
            value !== undefined) {
          if (!value.fileNames.toLowerCase().endsWith('pdf')) {
            return this.formGroup.get('fileInputArquivo').setErrors({ formatoInvalido: true });
          }
          this.readFile(<File>value.files[0]).subscribe(res => this.arquivoBase64 = res);
        }
      });

  }

  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }

  onClose(): void {
    this.dialogRef.close();
  }

  readFile(file: File | Blob): Observable<any> {
    const reader = new FileReader();
    const loadend = fromEvent(reader, 'loadend').pipe(
      map((read: any) => {
        return read.target.result;
      })
    );
    reader.readAsDataURL(file);
    return loadend;
  }

  obterDescricaoExtensao(texto: string, tipo: string): any {
    if (tipo === 'nome') {
      return texto.split('.')[0];
    } else if (tipo === 'extencao') {
      return texto.toLowerCase().split('.').pop();
    }
  }

  submitArquivo(event: Event): void {
    event.preventDefault();

    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos obrigatórios');
    }

    const dados: ArquivoDemandaModel = {
      dataProtocolo: this.formGroup.get('dataProtocolo').value,
      autorDemanda: {id: this.data.autorDemandaId},
      arquivo64: this.arquivoBase64.split(',')[1],
      descricao:  this.obterDescricaoExtensao(this.formGroup.get('fileInputArquivo').value.files[0].name, 'nome'),
      extencao: this.obterDescricaoExtensao(this.formGroup.get('fileInputArquivo').value.files[0].name, 'extencao'),
      diretorio: {id: this.data.diretorio.id}
    };

    this.loading = true;
    this.arquivoArvoreService.incluir(dados)
    .pipe(finalize(() => this.loading = false))
    .subscribe(
      (response) => this.dialogRef.close(response),
      (error: ErrorModel) => this.dialogService.showAlert(error.message));


  }

}
