import { Component, OnInit, ViewChild, OnDestroy, ChangeDetectorRef } from '@angular/core';
import { Router } from '@angular/router';
import { FormBuilder, FormGroup } from '@angular/forms';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { MatPaginator, MatTabGroup, PageEvent, MatDialog } from '@angular/material';
import { Observable, Subscription } from 'rxjs';
import { finalize, tap, map } from 'rxjs/operators';
import { CdkScrollable, ScrollDispatcher } from '@angular/cdk/overlay';

import { LayoutService, DialogService } from '@datasus/layout';
import { requestItemsParams, PageableResponseModel, ErrorModel, DominioItemModel } from '@datasus/core';

import { DemandaModel } from '../../models/demanda.model';
import { DemandasService } from '../../services/demandas.service';
import { UnidadeOrganizacionalService } from 'src/app/shared/services/unidade-organizacional.service';
import { TipoDemandaService } from 'src/app/shared/services/tipo-demanda.service';
import { AutorDemandaModel } from '../../models/autor-demanda.model';
import { DialogAutorComponent } from '../../components/dialog-autor/dialog-autor.component';

@Component({
  selector: 'scodes-demandas-list',
  templateUrl: './demandas-list.component.html',
  styleUrls: ['./demandas-list.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class DemandasListComponent implements OnInit, OnDestroy {

  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

  // Retrived Data Source
  items: AutorDemandaModel[] = [];
  infinityItems: AutorDemandaModel[] = [];
  showField: boolean;
  placeholder = 'Informe o CPF';
  listaUnidadeOrganizacional: DominioItemModel[];
  listaTipoDemanda: DominioItemModel[];

  // Data Table Config
  displayedColumns: string[] = [
    'dataCadastro',
    'protocolo',
    'processo',
    'autor',
    'documento',
    'tipoProcesso',
    'tipoDemanda',
    'actions'
  ];

  // Viewport Observable
  onMobile$: Observable<boolean>;

  // Form Group
  formGroup: FormGroup;

  // Data Table Controls (begin)
  loading = false;
  infinityItemsMessage = '';
  total = 0;
  pageSize = 0;
  // Data Table Controls (end)

  // View Controls (begin)
  viewChars = false;
  actionButtonCtrl: Map<DemandaModel, boolean> = new Map();
  // View Controls (end)

  listaDocumentos = [
    { name: 'CPF', value: 1 },
    { name: 'RG', value: 2 },
    { name: 'CNS', value: 3 }
  ];

  private scrollDispatcherSubscription: Subscription;

  constructor(
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private changeDetectorRef: ChangeDetectorRef,
    private scrollDispatcher: ScrollDispatcher,
    private demandaService: DemandasService,
    private unidadeOrganizacionalService: UnidadeOrganizacionalService,
    private tipoDemandaService: TipoDemandaService,
    private matDialog: MatDialog,
  ) { }

  ngOnInit() {
    this.layoutService.setTitle('Pesquisa de Demanda');

    this.formGroup = this.formBuilder.group({
      documento: [null],
      idTipoDemanda: [null],
      tipoDocumento: 1,
      idUnidadeOrganizacional: [null],
      nomeAutor: [null],
      nomeMae: [null],
      processo: [null],
      protocolo: [null]
    });


    this.formGroup.get('tipoDocumento').valueChanges
      .subscribe((valor) => {
        if (valor) {
          this.showField = true;
          this.formGroup.get('documento').setValue(null);
          this.formGroup.get('documento').setValidators(null);
          if (valor === 1) {
            this.placeholder = 'Informe o CPF';
          } else if (valor === 2) {
            this.placeholder = 'Informe o RG';
          } else if (valor === 3) {
            this.placeholder = 'Informe o CNS';
          }
        } else {
          this.showField = false;
        }
      });

    this.obterUnidadeOrganizacional();
    this.obterTipoDemanda();

    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );

  }

  ngOnDestroy(): void {
    this.scrollRegister(false);
    this.infinityItems = [];
  }

  obterUnidadeOrganizacional(): void {
    this.unidadeOrganizacionalService
      .obterDominio()
      .subscribe(
        (response) => this.listaUnidadeOrganizacional = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  obterTipoDemanda(): void {
    this.tipoDemandaService
      .obterDominio().pipe(
        map((tipoDemanda: DominioItemModel[]) => {
          const arr: DominioItemModel[] = [];
          tipoDemanda.map((i) => {
            if (i.codigo !== '1') {
              arr.push(i);
            }
          });
          return arr;
        })
      )
      .subscribe(
        (response) => this.listaTipoDemanda = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  // Data Events (begin)
  search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    if (tabs) {
      tabs.selectedIndex = 1;
    }
    this.infinityItems = [];
    this.requestItems();
  }

  formReset() {
    setTimeout(() => {
      this.formGroup.get('tipoDocumento').setValue(1);
      this.formGroup.get('documento').setValue(null);
    });
  }

  requestItems(event?: PageEvent): void {
    this.loading = true;
    this.infinityItemsMessage = '';
    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }
    this.demandaService.obterTodosPaginado({
      filter: this.formGroup.value,
      ...requestItemsParams({ to: this, withPageEvent: event }),
    }).pipe(
      finalize(() => {
        this.loading = false;
        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      })
    ).subscribe(
      (page: PageableResponseModel<AutorDemandaModel>) => {
        this.total = page.total;
        this.items = page.items;
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
  // Data Events (end)

  // Item Actions (begin)
  edit(item: DemandaModel) {
    this.router.navigate(['/demanda/menu/protocolo', item.id]);
  }

  delete(item: DemandaModel) {
    const buttons: Map<string, () => void> = new Map();
    buttons.set('Sim', () => {
      this.layoutService.loading();
      console.log('Ação #2', item);
      this.demandaService.excluir(item.id).pipe(
        finalize(() => this.layoutService.loaded()),
      ).subscribe(
        () => {
          this.dialogService.showNotification('Registro excluído com sucesso');
          this.requestItems();
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
    });
    buttons.set('Não', () => console.log('Abortou a ação #2', item));

    this.dialogService.showConfirmation({
      question: 'Deseja mesmo remover?',
      title: 'Cuidado!',
      anwsers: buttons,
      type: 'warn',
    });
  }
  // Item Actions (end)


  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }

  abrirAutor(item: AutorDemandaModel): void {
    const dialogAdicionarArquivo = this.matDialog.open(DialogAutorComponent, {
      disableClose: true,
      minWidth: '50%',
      data: { ...item }
    });

    dialogAdicionarArquivo.afterClosed().subscribe((resultado: boolean) => {
      if (resultado) {
        console.log(resultado);
      }
    });
  }

  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }

}
