import { Component, OnInit } from '@angular/core';
import { FormGroup, FormBuilder } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { MatAutocompleteSelectedEvent } from '@angular/material';
import { Observable } from 'rxjs';
import { tap, switchMap, debounceTime } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { DominioItemModel, ErrorModel } from '@datasus/core';
import { FormValidators } from '@datasus/forms';

import { CidadeModel } from 'src/app/shared/models/cidade.model';
import { TipoAcaoContraModel } from 'src/app/shared/models/tipo-acao-contra.model';
import { UfModel } from 'src/app/shared/models/uf.model';
import { ProfissionalModel } from 'src/app/shared/models/profissional.model';
import { CidadeService } from 'src/app/shared/services/cidade.service';
import { UfService } from 'src/app/shared/services/uf.service';
import { TipoAcaoService } from 'src/app/rotina/tabela/tipo-acao/services/tipo-acao.service';

@Component({
  selector: 'scodes-processual-form',
  templateUrl: './processual-form.component.html',
  styleUrls: ['./processual-form.component.scss']
})
export class ProcessualFormComponent implements OnInit {

  listaUfs: UfModel[];
  listaPrazos: any;
  exibirMunicipio: boolean;
  isAcaoContra: boolean;
  isOab: boolean;
  oabLoading: boolean;
  isSalvo: boolean;
  isEstadoMunicipio: boolean;
  listaCidades: CidadeModel[];
  listaProfissionais: ProfissionalModel[];
  listaTipoAcao: DominioItemModel[];

  // Form Group
  formGroup: FormGroup;
  // Viewport Observable
  onMobile$: Observable<boolean>;

  listaTipoAcaoContra: TipoAcaoContraModel[] = [
    { id: 'E', descricao: 'Estado' },
    { id: 'M', descricao: 'Município' },
    { id: 'EM', descricao: 'Estado + Município' },
    { id: 'UEM', descricao: 'Estado + Município + União' },
    { id: 'UE', descricao: 'Estado + União' },
    { id: 'UM', descricao: 'Município + União' }
  ];

  listaPatrono: DominioItemModel[] = [
    { codigo: '4', descricao: 'Advogado' },
    { codigo: '5', descricao: 'Defensor Público' },
    { codigo: '6', descricao: 'Promotor' },
    { codigo: '8', descricao: 'Procurador' },
    { codigo: '88', descricao: 'Jefaz' },
    { codigo: '29', descricao: 'JEC' },
  ];

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private cidadeService: CidadeService,
    private ufService: UfService,
    private tipoAcaoService: TipoAcaoService
  ) { }

  ngOnInit() {
    this.layoutService.setTitle('Cadastro de Demanada - Processual');

    this.formGroup = this.formBuilder.group({
      prazo: [null],
      segredoJustica: [null],
      tipoAcao: [null],
      estado: [null],
      municipioVara: [null],
      vara: [null],
      acaoContra: [null],
      patrono: [null],
      evento: [null],
      estadoDemanda: [null],
      cidadeDemanda: [null],
      estadoPatrono: [null, FormValidators.requiredIf(() => this.isEstadoMunicipio)],
      oab: [null, FormValidators.requiredIf(() => this.isOab)],
      profissional: [null, FormValidators.requiredIf(() => this.isEstadoMunicipio)],
    });

    this.tipoAcaoService
      .obterDominio()
      .subscribe(
        (response) => this.listaTipoAcao = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );

    this.ufService.obterTodos()
      .subscribe(
        (response) => this.listaUfs = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );

    this.formGroup.get('estadoDemanda').valueChanges
      .pipe(
        tap(() => this.formGroup.get('cidadeDemanda').setValue(null)),
        switchMap((uf: UfModel) => {
          if (uf) {
            return this.cidadeService.obterCidadesPorUf(uf.id);
          }
          return [];
        }),
      ).subscribe(
        (resp: any) => this.listaCidades = resp);

    this.formGroup.get('cidadeDemanda').valueChanges
      .pipe(debounceTime(350))
      .subscribe(
        (cidade) => {
          if (!!this.formGroup.get('estadoDemanda').value && typeof cidade === 'string') {
            return this.cidadeService.obterCidadesPorNome(cidade, this.formGroup.get('estadoDemanda').value.id)
              .subscribe(
                (response) => this.listaCidades = response,
                (error: ErrorModel) => this.dialogService.showNotification(error.message));
          }
        });

    this.formGroup.get('acaoContra').valueChanges
      .subscribe((valor) => {
        if (valor === 'M' || valor === 'EM' || valor === 'UEM' || valor === 'UM') {
          this.exibirMunicipio = true;
          this.isAcaoContra = true;
        } else {
          this.exibirMunicipio = false;
          this.isAcaoContra = false;
        }
      });

    this.formGroup.get('patrono').valueChanges
      .subscribe((valor) => {
        if (valor === '4' || valor === '5' || valor === '8') {
          this.isOab = true;
          this.isEstadoMunicipio = true;
        } else if (valor === '88' || valor === '29') {
          this.isEstadoMunicipio = false;
          this.isOab = false;
        } else if (valor === '6') {
          this.isEstadoMunicipio = true;
          this.isOab = false;
        } else {
          this.isEstadoMunicipio = false;
          this.isOab = false;
        }
      });

  }



  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }

  displayFn(cidadeDominio?: CidadeModel): string | undefined {
    return cidadeDominio ? cidadeDominio.descricao : null;
  }

  displayProfissioanal(profissional: ProfissionalModel): string | undefined {
    return profissional ? profissional.nome : null;
  }

  obterCidades(event: MatAutocompleteSelectedEvent) {
    if (event.option.value && event.option.value.descricao) {
      this.cidadeService
        .obterPorId(event.option.value.id)
        .subscribe(
          (response) => this.formGroup.get('cidade').setValue(response),
          (error: ErrorModel) => this.dialogService.showNotification(error.message));
    }
  }

  obterCidadesDemanda(event: MatAutocompleteSelectedEvent) {
    if (event.option.value && event.option.value.descricao) {
      this.cidadeService
        .obterPorId(event.option.value.id)
        .subscribe(
          (response) => this.formGroup.get('cidadeDemanda').setValue(response),
          (error: ErrorModel) => this.dialogService.showNotification(error.message));
    }
  }

  oabFinder(event: Event): void {
    event.preventDefault();

  }

  submitProcessual(event: Event): void {
    event.preventDefault();
  }

}
