import { Component, Inject, LOCALE_ID, OnInit } from '@angular/core';
import { formatDate } from '@angular/common';
import { ActivatedRoute, Router } from '@angular/router';
import { AbstractControl, FormArray, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MatDialog, MatSelectChange } from '@angular/material';

import { Observable } from 'rxjs';
import { filter, finalize } from 'rxjs/operators';

import { DominioItemModel, ErrorModel } from '@datasus/core';
import { DialogService, LayoutService } from '@datasus/layout';

import { MensagemEnum } from '../../../../shared/enums/mensagem.enum';
import { TipoMovimentacaoEnum } from '../../../../shared/enums/tipo-movimentacao.enum';
import { TipoMovimentacaoTipoEnum } from '../../../../shared/enums/tipo-movimentacao-tipo.enum';

import { NumberValidators } from '../../../../shared/validators/number.validators';

import { TipoMovimentacaoService } from '../../../../shared/services/tipo-movimentacao.service';
import { FornecedorService } from '../../../../shared/services/fornecedor.service';
import { FabricanteService } from '../../../../shared/services/fabricante.service';
import { EntradaEstoqueService } from '../../services/entrada-estoque.service';
import { CompraModalidadeService } from '../../services/compra-modalidade.service';
import { EstoqueAuxiliarService } from '../../../../shared/services/estoque-auxiliar.service';

import { DialogPesquisaItemComponent } from '../../../../shared/components/dialog-pesquisa-item/dialog-pesquisa-item.component';

import { EntradaEstoqueModel } from '../../models/entrada-estoque.model';
import { CompraTrocaModel } from '../../models/compra-troca.model';
import { ItemModel } from '../../../tabela/produto/models/item.model';
import { UnidadeDispensadoraModel } from '../../../../shared/models/unidade-dispensadora.model';

@Component({
  selector: 'scodes-entrada-estoque',
  styleUrls: ['./entrada-estoque.component.scss'],
  templateUrl: './entrada-estoque.component.html',
})
export class EntradaEstoqueComponent implements OnInit {
  unidadeDispensadora: UnidadeDispensadoraModel;

  formGroup: FormGroup;

  currencyMaskOptions = {
    align: 'left',
    prefix: 'R$ ',
    thousands: '.',
    decimal: ','
  };

  tiposMovimentacoes: DominioItemModel[];
  fornecedores: DominioItemModel[];
  fabricantes: DominioItemModel[];
  compraModalidades: DominioItemModel[];

  loadingTipoMovimentacao = false;
  loadingFornecedor = false;
  loadingFabricante = false;
  loadingCompraModalidade = false;

  saving = false;
  onMobile$: Observable<boolean>;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private formBuilder: FormBuilder,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private matDialog: MatDialog,
    private tipoMovimentacaoService: TipoMovimentacaoService,
    private fornecedorService: FornecedorService,
    private fabricanteService: FabricanteService,
    private entradaEstoqueService: EntradaEstoqueService,
    private compraModalidadeService: CompraModalidadeService,
    private estoqueAuxiliarService: EstoqueAuxiliarService,
    @Inject(LOCALE_ID) private locale: string,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Entrada Externa');
    this.onMobile$ = this.layoutService.onMobile();

    this.unidadeDispensadora = this.route.snapshot.data['unidadeDispensadora'];

    this.formGroup = this.formBuilder.group({
      tipoMovimentacao: [null],
      dataEntrada: [{value: this.currentDate, disabled: true}],
      compraModalidade: [null],
      notaEmpenho: [null],
      notaFiscal: [null],
      fornecedor: [null],
      valorNotaFiscal: [{value: 0, disabled: true}],
      itens: this.formBuilder.array([])
    });

    this.loadTiposMovimentacoes();
    this.loadCompraModalidades();
    this.loadFornecedores();
    this.loadFabricantes();
  }

  get tipoMovimentacao(): number {
    return Number(this.formGroup.get('tipoMovimentacao').value);
  }

  get isTipoCompra(): boolean {
    return this.tipoMovimentacao === TipoMovimentacaoEnum.COMPRA_ENTRADA;
  }

  get isTipoCartaTroca(): boolean {
    return this.tipoMovimentacao === TipoMovimentacaoEnum.CARTA_DE_TROCA_ENTRADA;
  }

  get isTipoOutra(): boolean {
    return this.tipoMovimentacao === TipoMovimentacaoEnum.OUTRA_ENTRADA;
  }

  get currentDate(): string {
    return formatDate(Date.now(), 'dd/MM/yyyy', this.locale);
  }

  get itens(): FormArray {
    return <FormArray> this.formGroup.get('itens');
  }

  save(event: Event): void {
    event.preventDefault();

    if (this.formGroup.invalid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }

    if (this.itens.length === 0) {
      return this.dialogService.showNotification(MensagemEnum.LISTA_ITEM_VAZIA);
    }

    const value = this.formGroup.value;

    const body = <EntradaEstoqueModel> {
      tipoMovimentacao: {id: this.tipoMovimentacao},
      itens: value.itens.map(v => {
        v.fabricante = v.fabricante != null ? {id: Number(v.fabricante)} : null;
        return v;
      }),
    };

    if (this.tipoMovimentacao === TipoMovimentacaoEnum.COMPRA_ENTRADA ||
      this.tipoMovimentacao === TipoMovimentacaoEnum.CARTA_DE_TROCA_ENTRADA) {
      body.compraTroca = <CompraTrocaModel> {
        compraModalidade: value.compraModalidade != null ? {id: Number(value.compraModalidade)} : null,
        notaEmpenho: value.notaEmpenho,
        notaFiscal: value.notaFiscal,
        fornecedor: value.fornecedor != null ? {id: Number(value.fornecedor)} : null,
      };
    }

    this.entradaEstoqueService.incluir(body).pipe(
        finalize(() => this.saving = false)
      ).subscribe(
        () => {
          this.dialogService.showNotification(MensagemEnum.REGISTRO_SALVO);
          this.router.navigate(['/rotina/gestao-estoque']);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
  }

  resetForm(event: Event): void {
    setTimeout(() => {
      this.formGroup.get('dataEntrada').setValue(this.currentDate);
      this.formGroup.get('valorNotaFiscal').setValue(0);
      this.formGroup.setControl('itens', this.formBuilder.array([]));
    });
  }

  isInvalid(controlName: string, control: AbstractControl): boolean {
    return control.get(controlName).invalid;
  }

  getErrorMessage(controlName: string, control: AbstractControl): string {
    const fc = control.get(controlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      fc.hasError('onlyDigits') ? MensagemEnum.ONLY_DIGITS :
      '';
  }

  changeTipoMovimentacao(event: MatSelectChange): void {
    this.formGroup.get('compraModalidade').setValue(null);
    this.formGroup.get('notaEmpenho').setValue(null);
    this.formGroup.get('notaFiscal').setValue(null);
    this.formGroup.get('fornecedor').setValue(null);

    this.itens.controls.forEach(formGroup => {
      formGroup.get('acertoEmbalagem').setValue(null);
      formGroup.get('valor').setValue(null);
      formGroup.get('valorTotalItem').setValue(0);
    });

    this.calculaValorNotaFiscal();
  }

  calculaValorTotalItem(control: AbstractControl): void {
    const qtd = Number(control.get('quantidade').value);
    const valor = Number(control.get('valor').value);
    control.get('valorTotalItem').setValue(qtd * valor);
    this.calculaValorNotaFiscal();
  }

  addItem(event: MouseEvent): void {
    event.preventDefault();
    this.matDialog
      .open(DialogPesquisaItemComponent, {width: '80vw'})
      .afterClosed()
      .pipe(filter(value => !!value))
      .subscribe(value => this.onSelectItem(value));
  }

  onSelectItem(value: ItemModel): void {
    if (this.itens.controls.some(fc => fc.get('idItem').value === value.id)) {
      return this.dialogService.showNotification(MensagemEnum.ITEM_JA_NA_LISTA);
    }
    this.estoqueAuxiliarService.obterQuantidade(value.id, this.unidadeDispensadora.id)
      .subscribe((qtdItem: number) => {
        this.itens.push(this.createItem(value, qtdItem));
      });
  }

  removeItem(index: number): void {
    this.itens.removeAt(index);
  }

  private createItem(item: ItemModel, quantidadeItem: number): FormGroup {
    return this.formBuilder.group({
      descricao: [{value: item.descricao, disabled: true}],
      idItem: [item.id],
      idUnidadeDispensadora: [this.unidadeDispensadora.id],
      quantidade: [null, Validators.compose([NumberValidators.onlyDigits])],
      acertoEmbalagem: [null, Validators.compose([NumberValidators.onlyDigits])],
      valor: [null],
      termolabel: [null],
      fabricante: [null],
      valorTotalItem: [{value: 0, disabled: true}],
      qtdEstoque: [{value: quantidadeItem, disabled: true}]
    });
  }

  private calculaValorNotaFiscal(): void {
    const valorNota = this.itens.controls
      .map(formGroup => Number(formGroup.get('valorTotalItem').value))
      .reduce((accum, curr) => accum + curr, 0);
    this.formGroup.get('valorNotaFiscal').setValue(valorNota);
  }

  private loadTiposMovimentacoes(): void {
    this.loadingTipoMovimentacao = true;
    this.tipoMovimentacaoService.obterDominioPorTipo(TipoMovimentacaoTipoEnum.ENTRADA)
      .pipe(finalize(() => this.loadingTipoMovimentacao = false))
      .subscribe(
        (data) => this.tiposMovimentacoes = data,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private loadCompraModalidades(): void {
    this.loadingCompraModalidade = true;
    this.compraModalidadeService.obterDominio()
      .pipe(finalize(() => this.loadingCompraModalidade = false))
      .subscribe(
        (data) => this.compraModalidades = data,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private loadFornecedores(): void  {
    this.loadingFornecedor = true;
    this.fornecedorService.obterDominio()
      .pipe(finalize(() => this.loadingFornecedor = false))
      .subscribe(
        (data) => this.fornecedores = data,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private loadFabricantes(): void  {
    this.loadingFabricante = true;
    this.fabricanteService.obterDominio()
      .pipe(finalize(() => this.loadingFabricante = false))
      .subscribe(
        (data) => this.fabricantes = data,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }
}
