import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';

import { Observable } from 'rxjs';
import { finalize } from 'rxjs/operators';

import { DialogService, LayoutService } from '@datasus/layout';
import { DominioItemModel, ErrorModel } from '@datasus/core';

import { MensagemEnum } from '../../../../shared/enums/mensagem.enum';
import { UnidadeDispensadoraService } from '../../../../shared/services/unidade-dispensadora.service';

@Component({
  selector: 'scodes-gestao-estoque',
  templateUrl: './gestao-estoque.component.html',
})
export class GestaoEstoqueComponent implements OnInit {

  formGroup: FormGroup;

  unidadesDispensadoras: DominioItemModel[];
  acoes: DominioItemModel[];

  loadingUnidadeDispensadora = false;

  onMobile$: Observable<boolean>;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private unidadeDispensadoraService: UnidadeDispensadoraService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Gestão de Estoque');

    this.onMobile$ = this.layoutService.onMobile();

    this.formGroup = this.formBuilder.group({
      unidadeDispensadora: [
        null,
        Validators.compose([
          Validators.required,
        ])
      ],
      acao: [
        null,
        Validators.compose([
          Validators.required,
        ])
      ],
    });

    this.acoes = [
      { codigo: 'entrada', descricao: 'Entrada Externa'},
      { codigo: 'saida', descricao: 'Saída Externa'},
    ];

    this.loadingUnidadeDispensadora = true;
    this.unidadeDispensadoraService.obterDominio()
      .pipe(finalize(() => this.loadingUnidadeDispensadora = false))
      .subscribe(
        (data) => this.unidadesDispensadoras = data,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  redirect(event: Event): void {
    event.preventDefault();

    if (this.formGroup.invalid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }

    const unidadeDispensadoraId = this.formGroup.get('unidadeDispensadora').value;
    const acao = this.formGroup.get('acao').value;

    this.router.navigate(['/rotina/gestao-estoque', acao, unidadeDispensadoraId]);
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
        '';
  }

}
