import { Component, Inject, LOCALE_ID, OnInit } from '@angular/core';
import { formatDate } from '@angular/common';
import { ActivatedRoute, Router } from '@angular/router';
import { AbstractControl, FormArray, FormBuilder, FormGroup, ValidationErrors, ValidatorFn, Validators } from '@angular/forms';
import { MatDialog, MatSelectChange } from '@angular/material';

import { Observable } from 'rxjs';
import { filter, finalize } from 'rxjs/operators';

import { DominioItemModel, ErrorModel } from '@datasus/core';
import { DialogService, LayoutService } from '@datasus/layout';

import { MensagemEnum } from '../../../../shared/enums/mensagem.enum';
import { TipoMovimentacaoEnum } from '../../../../shared/enums/tipo-movimentacao.enum';
import { TipoMovimentacaoTipoEnum } from '../../../../shared/enums/tipo-movimentacao-tipo.enum';

import { NumberValidators } from '../../../../shared/validators/number.validators';

import { TipoMovimentacaoService } from '../../../../shared/services/tipo-movimentacao.service';
import { FornecedorService } from '../../../../shared/services/fornecedor.service';
import { UnidadeTransferenciaService } from '../../../tabela/unidade-transferencia/services/unidade-transferencia.service';
import { SaidaEstoqueService } from '../../services/saida-estoque.service';
import { EstoqueAuxiliarService } from '../../../../shared/services/estoque-auxiliar.service';

import { DialogPesquisaItemComponent } from '../../../../shared/components/dialog-pesquisa-item/dialog-pesquisa-item.component';

import { SaidaEstoqueModel } from '../../models/saida-estoque.model';
import { CompraTrocaModel } from '../../models/compra-troca.model';
import { ItemModel } from '../../../tabela/produto/models/item.model';
import { UnidadeDispensadoraModel } from '../../../../shared/models/unidade-dispensadora.model';
import { TransferenciaModel } from '../../models/transferencia.model';
import { TipoTransferenciaEnum } from '../../enums/tipo-transferencia.enum';

@Component({
  selector: 'scodes-saida-estoque',
  styleUrls: ['./saida-estoque.component.scss'],
  templateUrl: './saida-estoque.component.html',
})
export class SaidaEstoqueComponent implements OnInit {
  unidadeDispensadora: UnidadeDispensadoraModel;

  formGroup: FormGroup;

  tiposMovimentacoes: DominioItemModel[];
  fornecedores: DominioItemModel[];
  unidadesTransferencia: DominioItemModel[];

  loadingTipoMovimentacao = false;
  loadingFornecedor = false;
  loadingUnidadeTransferencia = false;

  saving = false;
  onMobile$: Observable<boolean>;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private formBuilder: FormBuilder,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private matDialog: MatDialog,
    private tipoMovimentacaoService: TipoMovimentacaoService,
    private fornecedorService: FornecedorService,
    private unidadeTransferenciaService: UnidadeTransferenciaService,
    private saidaEstoqueService: SaidaEstoqueService,
    private estoqueAuxiliarService: EstoqueAuxiliarService,
    @Inject(LOCALE_ID) private locale: string,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Saída Externa');
    this.onMobile$ = this.layoutService.onMobile();

    this.unidadeDispensadora = this.route.snapshot.data['unidadeDispensadora'];

    this.formGroup = this.formBuilder.group({
      tipoMovimentacao: [null],
      dataSaida: [{value: this.currentDate, disabled: true}],
      notaEmpenho: [null],
      fornecedor: [null],
      motivo: [null],
      docTransferencia: [null],
      unidadeTransferencia: [null],
      itens: this.formBuilder.array([])
    });

    this.loadTiposMovimentacoes();
    this.loadFornecedores();
    this.loadUnidadesTransferencia();
  }

  get tipoMovimentacao(): number {
    return Number(this.formGroup.get('tipoMovimentacao').value);
  }

  get isTipoTransferencia(): boolean {
    return this.tipoMovimentacao === TipoMovimentacaoEnum.TRANSFERENCIA_SAIDA;
  }

  get isTipoCartaTroca(): boolean {
    return this.tipoMovimentacao === TipoMovimentacaoEnum.CARTA_DE_TROCA_SAIDA;
  }

  get isTipoOutra(): boolean {
    return this.tipoMovimentacao === TipoMovimentacaoEnum.OUTRA_SAIDA;
  }

  get currentDate(): string {
    return formatDate(Date.now(), 'dd/MM/yyyy', this.locale);
  }

  get itens(): FormArray {
    return <FormArray> this.formGroup.get('itens');
  }

  save(event: Event): void {
    event.preventDefault();

    if (this.formGroup.invalid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }

    if (this.itens.length === 0) {
      return this.dialogService.showNotification(MensagemEnum.LISTA_ITEM_VAZIA);
    }

    const value = this.formGroup.value;

    const body = <SaidaEstoqueModel> {
      tipoMovimentacao: {id: this.tipoMovimentacao},
      motivo: value.motivo,
      itens: value.itens,
    };

    if (this.tipoMovimentacao === TipoMovimentacaoEnum.TRANSFERENCIA_SAIDA) {
      body.transferencia = <TransferenciaModel> {
        descricao: value.docTransferencia,
        tipo: TipoTransferenciaEnum.SAIDA,
        unidadeTransferencia: value.unidadeTransferencia != null ? {id: Number(value.unidadeTransferencia)} : null,
      };
    } else if (this.tipoMovimentacao === TipoMovimentacaoEnum.CARTA_DE_TROCA_SAIDA) {
        body.compraTroca = <CompraTrocaModel> {
          notaEmpenho: value.notaEmpenho,
          fornecedor: value.fornecedor != null ? {id: Number(value.fornecedor)} : null,
        };
    }

    this.saidaEstoqueService.incluir(body).pipe(
        finalize(() => this.saving = false)
      ).subscribe(
        () => {
          this.dialogService.showNotification(MensagemEnum.REGISTRO_SALVO);
          this.router.navigate(['/rotina/gestao-estoque']);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
  }

  resetForm(event: Event): void {
    setTimeout(() => {
      this.formGroup.get('dataSaida').setValue(this.currentDate);
      this.formGroup.setControl('itens', this.formBuilder.array([]));
    });
  }

  isInvalid(controlName: string, control: AbstractControl): boolean {
    return control.get(controlName).invalid;
  }

  getErrorMessage(controlName: string, control: AbstractControl): string {
    const fc = control.get(controlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      fc.hasError('onlyDigits') ? MensagemEnum.ONLY_DIGITS :
      fc.hasError('qtdSaidaItem') ? MensagemEnum.QTD_SAIDA_ITEM :
      '';
  }

  changeTipoMovimentacao(event: MatSelectChange): void {
    this.formGroup.get('notaEmpenho').setValue(null);
    this.formGroup.get('fornecedor').setValue(null);
    this.formGroup.get('motivo').setValue(null);
    this.formGroup.get('docTransferencia').setValue(null);
    this.formGroup.get('unidadeTransferencia').setValue(null);
  }

  addItem(event: MouseEvent): void {
    event.preventDefault();
    this.matDialog
      .open(DialogPesquisaItemComponent, {width: '80vw'})
      .afterClosed()
      .pipe(filter(value => !!value))
      .subscribe(value => this.onSelectItem(value));
  }

  onSelectItem(value: ItemModel): void {
    if (this.itens.controls.some(fc => fc.get('idItem').value === value.id)) {
      return this.dialogService.showNotification(MensagemEnum.ITEM_JA_NA_LISTA);
    }
    this.estoqueAuxiliarService.obterQuantidade(value.id, this.unidadeDispensadora.id)
      .subscribe((qtdItem: number) => {
        this.itens.push(this.createItem(value, qtdItem));
      });
  }

  removeItem(index: number): void {
    this.itens.removeAt(index);
  }

  private createItem(item: ItemModel, quantidadeItem: number): FormGroup {
    return this.formBuilder.group({
      descricao: [{value: item.descricao, disabled: true}],
      idItem: [item.id],
      idUnidadeDispensadora: [this.unidadeDispensadora.id],
      quantidade: [null, Validators.compose([NumberValidators.onlyDigits, this.qtdSaidaItemValidator(quantidadeItem)])],
      qtdEstoque: [{value: quantidadeItem, disabled: true}]
    });
  }

  private loadTiposMovimentacoes(): void {
    this.loadingTipoMovimentacao = true;
    this.tipoMovimentacaoService.obterDominioPorTipo(TipoMovimentacaoTipoEnum.SAIDA)
      .pipe(finalize(() => this.loadingTipoMovimentacao = false))
      .subscribe(
        (data) => this.tiposMovimentacoes = data,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private loadFornecedores(): void  {
    this.loadingFornecedor = true;
    this.fornecedorService.obterDominio()
      .pipe(finalize(() => this.loadingFornecedor = false))
      .subscribe(
        (data) => this.fornecedores = data,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private loadUnidadesTransferencia(): void  {
    this.loadingUnidadeTransferencia = true;
    this.unidadeTransferenciaService.obterDominio()
      .pipe(finalize(() => this.loadingUnidadeTransferencia = false))
      .subscribe(
        (data) => this.unidadesTransferencia = data,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private qtdSaidaItemValidator(qtdEstoque: number): ValidatorFn | null {
    return (control: AbstractControl): ValidationErrors | null => {
      return Number(control.value) > qtdEstoque ? { qtdSaidaItem: true } : null;
    };
  }
}
