import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';

import { finalize } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel, DominioItemModel } from '@datasus/core';

import { MensagemEnum } from 'src/app/shared/enums/mensagem.enum';
import { UnidadeDispensadoraService } from 'src/app/shared/services/unidade-dispensadora.service';

import { InventarioModel } from '../../models/inventario.model';
import { InventarioService } from '../../services/inventario.service';

@Component({
  selector: 'scodes-inventario-form',
  templateUrl: './inventario-form.component.html',
  styleUrls: ['./inventario-form.component.scss']
})
export class InventarioFormComponent implements OnInit {

  formGroup: FormGroup;
  saving: boolean;

  loadingUnidadeDispensadora = false;
  listaUnidadeDispensadora: DominioItemModel[];

  private id: number;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private formBuilder: FormBuilder,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private inventarioService: InventarioService,
    private unidadeDispensadoraService: UnidadeDispensadoraService,
  ) { }

  ngOnInit() {
    this.layoutService.setTitle('Cadastro de Inventário');

    this.formGroup = this.formBuilder.group({
      unidadeDispensadora: [null, Validators.required],
    });

    this.loadingUnidadeDispensadora = true;
    this.unidadeDispensadoraService
      .obterDominio()
      .pipe(finalize(() => this.loadingUnidadeDispensadora = false))
      .subscribe(
        (response) => this.listaUnidadeDispensadora = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  save(event: Event): void {
    event.preventDefault();

    if (this.formGroup.invalid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }

    const inventario: InventarioModel = {
      unidadeDispensadora: {
        id: this.formGroup.value.unidadeDispensadora
      }
    };

    this.saving = true;
    this.inventarioService.incluir(inventario).pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification(MensagemEnum.REGISTRO_SALVO);
        this.router.navigate([`/rotina/inventario/listar`]);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  isInvalid(controlName: string): boolean {
    return this.formGroup.get(controlName).invalid;
  }

  getErrorMessage(controlName: string): string {
    const fc = this.formGroup.get(controlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED : '';
  }

}
