import { Component, OnInit, ViewChild, ChangeDetectorRef, OnDestroy, Inject, LOCALE_ID } from '@angular/core';
import { FormGroup, FormBuilder, FormArray, Validators, AbstractControl } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { MatPaginator, MatTabGroup, PageEvent } from '@angular/material';

import { Observable, Subscription, Subject } from 'rxjs';
import { finalize, debounceTime, switchMap, takeUntil, tap } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel, DominioItemModel, PageableResponseModel, requestItemsParams, download, stringToBlob } from '@datasus/core';

import { MensagemEnum } from 'src/app/shared/enums/mensagem.enum';
import { InventarioService } from '../../services/inventario.service';
import { InventarioItemService } from '../../services/inventario-item.service';
import { ProdutoService } from 'src/app/rotina/tabela/produto/services/produto.service';
import { InventarioModel } from '../../models/inventario.model';
import { InventarioItemModel } from '../../models/inventario-item.model';
import { InventarioItemFilterModel } from '../../models/inventario-item-filter.model';
import { ArquivoBase64Model } from 'src/app/shared/models/arquivo-base64.model';
import { NumberValidators } from 'src/app/shared/validators/number.validators';
import { formatDate } from '@angular/common';

@Component({
  selector: 'scodes-inventario-item',
  templateUrl: './inventario-item.component.html',
  styleUrls: ['./inventario-item.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class InventarioItemComponent implements OnInit, OnDestroy {

  filtro: InventarioItemFilterModel;
  formGroupFilter: FormGroup;
  formArray: FormArray;
  listaProdutos: DominioItemModel[];

  processing = false;
  todosItensLancados = false;

  inventario: InventarioModel;

  // Viewport Observable
  onMobile$: Observable<boolean>;

  onDestroy$ = new Subject<void>();

  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

  // Data Table Controls (begin)
  loading = false;
  infinityItemsMessage = '';
  total = 0;
  pageSize = 10;
  // Data Table Controls (end)

  // Data Table Config
  displayedColumns: string[] = [
    'dataCadastro',
    'nomeInventario',
    'ud',
    'status',
    'actions',
  ];

  private scrollDispatcherSubscription: Subscription;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private formBuilder: FormBuilder,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private inventarioService: InventarioService,
    private inventarioItemService: InventarioItemService,
    private changeDetectorRef: ChangeDetectorRef,
    private produtoService: ProdutoService,
    @Inject(LOCALE_ID) private locale: string,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Listagem de Itens do Inventário');

    this.onMobile$ = this.layoutService.onMobile();

    this.formGroupFilter = this.formBuilder.group({
      produto: [null],
      codigoItem: [null]
    });

    this.formGroupFilter.get('produto').valueChanges
      .pipe(
        takeUntil(this.onDestroy$),
        debounceTime(300),
        switchMap(value => {
          if (typeof value === 'string') {
            return this.produtoService.obterAutocompleteDominioPorTermo(value);
          }
          return [];
        }),
      )
      .subscribe((response: DominioItemModel[]) => {
        this.listaProdutos = response;
      });

    this.formArray = this.formBuilder.array([]);

    if (this.route.snapshot.data.inventario) {
      this.inventario = this.route.snapshot.data.inventario;
    }

    this.verificaTodosItensLancados();
    this.requestItems();
  }

  ngOnDestroy(): void {
    this.onDestroy$.next();
    this.onDestroy$.complete();
  }

  save(event: Event): void {
    event.preventDefault();

    if (this.formArray.invalid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }

    const buttons: Map<string, () => void> = new Map();
    buttons.set('SIM', () => this.confirmSave());
    buttons.set('NÃO', () => {});

    this.dialogService.showConfirmation({
      title: 'Confirmação',
      question: MensagemEnum.CONFIRMA_LANCAMENTO_INVENTARIO,
      anwsers: buttons,
    });
  }

  confirmSave(): void {
    const itens = this.formArray.value
      .filter((value: any) => value.quantidadeFisica != null && value.dataContagem != null)
      .map((value: any) => ({
          ...value,
          dataContagem: new Date(value.dataContagem),
        })
      );

    this.processing = true;
    this.inventarioItemService.salvar(itens)
      .pipe(finalize(() => this.processing = false))
      .subscribe(() => {
          this.dialogService.showNotification(MensagemEnum.REGISTRO_SALVO);
          this.verificaTodosItensLancados();
          this.requestItems();
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
  }

  // Data Events (begin)
  search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();
    if (!this.formGroupFilter.valid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }
    if (tabs) {
      tabs.selectedIndex = 1;
    }
    this.requestItems();
  }

  requestItems(event?: PageEvent): void {
    const value = {...this.formGroupFilter.value};

    this.filtro = {
      idInventario: this.inventario.id,
      idProduto: value.produto != null ? value.produto.codigo : null,
      codigoItem: value.codigoItem
    };

    // Se nenhum filtro informado pesquisa somente os itens não lancados
    if (this.filtro.idProduto == null && this.filtro.codigoItem == null) {
      this.filtro.idIsNull = true;
    }

    this.loading = true;
    this.infinityItemsMessage = '';
    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }

    this.inventarioItemService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({
        to: { pageSize: this.pageSize },
        changingPageSizeDefaultTo: this.pageSize,
        withPageEvent: event,
      })
    })
    .pipe(finalize(() => this.loading = false))
    .subscribe(
      (page: PageableResponseModel<InventarioItemModel>) => {
        this.total = page.total;
        this.clearFormArray();
        if (!page.items) {
          return this.dialogService.showNotification(MensagemEnum.NO_RESULTS_FOUND);
        }
        page.items.forEach(inventarioItem => this.formArray.push(this.createItem(inventarioItem)));
        if (this.scrollDispatcherSubscription) {
          this.infinityItemsMessage = `${this.formArray.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  imprimir(event: Event): void {
    event.preventDefault();

    const idItens = this.formArray.controls
      .map(control => control.get('idItem').value);

    this.processing = true;
    this.inventarioService.relatorioItem(this.inventario.id, idItens)
      .pipe(finalize(() => this.processing = false))
      .subscribe(
        (arquivo: ArquivoBase64Model) => {
          download(stringToBlob(atob(arquivo.base64), arquivo.mimeType), arquivo.filename);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
  }

  consolidar(event: Event): void {
    event.preventDefault();

    this.processing = true;
    this.inventarioService.consolidar(this.inventario.id)
      .pipe(finalize(() => this.processing = false))
      .subscribe(() => {
          this.dialogService.showNotification(MensagemEnum.INVENTARIO_CONSOLIDADO);
          this.router.navigate(['/rotina/inventario/listar']);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
  }
  // Data Events (end)

  displayAutocomplete(produtoDominio?: DominioItemModel): string {
    return produtoDominio ? produtoDominio.descricao : '';
  }

  isInvalid(controlName: string, control: AbstractControl): boolean {
    return control.get(controlName).invalid;
  }

  getErrorMessage(controlName: string, control: AbstractControl): string {
    const fc = control.get(controlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      fc.hasError('onlyDigits') ? MensagemEnum.ONLY_DIGITS :
      '';
  }

  private clearFormArray() {
    this.formArray = this.formBuilder.array([]);
  }

  private createItem(inventarioItem: InventarioItemModel): FormGroup {
    return this.formBuilder.group({
      id: [inventarioItem.id],
      idItem: [inventarioItem.idItem],
      idInventario: [this.inventario.id],
      codigoItem: [{value: inventarioItem.codigoItem, disabled: true}],
      item: [{value: inventarioItem.descricaoItem, disabled: true}],
      unidadeMedida: [{value: inventarioItem.unidadeMedida, disabled: true}],
      quantidadeFisica: [inventarioItem.quantidadeFisica, [NumberValidators.onlyDigits]],
      dataContagem: [inventarioItem.dataContagem != null ?
        formatDate(inventarioItem.dataContagem, 'yyyy-MM-ddThh:mm', this.locale) :
        null
      ],
    });
  }

  private verificaTodosItensLancados() {
    this.inventarioItemService.obterTodosPaginado({
      filter: {
        idInventario: this.inventario.id,
        idIsNull: true
      },
      ...requestItemsParams({
        to: { pageSize: 1 },
        changingPageSizeDefaultTo: 1
      }),
    })
    .subscribe(
      (page: PageableResponseModel<InventarioItemModel>) => this.todosItensLancados = !page.total,
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
}
