import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Observable } from 'rxjs';
import { map, tap, finalize, take, filter } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';

import { LocalJefazService } from '../../services/local-jefaz.service';
import { LocalJefazModel } from '../../models/local-jefaz.model';
import {MensagemEnum} from '../../../../../shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-local-jefaz-form',
  templateUrl: './local-jefaz.form.component.html',
  styleUrls: ['./local-jefaz.form.component.scss'],
})
export class LocalJefazFormComponent implements OnInit {

  formGroup: FormGroup;
  saving: boolean;

  private id: number;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private localJefazService: LocalJefazService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Cadastro de Local Jefaz');
    this.id = null;

    this.formGroup = this.formBuilder.group({
      descricao: [
        null,
        Validators.compose([
          Validators.maxLength(50),
          Validators.required,
        ])
      ],
      situacaoAtivo: [
        true,
        Validators.compose([
          Validators.maxLength(1),
          Validators.required,
        ])
      ],
      tipo: [
        null,
        Validators.compose([
          Validators.maxLength(1),
          Validators.required,
        ])
      ],
    });

    this.route.data.pipe(
      take(1),
      map((data: { item: LocalJefazModel }) => data.item),
      filter((item: LocalJefazModel) => !!item),
      tap((item: LocalJefazModel) => this.editItem(item)),
    ).subscribe();
  }

  editItem(item: LocalJefazModel): void {
    this.layoutService.setTitle(`Alteração de Local Jefaz`);
    this.id = item.id;
    this.formGroup.get('descricao').setValue(item.descricao);
    this.formGroup.get('situacaoAtivo').setValue(item.situacaoAtivo);
    this.formGroup.get('tipo').setValue(item.tipo);
  }

  save(event: Event): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    this.saving = true;
    this.request().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate(['/rotina/tabela/local-jefaz/listar']);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  private request(): Observable<LocalJefazModel> {
    if (this.id) {
      return this.localJefazService.alterar(<LocalJefazModel>{
        id: this.id,
        ...this.formGroup.value,
      }, this.id);
    }
    return this.localJefazService.incluir(this.formGroup.value);
  }
  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      '';
  }
}
