import { Component, OnInit, OnDestroy } from '@angular/core';
import { MatCheckboxChange } from '@angular/material';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { Observable, Subject } from 'rxjs';
import { finalize, take, map, filter, tap, takeUntil } from 'rxjs/operators';
import { trigger, transition, style, animate } from '@angular/animations';

import { ErrorModel, DominioItemModel } from '@datasus/core';
import { LayoutService, DialogService } from '@datasus/layout';
import { FormValidators } from '@datasus/forms';

import { ProdutoModel } from '../../models/produto.model';
import { EspecificacaoModel } from '../../models/especificacao.model';
import { ItemService } from '../../../../../shared/services/item.service';
import { ItemModel } from '../../models/item.model';
import { TipoDominioItemModel } from '../../models/tipo-dominio-item-model';
import { TipoItemService } from '../../services/tipo-item.service';
import { MarcaService } from '../../services/marca.service';
import { NaturezaDespesaService } from '../../../natureza-despesa/services/natureza-despesa.service';
import { TipoDispensacaoService } from '../../services/tipo-dispensacao.service';
import { TipoDemandaService } from 'src/app/shared/services/tipo-demanda.service';
import { ProgramaService } from '../../services/programa.service';
import { TipoDemandaModel } from 'src/app/shared/models/tipo-demanda.model';

@Component({
  selector: 'scodes-item-form',
  templateUrl: './item-form.component.html',
  styleUrls: ['./item-form.component.scss'],
  animations: [
    trigger('fadeInOut', [
      transition(':enter', [
        style({ opacity: 0 }),
        animate(500, style({ opacity: 1 }))
      ]),
      transition(':leave', [
        animate(500, style({ opacity: 0 }))
      ])
    ])
  ]
})
export class ItemFormComponent implements OnInit, OnDestroy {


  onDestroy$ = new Subject<void>();

  formGroup: FormGroup;
  saving: boolean;
  listaTipos: DominioItemModel[];
  listaMarcas: DominioItemModel[];
  isSiafisico = false;
  isTipoMarca = false;
  showInputNovaMarca = false;
  codigo: number;

  itemDemandaId1: TipoDominioItemModel;
  itemDemandaId4: TipoDominioItemModel;

  produto: ProdutoModel;
  especificacao: EspecificacaoModel;

  listaTipoDispensacao: DominioItemModel[];
  listaTipoDemanda: TipoDominioItemModel[];
  listaProgramas: DominioItemModel[];

  tiposDemandasSelecionados: TipoDominioItemModel[] = [];
  listaNaturezaDespesas: DominioItemModel[];
  dataRevisadoSiafisico: Date | string;
  dtRevisado: Date | string;

  idProduto: number;
  idEspecificacao: number;

  private idItem: number;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private formBuilder: FormBuilder,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private itemService: ItemService,
    private tipoItemService: TipoItemService,
    private marcaService: MarcaService,
    private naturezaDespesaService: NaturezaDespesaService,
    private tipoDemandaService: TipoDemandaService,
    private tipoDispensacaoService: TipoDispensacaoService,
    private programaService: ProgramaService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Cadastro do Item');

    this.formGroup = this.formBuilder.group({
      codigo: [{ value: null, disabled: true }],
      importado: [null],
      tipo: [null, [Validators.required]],
      marca: [null, [FormValidators.requiredIf(() => (this.isTipoMarca && !this.showInputNovaMarca))]],
      novaMarca: [null, [FormValidators.requiredIf(() => (this.isTipoMarca && this.showInputNovaMarca))]],
      programa: [null, [Validators.required]],
      codigoSiafisico: [null],
      descritivoSiafisico: [null, [FormValidators.requiredIf(() => this.isSiafisico)]],
      revisado: [false],
      minimoDispensacao: [null, [Validators.required]],
      minimoAlerta: [null, [Validators.required]],
      observacao: [null],
      tipoDispensacao: ['1', [Validators.required]],
      custo: [null],
      naturezaDespesa: [null]
    });

    this.formGroup.get('codigoSiafisico').valueChanges
      .pipe(takeUntil(this.onDestroy$))
      .subscribe((value: string) => {
        if (value && value.length >= 1) {
          this.isSiafisico = true;
          this.formGroup.get('revisado').setValue(true);
          this.dataRevisadoSiafisico = this.getDataRevisado(false);
        } else {
          this.isSiafisico = false;
          this.formGroup.get('revisado').setValue(false);
          this.dataRevisadoSiafisico = this.getDataRevisado(true);
        }
      });

    this.formGroup.get('revisado').valueChanges
      .pipe(takeUntil(this.onDestroy$))
      .subscribe((value: boolean) => {
        if (value) {
          this.dataRevisadoSiafisico = this.getDataRevisado(false);
        } else {
          this.dataRevisadoSiafisico = this.getDataRevisado(true);
        }
      });

    this.formGroup.get('tipo').valueChanges
      .pipe(takeUntil(this.onDestroy$))
      .subscribe((value) => {
        this.isTipoMarca = (value === '4');
        this.formGroup.get('marca').setValue(null);
        this.formGroup.get('novaMarca').setValue(null);
      });

    this.getDominios();

    this.route.data.pipe(
      take(1),
      map((data: { produto: ProdutoModel }) => data.produto),
      filter((produto: ProdutoModel) => !!produto),
      tap((produto: ProdutoModel) => this.getProduto(produto)),
    ).subscribe();

    this.route.data.pipe(
      take(1),
      map((data: { especificacao: EspecificacaoModel }) => data.especificacao),
      filter((especificacao: EspecificacaoModel) => !!especificacao),
      tap((especificacao: EspecificacaoModel) => this.getEspecificacao(especificacao)),
    ).subscribe();

    this.route.data.pipe(
      take(1),
      map((data: { item: ItemModel }) => data.item),
      filter((item: ItemModel) => !!item),
      tap((item: ItemModel) => this.editItem(item)),
    ).subscribe();

  }

  ngOnDestroy(): void {
    this.onDestroy$.next();
    this.onDestroy$.complete();
  }

  getDataRevisado(isNull: boolean) {
    if (isNull) {
      return this.dataRevisadoSiafisico ? this.dtRevisado : null;
    }
    return this.dataRevisadoSiafisico ? this.dataRevisadoSiafisico : new Date();
  }

  getProduto(produto: ProdutoModel): void {
    this.idProduto = produto.id;
    this.produto = produto;
  }

  getEspecificacao(especificacao: EspecificacaoModel): void {
    this.idEspecificacao = especificacao.id;
    this.especificacao = especificacao;

  }

  editItem(item: ItemModel): void {
    this.layoutService.setTitle(`Alteração do Item`);
    this.idItem = item.id;
    this.dataRevisadoSiafisico = item.dataRevisadoSiafisico;
    this.formGroup.get('codigo').setValue(item.codigo);
    this.formGroup.get('importado').setValue(item.importado);
    this.formGroup.get('tipo').setValue(String(item.tipoItem.id));
    this.formGroup.get('codigoSiafisico').setValue(item.codigoSiafisico);
    this.formGroup.get('marca').setValue(item.marca != null ?
      String(item.marca.id) : null);
    this.formGroup.get('programa').setValue(String(item.programa.id));
    this.formGroup.get('minimoDispensacao').setValue(item.minimoDispensacao);
    this.formGroup.get('minimoAlerta').setValue(item.minimoAlerta);
    this.formGroup.get('custo').setValue(item.custo);
    this.formGroup.get('naturezaDespesa').setValue(item.naturezaDespesa != null ?
      String(item.naturezaDespesa.id) : null);
    this.formGroup.get('tipoDispensacao').setValue(String(item.tipoDispensacao.id));
    this.formGroup.get('descritivoSiafisico').setValue(item.descricaoSiafisico);
    this.formGroup.get('observacao').setValue(item.observacao);
    this.setarNaListaDemanda(item.tipoDemandas);
  }

  getDominios() {
    this.tipoItemService.obterDominio()
      .subscribe(
        (response) => this.listaTipos = this.produto.categoria.id === 1 ?
          response.filter(itemList => itemList.descricao !== 'Manipulado' && itemList.descricao !== 'Homeopático') : response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message));

    this.marcaService.obterDominio()
      .subscribe(
        (response) => this.listaMarcas = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message));

    this.naturezaDespesaService.obterDominio()
      .subscribe(
        (response) => this.listaNaturezaDespesas = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message));

    this.tipoDemandaService.obterDominio()
      .subscribe(
        (response) => {

          let m: TipoDominioItemModel;
          response.forEach((item) => {
            if (item.codigo === '1') { this.itemDemandaId1 = item; }
            if (item.codigo === '4') { this.itemDemandaId4 = item; }
          });
          m = {
            codigo: this.itemDemandaId1.codigo,
            descricao: `${this.itemDemandaId1.descricao}/${this.itemDemandaId4.descricao}`,
            checked: false
          };
          this.listaTipoDemanda = response.filter(itemList => {
            return this.itemDemandaId1.codigo !== itemList.codigo && this.itemDemandaId4.codigo !== itemList.codigo;
          });
          this.listaTipoDemanda.unshift(m);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message));

    this.tipoDispensacaoService.obterDominio()
      .subscribe(
        (response) => this.listaTipoDispensacao = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message));

    this.programaService.obterDominio()
      .subscribe(
        (response) => this.listaProgramas = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message));
  }


  save(event: Event): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    this.saving = true;
    this.request().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate([`/rotina/tabela/produto/listar`]);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }

  setarNaListaDemanda(tiposDemandas: TipoDemandaModel[]) {
    const event: MatCheckboxChange = new MatCheckboxChange();
    setTimeout(() => {
      if (tiposDemandas && tiposDemandas.length > 0) {
        event.checked = true;
        tiposDemandas.forEach((itemListaResp) => {
          const tempObj: TipoDominioItemModel = {
            codigo: itemListaResp.id.toString(),
            descricao: itemListaResp.descricao
          };
          this.listaTipoDemanda.forEach((itemDaLista) => {
            if (itemListaResp.id === 1 || itemListaResp.id === 4) {
              if (itemDaLista.codigo === '1' && !itemDaLista.checked) {
                this.addOrRemoveTipo(event, tempObj);
                return itemDaLista.checked = true;
              }
            } else {
              if (itemListaResp.descricao === itemDaLista.descricao && !itemDaLista.checked) {
                this.addOrRemoveTipo(event, tempObj);
                return itemDaLista.checked = true;
              }
            }
          });

        });
      }
    }, 500);
  }

  changeInput(event: MouseEvent) {
    event.preventDefault();
    event.stopPropagation();
    this.showInputNovaMarca = !this.showInputNovaMarca;
    this.formGroup.get('marca').setValue(null);
    this.formGroup.get('novaMarca').setValue(null);
  }

  addOrRemoveTipo(event: MatCheckboxChange, tipo: TipoDominioItemModel) {
    if (event.checked) {
      if (tipo.codigo === '1') {
        this.tiposDemandasSelecionados.push(this.itemDemandaId1);
        return this.tiposDemandasSelecionados.push(this.itemDemandaId4);
      }
      this.tiposDemandasSelecionados.push(tipo);
    } else {
      if (tipo.codigo === '1') {
        this.tiposDemandasSelecionados = this.tiposDemandasSelecionados.filter(itemList => {
          return this.itemDemandaId1.codigo !== itemList.codigo && this.itemDemandaId4.codigo !== itemList.codigo;
        });
      }
      this.tiposDemandasSelecionados = this.tiposDemandasSelecionados.filter(itemList => {
        return tipo.codigo !== itemList.codigo;
      });
    }
  }

  private request(): Observable<ItemModel> {
    const value = {...this.formGroup.value};

    const item: ItemModel = {
      importado: !!value.importado,
      marca: null,
      minimoAlerta: Number(value.minimoAlerta),
      minimoDispensacao: Number(value.minimoDispensacao),
      observacao: value.observacao,
      codigoSiafisico: value.codigoSiafisico,
      dataRevisadoSiafisico: new Date(this.dataRevisadoSiafisico),
      descricaoSiafisico: value.descritivoSiafisico,
      tipoDispensacao: { id: Number(value.tipoDispensacao) },
      dataImportado: value.importado ? new Date() : null,
      tipoItem: { id: Number(value.tipo) },
      programa: { id: Number(value.programa) },
      especificacao: { id: this.especificacao.id },
      custo: value.custo != null ? Number(value.custo) : null,
      naturezaDespesa: value.naturezaDespesa != null ? { id: Number(value.naturezaDespesa) } : null,
      tipoDemandas: this.tiposDemandasSelecionados.map(v => ({ id: Number(v.codigo) }))
    };

    if (value.marca) {
      item.marca = { id: Number(value.marca) };
    } else if (value.novaMarca) {
      item.marca = { nome: value.novaMarca };
    }

    if (this.idItem) {
      return this.itemService.alterar(<ItemModel>{
        id: this.idItem,
        ...item,
      }, this.idItem);
    }
    return this.itemService.incluir(item);
  }

}
