import { Component, OnInit, ViewChild, ChangeDetectorRef, OnDestroy } from '@angular/core';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { MatPaginator, MatTabGroup, PageEvent, MatAutocompleteSelectedEvent, MatSelectChange } from '@angular/material';
import { Router } from '@angular/router';
import { FormGroup, FormBuilder } from '@angular/forms';
import { ScrollDispatcher, CdkScrollable } from '@angular/cdk/overlay';
import { Observable, Subscription, Subject } from 'rxjs';
import { tap, finalize, debounceTime, takeUntil } from 'rxjs/operators';

import { DialogService, LayoutService } from '@datasus/layout';
import { ErrorModel, requestItemsParams, PageableResponseModel, DominioItemModel } from '@datasus/core';

import { ItemModel } from '../../models/item.model';
import { CategoriaService } from '../../services/categoria.service';
import { GrupoEspecificacaoService } from '../../services/grupo-especificacao.service';
import { ProdutoService } from '../../services/produto.service';
import { ProdutoModel } from '../../models/produto.model';
import { EspecificacaoService } from '../../services/especificacao.service';
import { ProdutoFilterModel } from '../../models/produto-filter.model';
import { MensagemEnum } from 'src/app/shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-list',
  templateUrl: './list.component.html',
  styleUrls: ['./list.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class ListComponent implements OnInit, OnDestroy {

  filtro: ProdutoFilterModel;
  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

  // Retrived Data Source
  items: ProdutoModel[] = [];
  infinityItems: ProdutoModel[] = [];

  // Data Table Config
  displayedColumns: string[] = [
    'produto',
    'actions',
  ];

  // Viewport Observable
  onMobile$: Observable<boolean>;
  onDestroy$ = new Subject<void>();

  // Form Group
  formGroup: FormGroup;

  // Data Table Controls (begin)
  loading = false;
  infinityItemsMessage = '';
  total = 0;
  pageSize = 0;
  // Data Table Controls (end)

  // View Controls (begin)
  viewChars = false;
  actionButtonCtrl: Map<ItemModel, boolean> = new Map();
  // View Controls (end)

  listaCategorias: DominioItemModel[];
  listaGrupos: DominioItemModel[];
  listaProdutos: DominioItemModel[];
  listaEspecificacoes: DominioItemModel[];
  loadingGrupo = false;
  isProduto = false;

  private scrollDispatcherSubscription: Subscription;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private scrollDispatcher: ScrollDispatcher,
    private changeDetectorRef: ChangeDetectorRef,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private produtoService: ProdutoService,
    private categoriaService: CategoriaService,
    private grupoEspecificacaoService: GrupoEspecificacaoService,
    private especificacaoService: EspecificacaoService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Listagem de Produto');

    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );

    this.formGroup = this.formBuilder.group({
      categoria: [null],
      grupo: [null],
      produto: [null],
      especificacao: [null],
      codigo: [null],
    });

    this.formGroup.get('produto').disable();

    this.categoriaService.obterDominio()
      .subscribe(
        (response) => this.listaCategorias = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );

    this.formGroup.get('produto').valueChanges
      .pipe(
        debounceTime(350),
        takeUntil(this.onDestroy$),
      )
      .subscribe(
        (termo) => {
          if (typeof termo === 'string') {
            return this.produtoService
              .obterAutocompleteDominio(termo, this.formGroup.get('categoria').value)
              .subscribe(
                (response) => this.listaProdutos = response,
                (error: ErrorModel) => this.dialogService.showNotification(error.message));
          }
        });
  }

  ngOnDestroy(): void {
    this.onDestroy$.next();
    this.onDestroy$.complete();
    this.scrollRegister(false);
    this.infinityItems = [];
  }

  displayFn(produtoDominio?: DominioItemModel): string | undefined {
    return produtoDominio ? produtoDominio.descricao : undefined;
  }

  obterEspecificacoes(event: MatAutocompleteSelectedEvent): void {
    if (event.option.value && event.option.value.codigo) {
      this.especificacaoService
        .obterDominio(event.option.value.codigo)
        .subscribe(
          (response) => this.listaEspecificacoes = response,
          (error: ErrorModel) => this.dialogService.showNotification(error.message));
    }
    this.formGroup.get('grupo').setValue(null);
    this.formGroup.get('especificacao').setValue(null);
    this.formGroup.get('codigo').setValue(null);
  }

  obterGrupos(idCategoria: number): void {
    this.loadingGrupo = true;
    this.grupoEspecificacaoService
      .obterDominio(idCategoria)
      .pipe(finalize(() => this.loadingGrupo = false))
      .subscribe(
        (response) => {
          this.formGroup.get('produto').enable();
          this.listaGrupos = response;
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  // Data Events (begin)
  search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    if (tabs) {
      tabs.selectedIndex = 1;
    }
    this.infinityItems = [];
    this.requestItems();
  }

  requestItems(event?: PageEvent): void {
    this.filtro = {
      id: this.formGroup.value.produto ? this.formGroup.value.produto.codigo : null,
      nome: this.formGroup.value.produto ? this.formGroup.value.produto.descricao : null,
      idCategoria: this.formGroup.value.categoria,
      idEspecificacao: this.formGroup.value.especificacao,
      idGrupoEspecificacao: this.formGroup.value.grupo,
      codigoItem: this.formGroup.value.codigo,
    };
    this.loading = true;
    this.infinityItemsMessage = '';
    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }
    this.produtoService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({ to: this, withPageEvent: event }),
    }).pipe(
      finalize(() => this.loading = false)
    ).subscribe(
      (page: PageableResponseModel<ProdutoModel>) => {
        this.total = page.total;
        this.items = page.items;
        if (!page.items) {
          return this.dialogService.showNotification(MensagemEnum.NO_RESULTS_FOUND);
        }
        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  changeCategoria(event: MatSelectChange): void {
    this.listaGrupos = [];
    this.formGroup.get('grupo').setValue(null);
    this.formGroup.get('produto').setValue(null);
    this.formGroup.get('especificacao').setValue(null);
    this.formGroup.get('codigo').setValue(null);
    this.formGroup.get('produto').disable();
    if (event.value != null) {
      this.obterGrupos(event.value);
    }
  }

  changeGrupo(event: MatSelectChange): void {
    this.formGroup.get('produto').setValue(null);
    this.formGroup.get('especificacao').setValue(null);
    this.formGroup.get('codigo').setValue(null);
  }

  inputCodigo(event: any): void {
    if (event.target.value != null && event.target.value.length > 0) {
      this.formGroup.get('categoria').setValue(null);
      this.formGroup.get('grupo').setValue(null);
      this.formGroup.get('produto').setValue(null);
      this.formGroup.get('especificacao').setValue(null);
      this.formGroup.get('produto').disable();
    }
  }

  // Data Events (end)

  // Item Actions (begin)
  edit(item: ItemModel) {
    this.router.navigate([`/rotina/tabela/produto/${item.id}/editar`]);
  }

  delete(item: ItemModel) {
    const buttons: Map<string, () => void> = new Map();
    buttons.set('Sim', () => {
      this.layoutService.loading();
      console.log('Ação #2', item);
      this.produtoService.excluir(item.id).pipe(
        finalize(() => this.layoutService.loaded()),
      ).subscribe(
        () => {
          this.dialogService.showNotification('Registro excluído com sucesso');
          this.requestItems();
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
    });
    buttons.set('Não', () => console.log('Abortou a ação #2', item));

    this.dialogService.showConfirmation({
      question: 'Deseja mesmo remover?',
      title: 'Cuidado!',
      anwsers: buttons,
      type: 'warn',
    });
  }
  // Item Actions (end)

  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }



}
