import { Component, OnInit, ViewChild, OnDestroy, ChangeDetectorRef } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { take, map, filter, tap, finalize } from 'rxjs/operators';
import { Observable, Subscription } from 'rxjs';
import { MatPaginator } from '@angular/material';
import { CdkScrollable, ScrollDispatcher } from '@angular/cdk/overlay';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel, DominioItemModel, requestItemsParams, PageableResponseModel } from '@datasus/core';

import { ProdutoService } from '../../services/produto.service';
import { ProdutoModel } from '../../models/produto.model';
import { CategoriaService } from '../../services/categoria.service';
import { TipoControladoService } from '../../services/tipo-controlado.service';
import { EspecificacaoModel } from '../../models/especificacao.model';
import { ItemModel } from '../../models/item.model';
import { EspecificacaoService } from '../../services/especificacao.service';

@Component({
  selector: 'scodes-produto.form',
  templateUrl: './produto-form.component.html',
  styleUrls: ['./produto-form.component.scss']
})
export class ProdutoFormComponent implements OnInit, OnDestroy {

  formGroup: FormGroup;
  saving: boolean;

  listaCategorias: DominioItemModel[];
  listaTiposControlados: DominioItemModel[];
  tipoControlado: string;

  // Data Table Config
  displayedColumns: string[] = [
    'descricao',
    'apresentacao',
    'grupo',
    'actions',
  ];

  filtro: any;

  // Retrived Data Source
  items: EspecificacaoModel[] = [];
  infinityItems: EspecificacaoModel[] = [];

  // Viewport Observable
  onMobile$: Observable<boolean>;

  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

  // Data Table Controls (begin)
  loading = false;
  infinityItemsMessage = '';
  total = 0;
  pageSize = 0;
  // Data Table Controls (end)

  // View Controls (begin)
  viewChars = false;
  actionButtonCtrl: Map<EspecificacaoModel, boolean> = new Map();
  // View Controls (end)

  idProduto: number;
  private idCategoria: number;

  private scrollDispatcherSubscription: Subscription;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private produtoService: ProdutoService,
    private categoriaService: CategoriaService,
    private tipoControladoService: TipoControladoService,
    private changeDetectorRef: ChangeDetectorRef,
    private scrollDispatcher: ScrollDispatcher,
    private especificacaoService: EspecificacaoService,

  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Cadastro do Produto');
    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );

    this.idProduto = null;

    this.formGroup = this.formBuilder.group({
      categoria: [null, [Validators.required]],
      produto: [null, [Validators.required]],
      tipoControlado: [null, [Validators.required]]
    });

    this.categoriaService.obterDominio()
      .subscribe(
        (value) => this.listaCategorias = value,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );

    this.tipoControladoService.obterDominio()
      .subscribe(
        (value) => this.listaTiposControlados = value,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );

    this.formGroup.get('categoria').valueChanges
      .subscribe((value) => {
        if (value !== '1') {
          this.tipoControlado = '5';
          this.formGroup.get('tipoControlado').setValue('5');
          this.formGroup.get('tipoControlado').disable();
        } else {
          this.tipoControlado = null;
          this.formGroup.get('tipoControlado').setValue(null);
          this.formGroup.get('tipoControlado').enable();
        }

      });


    this.route.data.pipe(
      take(1),
      map((data: { produto: ProdutoModel }) => data.produto),
      filter((produto: ProdutoModel) => !!produto),
      tap((produto: ProdutoModel) => this.editItem(produto)),
    ).subscribe();


  }

  ngOnDestroy(): void {
    this.scrollRegister(false);
    this.infinityItems = [];
  }

  editItem(produto: ProdutoModel): void {
    this.layoutService.setTitle(`Alteração do Produto`);
    this.idProduto = produto.id;
    this.formGroup.get('categoria').setValue(`${produto.categoria.id}`);
    this.idCategoria = produto.categoria.id;
    this.formGroup.get('categoria').disable();
    this.formGroup.get('produto').setValue(produto.nome);
    this.formGroup.get('tipoControlado').setValue(`${produto.tipoControlado.id}`);
    this.requestItems(this.idProduto);
  }

  validateAndSave(event: Event): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    this.saving = true;
    this.save().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      (resp) => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate([`/rotina/tabela/produto/${resp.id}/especificacao/inserir`]);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }

  // Data Events (begin)

  requestItems(id: number): void {
    this.filtro = { idProduto: id };
    this.loading = true;
    this.infinityItemsMessage = '';
    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }
    this.especificacaoService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({ to: this }),
    }).pipe(
      finalize(() => {
        this.loading = false;
        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      })
    ).subscribe(
      (page: PageableResponseModel<EspecificacaoModel>) => {
        this.total = page.total;
        this.items = page.items;
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
  // Data Events (end)

  novaEspecificacao() {
    this.router.navigate([`/rotina/tabela/produto/${this.idProduto}/especificacao/inserir`]);
  }

  // Item Actions (begin)
  edit(especificacao: EspecificacaoModel) {
    this.router.navigate([`/rotina/tabela/produto/${this.idProduto}/especificacao/${especificacao.id}/editar`]);
  }

  delete(item: EspecificacaoModel) {
    const buttons: Map<string, () => void> = new Map();
    buttons.set('Sim', () => {
      this.layoutService.loading();
      console.log('Ação #2', item);
      this.especificacaoService.excluir(item.id).pipe(
        finalize(() => this.layoutService.loaded()),
      ).subscribe(
        () => {
          this.dialogService.showNotification('Registro excluído com sucesso');
          this.requestItems(this.idProduto);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
    });
    buttons.set('Não', () => console.log('Abortou a ação #2', item));

    this.dialogService.showConfirmation({
      question: 'Deseja mesmo remover?',
      title: 'Cuidado!',
      anwsers: buttons,
      type: 'warn',
    });
  }
  // Item Actions (end)

  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }



  private save(): Observable<ProdutoModel> {
    const obj: ProdutoModel = {
      categoria: {
        id: Number(this.formGroup.get('categoria').value ? this.formGroup.get('categoria').value : this.idCategoria)
      },
      nome: this.formGroup.get('produto').value,
      tipoControlado: {
        id: Number(this.formGroup.get('tipoControlado').value)
      }
    };

    if (this.idProduto) {
      return this.produtoService.alterar(<ProdutoModel>{
        id: this.idProduto,
        ...obj,
      }, this.idProduto);
    }
    return this.produtoService.incluir(obj);
  }

}
