import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MatAutocompleteSelectedEvent } from '@angular/material';
import { ActivatedRoute, Router } from '@angular/router';
import { ErrorModel } from '@datasus/core';
import { DialogService, LayoutService } from '@datasus/layout';
import { Observable } from 'rxjs';
import { filter, finalize, map, take, tap } from 'rxjs/operators';
import { ProfissionalModel } from 'src/app/shared/models/profissional.model';
import { TipoProfissaoModel } from 'src/app/shared/models/tipo-profissao.model';
import { UfModel } from 'src/app/shared/models/uf.model';
import { ProfissionalService } from 'src/app/shared/services/profissional.service';
import { TipoProfissaoService } from 'src/app/shared/services/tipo-profissao.service';
import { UfService } from 'src/app/shared/services/uf.service';




@Component({
  selector: 'scodes-profissional-form',
  templateUrl: './profissional.form.component.html',
  styleUrls: ['./profissional.form.component.scss'],
})
export class ProfissionalFormComponent implements OnInit {

  formGroup: FormGroup;
  isSalvo: boolean;
  listaTipoProfissao: TipoProfissaoModel[];
  listaUfs: UfModel[];
  listaProfissoesCamera = [
    { id: 4, descricao: 'Advogado'},
    { id: 5, descricao: 'Defensor público'},
    { id: 6, descricao: 'Promotor'},
    { id: 7, descricao: 'Juiz'}
  ];
  descricao = '';
  listaStatus = [
    { label: 'Ativo', value: true },
    { label: 'Inativo', value: false }
  ];

  private id: number;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private profissionalService: ProfissionalService,
    private ufService: UfService,
    private tipoProfissaoService: TipoProfissaoService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Cadastro de Profissional');
    this.id = null;

    this.formGroup = this.formBuilder.group({
      status: [true, [Validators.required]],
      nome: [null, [Validators.required]],
      idTipoProfissao: [null, [Validators.required]],
      uf: [null, [Validators.required]],
      camara: [null],
      conselho: [null],
      motivo: [null],
      titulo: [null],
      email: [null, Validators.compose([Validators.email, Validators.minLength(5)])],
    });

    this.route.data.pipe(
      take(1),
      map((data: { item: ProfissionalModel }) => data.item),
      filter((item: ProfissionalModel) => !!item),
      tap((item: ProfissionalModel) => this.editItem(item)),
    ).subscribe();

    this.ufService.obterTodos()
    .subscribe(
      (response) => this.listaUfs = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );

    this.tipoProfissaoService.obterTodos().subscribe(
      (response: TipoProfissaoModel[]) => this.listaTipoProfissao = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message)
    );

    this.formGroup.get('email').disable();

    this.formGroup.get('idTipoProfissao').valueChanges
    .subscribe((valor) => {
      if (valor === 9) {
        this.formGroup.get('email').enable();
      } else {
        this.formGroup.get('email').setValue(null);
        this.formGroup.get('email').disable();
      }
    });

    this.formGroup.get('camara').disable();

    this.formGroup.get('idTipoProfissao').valueChanges
    .subscribe((valor) => {
      for (let index = 0; index < this.listaProfissoesCamera.length; index++) {
        const tipo = this.listaProfissoesCamera[index];
        console.log(tipo.id === valor);
        if (tipo.id === valor) {
          this.formGroup.get('camara').enable();
           break;
        } else {
          this.formGroup.get('camara').setValue(null);
          this.formGroup.get('camara').disable();
        }
      }
    });

  }

  editItem(item: ProfissionalModel): void {

    this.layoutService.setTitle('Alteração de Profissional');
    this.id = item.id;
    this.formGroup.get('status').setValue(item.status);
    this.formGroup.get('nome').setValue(item.nome);
    this.formGroup.get('idTipoProfissao').setValue(item.tipoProfissao.id);
    this.formGroup.get('uf').setValue(item.uf);
    this.formGroup.get('camara').setValue(item.camara);
    this.formGroup.get('conselho').setValue(item.conselho);
    this.formGroup.get('motivo').setValue(item.motivo);
    this.formGroup.get('titulo').setValue(item.titulo);
    this.formGroup.get('email').setValue(item.email);
  }


  submitProfissional(event: Event) {
    event.preventDefault();

    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos obrigatórios');
    }

    this.isSalvo = true;
    this.request().pipe(
      finalize(() => this.isSalvo = false)
      ).subscribe(
        () => {
          this.dialogService.showNotification('Registro salvo com sucesso');
          this.formGroup.reset();
          this.router.navigate(['/rotina/tabela/profissional/listar']);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
        );

  }

  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }

  compareUf(option: UfModel, selectValue: UfModel): boolean {
    return !!selectValue && selectValue.id === option.id;
  }

  private request(): Observable<ProfissionalModel> {
    const postData: ProfissionalModel  = {
      status: this.formGroup.value.status,
      nome: this.formGroup.value.nome,
      tipoProfissao: {
        id: this.formGroup.value.idTipoProfissao,
      } as TipoProfissaoModel,
      uf: {
        id: this.formGroup.value.uf,
      } as UfModel,
      camara: this.formGroup.value.camara,
      conselho: this.formGroup.value.conselho,
      motivo: this.formGroup.value.motivo,
      titulo: this.formGroup.value.titulo,
      email: this.formGroup.value.email,
    };

    if (this.id) {
      return this.profissionalService.alterar(<ProfissionalModel>{
        id: this.id,
        ...postData,
      }, this.id);
    }
    return this.profissionalService.incluir(postData);
  }

  obterUfs(event: MatAutocompleteSelectedEvent) {
    if (event.option.value && event.option.value.descricao) {
      this.ufService
        .obterPorId(event.option.value.id)
        .subscribe(
          (response) => this.formGroup.get('uf').setValue(response),
          (error: ErrorModel) => this.dialogService.showNotification(error.message));
    }
  }
}
