import { animate, query, style, transition, trigger } from '@angular/animations';
import { CdkScrollable, ScrollDispatcher } from '@angular/cdk/overlay';
import { ChangeDetectorRef, Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MatAutocompleteSelectedEvent, MatTabGroup } from '@angular/material';
import { MatPaginator, PageEvent } from '@angular/material/paginator';
import { Router } from '@angular/router';
import { ErrorModel, PageableResponseModel, requestItemsParams } from '@datasus/core';
import { DialogService, LayoutService } from '@datasus/layout';
import { Observable, Subscription } from 'rxjs';
import { finalize, tap } from 'rxjs/operators';
import { UfModel } from 'src/app/shared/models/uf.model';
import { UfService } from 'src/app/shared/services/uf.service';
import { ProfissionalModel } from '../../../../../shared/models/profissional.model';
import { ProfissionalService } from '../../../../../shared/services/profissional.service';
import { TipoProfissaoService } from 'src/app/shared/services/tipo-profissao.service';
import { TipoProfissaoModel } from 'src/app/shared/models/tipo-profissao.model';

@Component({
  selector: 'scodes-profissional-list',
  templateUrl: './profissional.list.component.html',
  styleUrls: ['./profissional.list.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class ProfissionalListComponent implements OnInit, OnDestroy {

  filtro: ProfissionalModel;
  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

  // Retrived Data Source
  items: ProfissionalModel[] = [];
  infinityItems: ProfissionalModel[] = [];

  // Data Table Config
  displayedColumns: string[] = [
    'nome',
    'conselho',
    'actions',
  ];

  // Viewport Observable
  onMobile$: Observable<boolean>;

  // Form Group
  formGroup: FormGroup;

  // Data Table Controls (begin)
  loading = false;
  infinityItemsMessage = '';
  total = 0;
  pageSize = 0;
  // Data Table Controls (end)

  // View Controls (begin)
  viewChars = false;
  actionButtonCtrl: Map<ProfissionalModel, boolean> = new Map();
  // View Controls (end)

  private scrollDispatcherSubscription: Subscription;

  listaUfs: UfModel[];
  listaTipoProfissao: TipoProfissaoModel[];

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private scrollDispatcher: ScrollDispatcher,
    private changeDetectorRef: ChangeDetectorRef,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private modeloService: ProfissionalService,
    private ufService: UfService,
    private tipoProfissaoService: TipoProfissaoService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Listagem de Profissional');
    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );
    this.formGroup = this.formBuilder.group({
      nome: [null,[Validators.required]],
      conselho: [null, [Validators.required]],
      idTipoProfissao: [null],
      uf: [null],
      email: [null, Validators.compose([Validators.email, Validators.minLength(5)])],
    });

    this.ufService.obterTodos()
    .subscribe(
      (response) => this.listaUfs = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );

    this.tipoProfissaoService.obterTodos().subscribe(
      (response: TipoProfissaoModel[]) => this.listaTipoProfissao = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message)
    );

    this.formGroup.get('email').disable();

    this.formGroup.get('idTipoProfissao').valueChanges
    .subscribe((valor) => {
      if (valor === 9) {
        this.formGroup.get('email').enable();
      } else {
        this.formGroup.get('email').setValue(null);
        this.formGroup.get('email').disable();
      }
    });
  }

  ngOnDestroy(): void {
    this.scrollRegister(false);
    this.infinityItems = [];
  }

  // Data Events (begin)
  search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    if (tabs) {
      tabs.selectedIndex = 1;
    }
    this.infinityItems = [];
    this.requestItems();
  }

  requestItems(event?: PageEvent): void {
    this.filtro = this.formGroup.value;
    this.loading = true;
    this.infinityItemsMessage = '';
    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }
    this.modeloService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({ to: this, withPageEvent: event }),
    }).pipe(
      finalize(() => {
        this.loading = false;
        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      })
    ).subscribe(
      (page: PageableResponseModel<ProfissionalModel>) => {
        console.log(' --->', page);
        this.total = page.total;
        this.items = page.items;
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
  // Data Events (end)

  edit(item: ProfissionalModel) {
    this.router.navigate(['/rotina/tabela/profissional/editar', item.id]);
  }

  delete(item: ProfissionalModel) {
    const buttons: Map<string, () => void> = new Map();
    buttons.set('Sim', () => {
      this.layoutService.loading();
      console.log('Ação #2', item);
      this.modeloService.excluir(item.id).pipe(
        finalize(() => this.layoutService.loaded()),
      ).subscribe(
        () => {
          this.dialogService.showNotification('Registro excluído com sucesso');
          this.requestItems();
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
    });
    buttons.set('Não', () => console.log('Abortou a ação #2', item));

    this.dialogService.showConfirmation({
      question: 'Deseja mesmo remover?',
      title: 'Cuidado!',
      anwsers: buttons,
      type: 'warn',
    });
  }
  // Item Actions (end)

  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }

  obterUfs(event: MatAutocompleteSelectedEvent) {
    if (event.option.value && event.option.value.descricao) {
      this.ufService
        .obterPorId(event.option.value.id)
        .subscribe(
          (response) => this.formGroup.get('uf').setValue(response),
          (error: ErrorModel) => this.dialogService.showNotification(error.message));
    }
  }

  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }

  displayFn(ufDominio?: UfModel): string | undefined {
    return ufDominio ? ufDominio.descricao : undefined;
  }
}
