import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { FormGroup } from '@angular/forms';
import { take, map, filter, tap, finalize } from 'rxjs/operators';
import { Subscription, Observable } from 'rxjs';
import { MatDialog, MatTableDataSource, MatPaginator } from '@angular/material';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';

import { ItemModel } from '../../../produto/models/item.model';
import { ItemService } from '../../../../../shared/services/item.service';
import { DialogPesquisaItemComponent } from 'src/app/shared/components/dialog-pesquisa-item/dialog-pesquisa-item.component';
import { MensagemEnum } from 'src/app/shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-pesquisa-item',
  templateUrl: './pesquisa-item.component.html',
  styleUrls: ['./pesquisa-item.component.scss']
})
export class PesquisaItemComponent implements OnInit {

  item: ItemModel;
  formGroup: FormGroup;
  filtro: any;
  loading = false;

  // Retrived Data Source
  items: ItemModel[] = [];
  dataSource = new MatTableDataSource(this.items);


  // Data Table Config
  displayedColumns: string[] = [
    'codigo',
    'produto',
    'actions'
  ];

  @ViewChild(MatPaginator) paginator: MatPaginator;

  actionButtonCtrl: Map<ItemModel, boolean> = new Map();

  constructor(
    private route: ActivatedRoute,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private itemService: ItemService,
    private matDialog: MatDialog,
  ) { }

  ngOnInit() {
    this.layoutService.setTitle('Pesquisa de item');

    this.route.data.pipe(
      take(1),
      map((data: { item: ItemModel }) => data.item),
      filter((item: ItemModel) => !!item),
      tap((item: ItemModel) => this.item = item),
    ).subscribe();

    this.itemService.obterItensSimilaresPorIdItem(this.item.id)
      .subscribe(
        (response) => {
          this.items = response;
          this.dataConvert();
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message));


  }

  delete(item: ItemModel): void {
    this.items = this.items.filter((oldItem) => item.id !== oldItem.id);
    this.confirmation(false);
  }

  deleteAll() {
    this.items = [];
    this.confirmation(true);
  }


  showModalItem(): void {
    this.matDialog
      .open(DialogPesquisaItemComponent, { width: '80vw' })
      .afterClosed()
      .pipe(filter(value => !!value))
      .subscribe(result => {
        if (result.id === this.item.id) {
          return this.dialogService.showNotification(MensagemEnum.ITEM_SELECIONADO_IGUAL_ATUAL);
        }
        if (this.items.some(itemAtual => itemAtual.id === result.id)) {
          return this.dialogService.showNotification(MensagemEnum.ITEM_JA_NA_LISTA);
        }
        this.addItem(result);
      });
  }

  save(): Observable<ItemModel[]> {
    const listIds: number[] = this.items.map((item) => item.id);
    return this.itemService.alterarSimilaridadePorIdItem(listIds, this.item.id);
  }


  confirmation(todos: boolean): void {
    const buttons: Map<string, () => void> = new Map();
    let question = MensagemEnum.CONFIRMA_REMOCAO_ITEM_SIMILAR;
    if (todos) {
      question = MensagemEnum.CONFIRMA_REMOCAO_TODOS_ITENS_SIMILARES;
    }

    buttons.set('SIM', () => {
      this.loading = true;
      this.save()
        .pipe(finalize(() => this.loading = false))
        .subscribe(
          () => {
            this.dataConvert();
            this.dialogService.showNotification(
              todos ? MensagemEnum.TODOS_ITENS_REMOVIDOS : MensagemEnum.ITEM_REMOVIDO
            );
          },
          (error: ErrorModel) => this.dialogService.showAlert(error.message)
        );
    });
    buttons.set('NÃO', () => console.log('To bad!'));

    this.dialogService.showConfirmation({
      title: 'Confirmação',
      question: question,
      anwsers: buttons,
    });
  }

  dataConvert() {
    this.dataSource = new MatTableDataSource(this.items);
    this.dataSource.paginator = this.paginator;
  }


  private addItem(item: ItemModel): void {
    this.items.unshift(item);
    this.save()
      .pipe(finalize(() => this.loading = false))
      .subscribe(
        () => {
          this.dataConvert();
          this.dialogService.showNotification(MensagemEnum.ITEM_ADICIONADO);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

}
