import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Observable } from 'rxjs';
import { map, tap, finalize, take, filter } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';

import { TipoCobrancaService } from '../../services/tipo-cobranca.service';
import { TipoCobrancaModel } from '../../models/tipo-cobranca.model';
import {MensagemEnum} from '../../../../../shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-tipo-cobranca-form',
  templateUrl: './tipo-cobranca.form.component.html',
  styleUrls: ['./tipo-cobranca.form.component.scss'],
})
export class TipoCobrancaFormComponent implements OnInit {

  formGroup: FormGroup;
  saving: boolean;

  private id: number;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private tipoCobrancaService: TipoCobrancaService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Cadastro de Tipo de Cobrança');
    this.id = null;

    this.formGroup = this.formBuilder.group({
      descricao: [
    null,
    Validators.compose([
      Validators.maxLength(50),
      Validators.required,
    ])
    ],
      situacaoAtivo: [
    true,
    Validators.compose([
      Validators.maxLength(1),
      Validators.required,
    ])
    ],
    });

    this.route.data.pipe(
      take(1),
      map((data: {item: TipoCobrancaModel}) => data.item),
      filter((item: TipoCobrancaModel) => !!item),
      tap((item: TipoCobrancaModel) => this.editItem(item)),
    ).subscribe();
  }

  editItem(item: TipoCobrancaModel): void {
    this.layoutService.setTitle(`Alteração de Tipo de Cobrança`);
    this.id = item.id;
    this.formGroup.get('descricao').setValue(item.descricao);
    this.formGroup.get('situacaoAtivo').setValue(item.situacaoAtivo);
  }

  save(event: Event): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    this.saving = true;
    this.request().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate(['/rotina/tabela/tipo-cobranca/listar']);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  private request(): Observable<TipoCobrancaModel> {
    if (this.id) {
      return this.tipoCobrancaService.alterar(<TipoCobrancaModel>{
        id: this.id,
        ...this.formGroup.value,
      }, this.id);
    }
    return this.tipoCobrancaService.incluir(this.formGroup.value);
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      '';
  }
}
