import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators, FormGroupDirective } from '@angular/forms';
import { Observable } from 'rxjs';
import { map, tap, finalize, take, filter, debounceTime, switchMap } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';

import { VaraModel } from '../../../../../shared/models/vara.model';
import { VaraService } from 'src/app/shared/services/vara.service';
import { CidadeModel } from 'src/app/shared/models/cidade.model';
import { UfModel } from 'src/app/shared/models/uf.model';
import { UfService } from 'src/app/shared/services/uf.service';
import { CidadeService } from 'src/app/shared/services/cidade.service';
import { ErrorModel } from '@datasus/core';
import { TipoVaraModelService } from 'src/app/shared/services/tipo-vara.service';
import { MatAutocompleteSelectedEvent } from '@angular/material';
import { TipoVaraModel } from 'src/app/shared/models/tipo-vara';


@Component({
  selector: 'scodes-vara-form',
  templateUrl: './vara.form.component.html',
  styleUrls: ['./vara.form.component.scss'],
})
export class VaraFormComponent implements OnInit {

  formGroup: FormGroup;
  isSalvo: boolean;
  listaTipoVara: TipoVaraModel[];
  listaUfs: UfModel[];
  listaCidades: CidadeModel[] = [];
  descricao = '';

  private id: number;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private varaService: VaraService,
    private ufService: UfService,
    private cidadeService: CidadeService,
    private tipoVaraService: TipoVaraModelService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Cadastro de Vara');
    this.id = null;

    this.formGroup = this.formBuilder.group({
      descricao: [null, [Validators.required]],
      idTipoVara: [null, [Validators.required]],
      forum: [null, [Validators.required]],
      telefone: [null],
      cep: [null],
      bairro: [null, [Validators.required]],
      endereco: [null, [Validators.required]],
      estado: [null, [Validators.required]],
      cidade: [null, [Validators.required]],
    });

    this.route.data.pipe(
      take(1),
      map((data: { item: VaraModel }) => data.item),
      filter((item: VaraModel) => !!item),
      tap((item: VaraModel) => this.editItem(item)),
    ).subscribe();

    this.ufService.obterTodos()
    .subscribe(
      (response) => this.listaUfs = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );

    this.formGroup.get('estado').valueChanges
    .pipe(
      tap(() => this.formGroup.get('cidade').setValue(null)),
      switchMap((uf: UfModel) => this.cidadeService.obterCidadesPorUf(uf.id)),
    ).subscribe(
      (resp: any) => this.listaCidades = resp);

    this.formGroup.get('cidade').valueChanges
    .pipe(debounceTime(350))
    .subscribe(
      (cidade) => {
        if (typeof cidade === 'string') {
          return this.cidadeService.obterCidadesPorNome(cidade, this.formGroup.get('estado').value.id)
          .subscribe(
              (response) => this.listaCidades = response,
              (error: ErrorModel) => this.dialogService.showNotification(error.message));
        }
      });

     this.tipoVaraService.obterDominio().subscribe(
      (response: TipoVaraModel[]) => this.listaTipoVara = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message)
    );


  }

  editItem(item: VaraModel): void {

    this.layoutService.setTitle('Alteração de Vara');
    this.id = item.id;
    this.formGroup.get('idTipoVara').setValue(item.tipoVara.id);
    this.formGroup.get('descricao').setValue(item.descricao);
    this.formGroup.get('forum').setValue(item.forum.toString());
    this.formGroup.get('telefone').setValue(item.telefone);
    this.formGroup.get('cep').setValue(item.cep);
    this.formGroup.get('bairro').setValue(item.bairro);
    this.formGroup.get('endereco').setValue(item.endereco);
    this.formGroup.get('cidade').setValue(item.cidade);
    this.formGroup.get('estado').setValue(item.cidade.uf);
  }


  submitVara(event: Event) {
    event.preventDefault();

    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos obrigatórios');
    }

    this.isSalvo = true;
    this.request().pipe(
      finalize(() => this.isSalvo = false)
      ).subscribe(
        () => {
          this.dialogService.showNotification('Registro salvo com sucesso');
          this.formGroup.reset();
          this.router.navigate(['/rotina/tabela/vara/listar']);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
        );

  }

  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }

  compareUf(option: UfModel, selectValue: UfModel): boolean {
    return !!selectValue && selectValue.id === option.id;
  }

  displayFn(cidadeDominio?: CidadeModel): string | undefined {
    return cidadeDominio ? cidadeDominio.descricao : undefined;
  }

  private request(): Observable<VaraModel> {
    const postData: VaraModel  = {
      descricao: this.formGroup.value.descricao,
      forum: this.formGroup.value.forum,
      telefone: this.formGroup.value.telefone,
      cep: this.formGroup.value.cep,
      bairro: this.formGroup.value.bairro,
      endereco: this.formGroup.value.endereco,
      cidade: {
        id: this.formGroup.value.cidade.id,
      } as CidadeModel,
      tipoVara: {
        id: this.formGroup.value.idTipoVara,
      },
    };

    if (this.id) {
      return this.varaService.alterar(<VaraModel>{
        id: this.id,
        ...postData,
      }, this.id);
    }
    return this.varaService.incluir(postData);
  }

  obterCidades(event: MatAutocompleteSelectedEvent) {
    if (event.option.value && event.option.value.descricao) {
      this.cidadeService
        .obterPorId(event.option.value.id)
        .subscribe(
          (response) => this.formGroup.get('cidade').setValue(response),
          (error: ErrorModel) => this.dialogService.showNotification(error.message));
    }
  }
}
